*DECK CFC
      SUBROUTINE CFC(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
C
C
C-----------------------------------------------------------------------
C
C MODULE TO COMPUTE THE FEEDBACK COEFFICIENTS (CFC) FOR BOTH THE CELL
C AND THE REFLECTOR.
C A CANDU-6 DATABASE IS GENERATED USING THE FEEDBACK MODEL
C PARAMETRIZATION OF THE CROSS-SECTIONS.
C
C-----------------------------------------------------------------------
C
C
C RECOVER AND INTERPOLATE MACROLIB INFORMATION FROM ONE OR MANY
C COMPO LINKED LISTS OR XSM FILES.
C
C-----------------------------------------------------------------------
C
C  INPUT/OUTPUT PARAMETERS:
C  NENTRY : NUMBER OF LINKED LISTS OR FILES USED BY THE MODULE.
C  HENTRY : CHARACTER*12 NAME OF EACH LINKED LIST OR FILE.
C  IENTRY : =1 LINKED LIST; =2 XSM FILE; =3 SEQUENTIAL BINARY FILE;
C           =4 SEQUENTIAL ASCII FILE.
C  JENTRY : =0 THE LINKED LIST OR FILE IS CREATED.
C           =1 THE LINKED LIST OR FILE IS OPEN FOR MODIFICATIONS;
C           =2 THE LINKED LIST OR FILE IS OPEN IN READ-ONLY MODE.
C  KENTRY : =FILE UNIT NUMBER; =LINKED LIST ADDRESS OTHERWISE.
C           DIMENSION HENTRY(NENTRY),IENTRY(NENTRY),JENTRY(NENTRY),
C           KENTRY(NENTRY)
C
C-----------------------------------------------------------------------
C
C  LINKED LIST / XSM FILE:
C  HENTRY(1) : CREATE OR MODIFICATION TYPE(REACTOR_XSDB).
C  HENTRY(2) : READ ONLY TYPE(L_COMPO) FOR REFERECE(NOMINAL).
C  HENTRY(3) : READ ONLY TYPE(L_COMPO) FOR FUEL-UP TEMP.
C  HENTRY(4) : READ ONLY TYPE(L_COMPO) FOR FUEL-DOWN TEMP.
C  HENTRY(5) : READ ONLY TYPE(L_COMPO) FOR COOL-UP TEMP.
C  HENTRY(6) : READ ONLY TYPE(L_COMPO) FOR COOL-DOWN TEMP.
C  HENTRY(7) : READ ONLY TYPE(L_COMPO) FOR MOD-UP TEMP
C  HENTRY(8) : READ ONLY TYPE(L_COMPO) FOR MOD-DOWN TEMP.
C  HENTRY(9) : READ ONLY TYPE(L_COMPO) FOR COOL-UP DEN.
C  HENTRY(10) : READ ONLY TYPE(L_COMPO) FOR COOL-DOWN DEN.
C  HENTRY(11) : READ ONLY TYPE(L_COMPO) FOR MOD-UP DEN.
C  HENTRY(12) : READ ONLY TYPE(L_COMPO) FOR MOD-DOWN DEN.
C  HENTRY(13) : READ ONLY TYPE(L_COMPO) FOR BORON.
C  HENTRY(14) : READ ONLY TYPE(L_COMPO) FOR PURITY.
C  HENTRY(15) : READ ONLY TYPE(L_COMPO) FOR XE135.
C  HENTRY(16) : READ ONLY TYPE(L_COMPO) FOR SM149.
C  HENTRY(17) : READ ONLY TYPE(L_COMPO) FOR NP239.
C  HENTRY(18) : READ ONLY TYPE(L_COMPO) FOR (T-FUEL,DEN).
C  HENTRY(19) : READ ONLY TYPE(L_COMPO) FOR (T-COOL,DEN).
C  HENTRY(20) : READ ONLY TYPE(L_COMPO) FOR POWER-UP.
C  HENTRY(21) : READ ONLY TYPE(L_COMPO) FOR POWER-IN.
C  HENTRY(22) : READ ONLY TYPE(L_COMPO) FOR POWER-D.
C  HENTRY(23) : READ ONLY TYPE(L_COMPO) FOR REFLEC. REF.
C  HENTRY(24) : READ ONLY TYPE(L_COMPO) FOR REFLEC. MOD-UP TEMP.
C  HENTRY(25) : READ ONLY TYPE(L_COMPO) FOR REFLEC. MOD-DW TEMP.
C  HENTRY(26) : READ ONLY TYPE(L_COMPO) FOR REFLEC. MOD-UP DEN.
C  HENTRY(27) : READ ONLY TYPE(L_COMPO) FOR REFLEC. MOD-DW DEN.
C  HENTRY(28) : READ ONLY TYPE(L_COMPO) FOR REFLEC. BORON.
C  HENTRY(29) : READ ONLY TYPE(L_COMPO) FOR REFLEC. PURITY.
C
C-----------------------------------------------------------------------
C--------------------------- AUTHOR: M.T. SISSAOUI ; 08/02/96 ----------
C-----------------------------------------------------------------------
C
      INTEGER       IOUT,NSTATE
      CHARACTER     NAMSBR*6
      PARAMETER    (IOUT=6,NSTATE=20,NAMSBR='CFC   ')
      CHARACTER     HENTRY(NENTRY)*12,TEXT12*12,HSMG*131,NAMLCM*12,
     >              TINFO*72,NAMMY*12,HSIGN*12,TEXTR*9,TEXT*5,CTITRE*72
C
      LOGICAL LCM
C
      INTEGER       IENTRY(NENTRY),JENTRY(NENTRY),KENTRY(NENTRY),
     >              ISTATE(NSTATE),KCHAR(18),IBASE(1)
C
      COMMON        BASE(1)
      EQUIVALENCE  (BASE(1),IBASE(1))
C
C-----
C     PARAMETER VALIDATION.
C-----
C
C     DATABSE
C
      IF(NENTRY.LE.28) CALL XABORT('CFC: 29 PARAMETER EXPECTED.')
      IF((IENTRY(1).NE.1).AND.(IENTRY(1).NE.2)) CALL XABORT('CFC:'
     1 //'  MACROLIB LINKED LIST OR XSM FILE EXPECTED AT LHS.')
      IF((JENTRY(1).NE.0).AND.(JENTRY(1).NE.1)) CALL XABORT('CFC: '
     1 //' MACROLIB IN CREATE OR MODIFICATION MODE EXPECTED.')
C
C     INDIVIDUAL LOCAL PARAMETER.
C
      IF((IENTRY(2).NE.1).AND.(IENTRY(2).NE.2)) CALL XABORT('CFC:'
     1 //' COMPO LINKED LIST OR XSM FILE EXPECTED AT RHS.')
      IF(JENTRY(2).NE.2) CALL XABORT('CFC: COMPO IN READ-ONLY '
     1 //'MODE EXPECTED AT RHS.')
      IF((IENTRY(3).NE.1).AND.(IENTRY(3).NE.2)) CALL XABORT('CFC: '
     1 //'COMPO LINKED LIST OR XSM FILE EXPECTED AT RHS.')
      IF(JENTRY(3).NE.2) CALL XABORT('CFC: COMPO IN READ-ONLY '
     1 //' MODE EXPECTED AT RHS.')
      IF((IENTRY(4).NE.1).AND.(IENTRY(4).NE.2)) CALL XABORT('CFC: '
     1 //'COMPO LINKED LIST OR XSM FILE EXPECTED AT RHS.')
      IF(JENTRY(4).NE.2) CALL XABORT('CFC: COMPO IN READ-ONLY '
     1 //'MODE EXPECTED AT RHS.')
      IF((IENTRY(5).NE.1).AND.(IENTRY(5).NE.2)) CALL XABORT('CFC: '
     1 //'COMPO LINKED LIST OR XSM FILE EXPECTED AT RHS.')
      IF(JENTRY(5).NE.2) CALL XABORT('CFC: COMPO IN READ-ONLY '
     1 //'MODE EXPECTED AT RHS.')
      IF((IENTRY(6).NE.1).AND.(IENTRY(6).NE.2)) CALL XABORT('CFC: '
     1 //'COMPO LINKED LIST OR XSM FILE EXPECTED AT RHS.')
      IF(JENTRY(6).NE.2) CALL XABORT('CFC: COMPO IN READ-ONLY '
     1 //'MODE EXPECTED AT RHS.')
      IF((IENTRY(7).NE.1).AND.(IENTRY(7).NE.2)) CALL XABORT('CFC: '
     1 //'COMPO LINKED LIST OR XSM FILE EXPECTED AT RHS.')
      IF(JENTRY(7).NE.2) CALL XABORT('CFC: COMPO IN READ-ONLY'
     1 //' MODE EXPECTED AT RHS.')
      IF((IENTRY(8).NE.1).AND.(IENTRY(8).NE.2)) CALL XABORT('CFC: '
     1 //'COMPO LINKED LIST OR XSM FILE EXPECTED AT RHS.')
      IF(JENTRY(8).NE.2) CALL XABORT('CFC: COMPO IN READ-ONLY '
     1 //'MODE EXPECTED AT RHS.')
      IF((IENTRY(9).NE.1).AND.(IENTRY(9).NE.2)) CALL XABORT('CFC: '
     1 //'COMPO LINKED LIST OR XSM FILE EXPECTED AT RHS.')
      IF(JENTRY(9).NE.2) CALL XABORT('CFC: COMPO IN READ-ONLY '
     1 //'MODE EXPECTED AT RHS.')
      IF((IENTRY(10).NE.1).AND.(IENTRY(10).NE.2)) CALL XABORT('CFC: '
     1 //'COMPO LINKED LIST OR XSM FILE EXPECTED AT RHS.')
      IF(JENTRY(10).NE.2) CALL XABORT('CFC: COMPO IN READ-ONLY '
     1 //'MODE EXPECTED AT RHS.')
      IF((IENTRY(11).NE.1).AND.(IENTRY(11).NE.2)) CALL XABORT('CFC: '
     1 //'COMPO LINKED LIST OR XSM FILE EXPECTED AT RHS.')
      IF(JENTRY(11).NE.2) CALL XABORT('CFC: COMPO IN READ-ONLY '
     1 //'MODE EXPECTED AT RHS.')
      IF((IENTRY(12).NE.1).AND.(IENTRY(12).NE.2)) CALL XABORT('CFC: '
     1 //'COMPO LINKED LIST OR XSM FILE EXPECTED AT RHS.')
      IF(JENTRY(12).NE.2) CALL XABORT('CFC: COMPO IN READ-ONLY'
     1 //' MODE EXPECTED AT RHS.')
      IF((IENTRY(13).NE.1).AND.(IENTRY(13).NE.2)) CALL XABORT('CFC: '
     1 //'COMPO LINKED LIST OR XSM FILE EXPECTED AT RHS.')
      IF(JENTRY(13).NE.2) CALL XABORT('CFC: COMPO IN READ-ONLY '
     1 //'MODE EXPECTED AT RHS.')
      IF((IENTRY(14).NE.1).AND.(IENTRY(14).NE.2)) CALL XABORT('CFC: '
     1 //'COMPO LINKED LIST OR XSM FILE EXPECTED AT RHS.')
      IF(JENTRY(14).NE.2) CALL XABORT('CFC: COMPO IN READ-ONLY '
     1 //'MODE EXPECTED AT RHS.')
C
C     STURATING ISOTOPES
C
      IF((IENTRY(15).NE.1).AND.(IENTRY(15).NE.2)) CALL XABORT('CFC:'
     1 //' COMPO LINKED LIST OR XSM FILE EXPECTED AT RHS.')
      IF(JENTRY(15).NE.2) CALL XABORT('CFC: COMPO IN READ-ONLY '
     1 //'MODE EXPECTED AT RHS.')
      IF((IENTRY(16).NE.1).AND.(IENTRY(16).NE.2)) CALL XABORT('CFC: '
     1 //'COMPO LINKED LIST OR XSM FILE EXPECTED AT RHS.')
      IF(JENTRY(16).NE.2) CALL XABORT('CFC: COMPO IN READ-ONLY '
     1 //' MODE EXPECTED AT RHS.')
C
C      MIXED PARAMETERS
C
      IF((IENTRY(17).NE.1).AND.(IENTRY(17).NE.2)) CALL XABORT('CFC: '
     1 //'COMPO LINKED LIST OR XSM FILE EXPECTED AT RHS.')
      IF(JENTRY(17).NE.2) CALL XABORT('CFC: COMPO IN READ-ONLY '
     1 //' MODE EXPECTED AT RHS.')
      IF((IENTRY(18).NE.1).AND.(IENTRY(18).NE.2)) CALL XABORT('CFC: '
     1 //'COMPO LINKED LIST OR XSM FILE EXPECTED AT RHS.')
      IF(JENTRY(18).NE.2) CALL XABORT('CFC: COMPO IN READ-ONLY '
     1 //' MODE EXPECTED AT RHS.')
      IF((IENTRY(19).NE.1).AND.(IENTRY(19).NE.2)) CALL XABORT('CFC: '
     1 //'COMPO LINKED LIST OR XSM FILE EXPECTED AT RHS.')
      IF(JENTRY(19).NE.2) CALL XABORT('CFC: COMPO IN READ-ONLY '
     1 //' MODE EXPECTED AT RHS.')
C
C     HISTORY PARAMETERS (FISSILE ISOTOPES)
C
      IF((IENTRY(20).NE.1).AND.(IENTRY(20).NE.2)) CALL XABORT('CFC: '
     1 //'COMPO LINKED LIST OR XSM FILE EXPECTED AT RHS.')
      IF(JENTRY(20).NE.2) CALL XABORT('CFC: COMPO IN READ-ONLY '
     1 //' MODE EXPECTED AT RHS.')
      IF((IENTRY(21).NE.1).AND.(IENTRY(21).NE.2)) CALL XABORT('CFC: '
     1 //'COMPO LINKED LIST OR XSM FILE EXPECTED AT RHS.')
      IF(JENTRY(21).NE.2) CALL XABORT('CFC: COMPO IN READ-ONLY '
     1 //' MODE EXPECTED AT RHS.')
      IF((IENTRY(22).NE.1).AND.(IENTRY(22).NE.2)) CALL XABORT('CFC: '
     1 //'COMPO LINKED LIST OR XSM FILE EXPECTED AT RHS.')
      IF(JENTRY(22).NE.2) CALL XABORT('CFC: COMPO IN READ-ONLY '
     1 //' MODE EXPECTED AT RHS.')
C
C     MODERATOR PROPERTIES
C
      IF((IENTRY(23).NE.1).AND.(IENTRY(23).NE.2)) CALL XABORT('CFC: '
     1 //'COMPO LINKED LIST OR XSM FILE EXPECTED AT RHS.')
      IF(JENTRY(23).NE.2) CALL XABORT('CFC: COMPO IN READ-ONLY '
     1 //' MODE EXPECTED AT RHS.')
      IF((IENTRY(24).NE.1).AND.(IENTRY(24).NE.2)) CALL XABORT('CFC: '
     1 //'COMPO LINKED LIST OR XSM FILE EXPECTED AT RHS.')
      IF(JENTRY(24).NE.2) CALL XABORT('CFC: COMPO IN READ-ONLY '
     1 //' MODE EXPECTED AT RHS.')
      IF((IENTRY(25).NE.1).AND.(IENTRY(25).NE.2)) CALL XABORT('CFC: '
     1 //'COMPO LINKED LIST OR XSM FILE EXPECTED AT RHS.')
      IF(JENTRY(25).NE.2) CALL XABORT('CFC: COMPO IN READ-ONLY '
     1 //' MODE EXPECTED AT RHS.')
      IF((IENTRY(26).NE.1).AND.(IENTRY(26).NE.2)) CALL XABORT('CFC: '
     1 //'COMPO LINKED LIST OR XSM FILE EXPECTED AT RHS.')
      IF(JENTRY(26).NE.2) CALL XABORT('CFC: COMPO IN READ-ONLY '
     1 //' MODE EXPECTED AT RHS.')
       IF((IENTRY(27).NE.1).AND.(IENTRY(27).NE.2)) CALL XABORT('CFC: '
     1 //'COMPO LINKED LIST OR XSM FILE EXPECTED AT RHS.')
      IF(JENTRY(27).NE.2) CALL XABORT('CFC: COMPO IN READ-ONLY '
     1 //' MODE EXPECTED AT RHS.')
      IF((IENTRY(28).NE.1).AND.(IENTRY(28).NE.2)) CALL XABORT('CFC: '
     1 //'COMPO LINKED LIST OR XSM FILE EXPECTED AT RHS.')
      IF(JENTRY(28).NE.2) CALL XABORT('CFC: COMPO IN READ-ONLY '
     1 //' MODE EXPECTED AT RHS.')
      IF((IENTRY(29).NE.1).AND.(IENTRY(29).NE.2)) CALL XABORT('CFC: '
     1 //'COMPO LINKED LIST OR XSM FILE EXPECTED AT RHS.')
      IF(JENTRY(29).NE.2) CALL XABORT('CFC: COMPO IN READ-ONLY '
     1 //' MODE EXPECTED AT RHS.')
C-----
C     END OF L_COMPO FILES REQUIRED
C-----
C
      ITYPE=JENTRY(1)
      IPLIST=KENTRY(1)
      IMPX=1
C-----
C     CHECK THE SIGNTURE OF THE LINKED LIST OR XSM FILE
C-----
      DO 200 I=2,29
      CALL LCMGET(KENTRY(I),'SIGNATURE',KCHAR)
      WRITE(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
      IF(HSIGN.NE.'L_COMPO') THEN
         TEXT12=HENTRY(I)
         CALL XABORT('CFC: SIGNATURE OF '//TEXT12//' IS '//HSIGN//
     1   '. L_COMPO EXPECTED.')
      ENDIF
 200  CONTINUE
C-----
C     READ THE DATABASE INFORMATION TITLE
C-----
         CALL REDGET (INDIC,NITMA,FLOTT,TEXT,DFLOTT)
         IF(INDIC.NE.3) CALL XABORT('CFC: CHARACTER DATA EXPECTED.')
          IF(TEXT.EQ.'INFOR') THEN
          CALL REDGET(INDIC,NITMA,FLOTT,TINFO,DFLOTT)
          IF(INDIC.NE.3) CALL XABORT('CFC: CHARACTER DATA EXPECTED.')
          ELSE
          CALL XABORT('CFC: INFOR KEY WORD EXPECTED.')
          ENDIF
C-----
C     CREATED OR MODIFIED LINKED LIST TYPE(L_COMPO)
C-----
         IF(JENTRY(1).EQ.0) THEN
         HSIGN='REACTOR_XSDB'
         READ(HSIGN,'(3A4)') (KCHAR(I),I=1,3)
         CALL LCMPUT(KENTRY(1),'SIGNATURE',3,3,KCHAR)
C
C        STORE THE INFORMATION TITLE
C
         READ(TINFO,'(18A4)') (KCHAR(I),I=1,18)
         CALL LCMPUT(KENTRY(1),'INFORMATION',18,3,KCHAR)
         ELSE
         CALL LCMGET(KENTRY(1),'SIGNATURE',KCHAR)
         WRITE(HSIGN,'(3A4)') (KCHAR(I),I=1,3)
            IF(HSIGN.NE.'REACTOR_XSDB') THEN
            TEXT12=HENTRY(1)
            CALL XABORT('CFC: SIGNATURE OF '//TEXT12//' IS '//HSIGN//
     1      '. REACTOR_XSDB EXPECTED.')
            ENDIF
C
C        RECOVER THE INFORMATION TITLE
C
         CALL LCMGET(KENTRY(1),'INFORMATION',KCHAR)
         WRITE(CTITRE,'(18A4)') (KCHAR(I),I=1,18)
            IF(TINFO.NE.CTITRE) THEN
            CALL XABORT('CFC: '//CTITRE//' INFOR TITLE EXPECTED.')
            ENDIF
      ENDIF
C-----
C     RECOVER SOME INFORMATIONS FROM THE FIRST COMPO (NOMINAL).
C-----
      TEXT12='SIGNATURE'
      CALL LCMNXT(KENTRY(2),TEXT12,NAMLCM,NAMMY,LCM)
      IF(TEXT12.EQ.'SIGNATURE') CALL XABORT('CRE: INVALID INPUT COMPO.')
      CALL LCMSIX(KENTRY(2),TEXT12,1)
C
C     RECOVER THE CPO TITLE.
C
      CALL LCMLEN(KENTRY(2),'TITLE',LENGT,ITYLCM)
      IF(LENGT.GT.0) THEN
         CALL LCMGET(KENTRY(2),'TITLE',KCHAR)
         WRITE(CTITRE,'(18A4)') (KCHAR(I),I=1,18)
      ELSE
         CTITRE='*** NO TITLE PROVIDED ***'
      ENDIF
      CALL LCMSIX(KENTRY(2),' ',2)
C
C     READ PARAMETERS
C
      CALL XDISET(ISTATE,NSTATE,0)
      CALL LCMGET(KENTRY(2),'STATE-VECTOR',ISTATE)
      IF(ISTATE(1).GT.1) THEN
      WRITE(HSMG,'(28HNUMBER OF MIXUTRE SAVED IS =,I5,
     1 28H ONLY ONE REGION IS REQUIRED)') ISTATE(1)
      CALL XABORT(HSMG)
      ENDIF
      NGRP=ISTATE(2)
      NISO=ISTATE(3)
      NL=ISTATE(4)
      NBURN=ISTATE(5)
      NXS=20+NL
C-----
C     CHECK  OTHERS CPO INFORMATIONS.
C-----
      DO 100 I=3,29
      TEXT12='SIGNATURE'
      CALL LCMNXT(KENTRY(I),TEXT12,NAMLCM,NAMMY,LCM)
      IF(TEXT12.EQ.'SIGNATURE') CALL XABORT('CFC: INVALID '
     1 //'INPUT COMPO.')
C      CALL LCMSIX(KENTRY(I),TEXT12,1)
      CALL XDISET(ISTATE,NSTATE,0)
      CALL LCMGET(KENTRY(I),'STATE-VECTOR',ISTATE)
C      CALL LCMSIX(KENTRY(I),' ',2)
C
C     ENERGY GROUP AND ORDER OF SCATTERING ANISOTROPY.
C
       IF(ISTATE(2).NE.NGRP) THEN
       WRITE(HSMG,'(7HNGRP = ,I5,13H IN REF NGRP=,I5)')
     1 ISTATE(2),NGRP
       CALL XABORT('CFC:  INCONSISTENT NB OF GROUPS '
     1 //TEXT12//' IS '//HSMG//' ')
       ELSE IF(ISTATE(4).LT.NL) THEN
       WRITE(HSMG,'(5HNL = ,I5,11H IN REF NL=,I5)')
     1 ISTATE(4),NL
       CALL XABORT('CFC:  INCONSISTENT NB OF LEGENDRE ORDERS.'
     1 //TEXT12//'IS '//HSMG//' ')
       ENDIF
 100   CONTINUE
C-----
C     READ THE RECORD NAME FOR THE DATABASE
C-----
         CALL REDGET (INDIC,NITMA,FLOTT,TEXT,DFLOTT)
         IF(INDIC.NE.3) CALL XABORT('CFC: CHARACTER DATA EXPECTED.')
         IF(TEXT.EQ.'DNAME') THEN
         CALL REDGET(INDIC,NITMA,FLOTT,TEXTR,DFLOTT)
         IF(INDIC.NE.3) CALL XABORT('CFC: CHARACTER DATA EXPECTED.')
         ENDIF
         CALL REDGET(INDIC,NITMA,FLOTT,TEXT,DFLOTT)
         IF(TEXT.NE.';') CALL XABORT('CFC: '//TEXT//' IS AN INVALID '
     1 //'KEY WORD.')
C-----
C     CFC DRIVER.
C-----
         CALL SETARA(BASE,NBURN*NGRP,ITOTAL)
         CALL SETARA(BASE,NBURN*NGRP,IZNUG)
         CALL SETARA(BASE,NBURN*NGRP,IDIFFX)
         CALL SETARA(BASE,NBURN*NGRP,IDIFFY)
         CALL SETARA(BASE,NBURN*NGRP,IDIFFZ)
         CALL SETARA(BASE,NBURN*NGRP,IH)
         CALL SETARA(BASE,NL*NGRP,IIJJ)
         CALL SETARA(BASE,NL*NGRP,INJJ)
         CALL SETARA(BASE,NL*NGRP,IMIJ)
         CALL SETARA(BASE,NL*NGRP,IMNJ)
         CALL SETARA(BASE,NBURN*NL*NGRP*NGRP,ISCAT)
         CALL SETARA(BASE,NGRP*(NGRP+1),IWORK3)
         CALL SETARA(BASE,NBURN,IIPOS)
         CALL SETARA(IBASE,3*NISO,IHISO)
         CALL SETARA(BASE,NISO,IDENSI)
         CALL SETARA(BASE,NISO,IEFJ)
         CALL SETARA(BASE,NBURN*NISO*NGRP,IMREFT)
         CALL SETARA(BASE,NBURN*NISO*NGRP*NGRP*NL,IMREFS)
         CALL SETARA(BASE,NBURN*NISO*NGRP,IMRFDX)
         CALL SETARA(BASE,NBURN*NISO*NGRP,IMRFDY)
         CALL SETARA(BASE,NBURN*NISO*NGRP,IMRFDZ)
         CALL SETARA(BASE,NBURN*NISO,IXCREF)
         CALL SETARA(BASE,NISO,IJTAB)
         CALL SETARA(BASE,NBURN*NISO*NGRP,IMREFF)
         CALL SETARA(BASE,NBURN*NISO*NGRP,IMREFH)
         CALL SETARA(BASE,2*NBURN,IDELTA)
         CALL SETARA(BASE,NXS,IIXS)
C
         CALL CFCDRV(KENTRY,NBURN,NGRP,NISO,NL,IMPX,BASE(ITOTAL),
     1   BASE(IZNUG),BASE(IDIFFX),BASE(IDIFFY),BASE(IDIFFZ),
     1   BASE(IH),BASE(IIJJ),BASE(INJJ),BASE(IMIJ),BASE(IMNJ),
     1   BASE(ISCAT),
     1   BASE(IWORK3),BASE(IIPOS),IBASE(IHISO),BASE(IDENSI),BASE(IEFJ),
     1   BASE(IMREFT),BASE(IMREFS),BASE(IMRFDX),
     1   BASE(IMRFDY),BASE(IMRFDZ),CTITRE,BASE(IXCREF),
     1   BASE(IJTAB),TEXTR,BASE(IMREFF),BASE(IMREFH),BASE(IDELTA),
     1   BASE(IIXS),NXS)
C
         CALL RLSARA(BASE(IIXS))
         CALL RLSARA(BASE(IDELTA))
         CALL RLSARA(BASE(IMREFH))
         CALL RLSARA(BASE(IMREFF))
         CALL RLSARA(BASE(IJTAB))
         CALL RLSARA(BASE(IXCREF))
         CALL RLSARA(BASE(IMRFDZ))
         CALL RLSARA(BASE(IMRFDY))
         CALL RLSARA(BASE(IMRFDX))
         CALL RLSARA(BASE(IMREFS))
         CALL RLSARA(BASE(IMREFT))
         CALL RLSARA(BASE(IEFJ))
         CALL RLSARA(BASE(IDENSI))
         CALL RLSARA(IBASE(IHISO))
         CALL RLSARA(BASE(IIPOS))
         CALL RLSARA(BASE(IWORK3))
         CALL RLSARA(BASE(ISCAT))
         CALL RLSARA(BASE(IMNJ))
         CALL RLSARA(BASE(IMIJ))
         CALL RLSARA(BASE(INJJ))
         CALL RLSARA(BASE(IIJJ))
         CALL RLSARA(BASE(IH))
         CALL RLSARA(BASE(IDIFFZ))
         CALL RLSARA(BASE(IDIFFY))
         CALL RLSARA(BASE(IDIFFX))
         CALL RLSARA(BASE(IZNUG))
         CALL RLSARA(BASE(ITOTAL))
C
      RETURN
      END
