*DECK EVO
      SUBROUTINE EVO(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
C
C-----------------------------------------------------------------------
C
C DRIVER FOR AN ISOTOPIC DEPLETION CALCULATION.
C
C INPUT/OUTPUT PARAMETERS:
C  NENTRY : NUMBER OF LINKED LISTS OR FILES USED BY THE MODULE.
C  HENTRY : CHARACTER*12 NAME OF EACH LINKED LIST OR FILE.
C  IENTRY : =1 LINKED LIST; =2 XSM FILE; =3 SEQUENTIAL BINARY FILE;
C           =4 SEQUENTIAL ASCII FILE.
C  JENTRY : =0 THE LINKED LIST OR FILE IS CREATED.
C           =1 THE LINKED LIST OR FILE IS OPEN FOR MODIFICATIONS;
C           =2 THE LINKED LIST OR FILE IS OPEN IN READ-ONLY MODE.
C  KENTRY : =FILE UNIT NUMBER; =LINKED LIST ADDRESS OTHERWISE.
C           DIMENSION HENTRY(NENTRY),IENTRY(NENTRY),JENTRY(NENTRY),
C           KENTRY(NENTRY)
C
C LINKED LIST / XSM FILE: (ORDER IS ARBITRARY FOR 2,3,4,5)
C  HENTRY(1) : CREATION OR MODIFICATION TYPE(L_BURNUP).
C  HENTRY(2) : CREATION OR MODIFICATION TYPE(L_LIBRARY).
C  HENTRY(3) : OPTIONAL READ-ONLY TYPE(L_LIBRARY).
C  HENTRY(4) : READ-ONLY TYPE(L_TRACK).
C  HENTRY(5) : OPTIONAL READ-ONLY TYPE(L_FLUX).
C
C-------------------------------------- AUTHOR: A. HEBERT ; 09/11/94 ---
C-------------------------------------- MODIF : G.M.      ; 97-02-12 ---
C
      IMPLICIT    NONE
      INTEGER     IOUT,NSTATE,NDPROC,NTC,ILCMUP,ILCMDN
      CHARACTER   NAMSBR*6
      PARAMETER  (IOUT=6,NSTATE=20,NDPROC=20,NTC=3,ILCMUP=1,
     >            ILCMDN=2,NAMSBR='EVO   ')
C----
C  ROUTINE PARAMETERS
C----
      INTEGER     NENTRY
      CHARACTER   HENTRY(NENTRY)*12
      INTEGER     IENTRY(NENTRY),JENTRY(NENTRY),KENTRY(NENTRY)
C----
C  MEMORY ALLOCATION
C----
      INTEGER     IBASE(1)
      REAL        RBASE
      COMMON      RBASE(1)
      EQUIVALENCE(RBASE(1),IBASE(1))
      INTEGER     IMAT,IVOL,IIDL,
     >            JJMIX,JJDEN,ISONAM,ISONRF,ISOAWR,JFLUXE,IFUNKN,
     >            ITIMES,ISONDP,LMASKM,LMASKG,IJM,IMILVO,IYDPL,
     >            IFLX,IVX,ISOCMB
C----
C  LOCAL PARAMETERS
C----
      CHARACTER   HSMG*131,HSIGN*12,TEXT12*12
      INTEGER     ISTATE(NSTATE),KCHAR(NTC),IPDEPL,ITC,NPRODE,
     >            IPFLUX,IKFLUX,IPTRK,IKTRK,IPLIB,IKLIB,IEN,
     >            IPLIBO,IKLIBO,NREG,NUN,NGMIX,MAXMIX,NBISO,
     >            NGROUP,NL,NDEPL,NCOMB,NPROC,IGR,KEYFLX,IPRINT,
     >            ISAVE,ISET,ITIXS,INR,INDREC,ITYPE,
     >            NREGCE,IGLOB,IEXTR,NTIME,IREG
      REAL        RPAR(5),XT(5)
C----
C  PARAMETER VALIDATION.
C----
      IF(NENTRY .LT. 4) CALL XABORT(NAMSBR//
     >': AT LEAST FOUR DATA STRUCTURES EXPECTED.')
      IF(IENTRY(1) .NE. 1 .AND. IENTRY(1) .NE. 2) CALL XABORT(NAMSBR//
     >': LINKED LIST OR XSM FILE EXPECTED AT LHS.')
C----
C  CHECK IF FIRST ENTRY IS BURNUP OR A NEW FILE
C----
      IPDEPL=KENTRY(1)
      IF(JENTRY(1) .EQ. 0) THEN
        HSIGN='L_BURNUP'
        READ(HSIGN,'(3A4)') (KCHAR(ITC),ITC=1,NTC)
        CALL LCMPUT(IPDEPL,'SIGNATURE',3,3,KCHAR)
      ELSE IF(JENTRY(1) .EQ. 1) THEN
        CALL LCMGET(IPDEPL,'SIGNATURE',KCHAR)
        WRITE(HSIGN,'(3A4)') (KCHAR(ITC),ITC=1,NTC)
        IF(HSIGN.NE.'L_BURNUP') THEN
          TEXT12=HENTRY(1)
          CALL XABORT(NAMSBR//': SIGNATURE OF '//TEXT12//
     >    ' IS '//HSIGN//' L_BURNUP EXPECTED')
        ENDIF
      ELSE
        CALL XABORT(NAMSBR//
     >  ': BURNUP DATA STRUCTURE NOT CREATE OR UPDATE MODE')
      ENDIF
C----
C  SCAN ENTRY(2) TO ENTRY(4) FOR FLUX, TRACK AND LIB
C----
      IPFLUX=0
      IKFLUX=0
      IPTRK=0
      IKTRK=0
      IPLIB=0
      IKLIB=0
      DO 100 IEN=2,4
        IF(IENTRY(IEN) .EQ. 1 .OR. IENTRY(IEN) .EQ. 2) THEN
          CALL LCMGET(KENTRY(IEN),'SIGNATURE',KCHAR)
          WRITE(HSIGN,'(3A4)') (KCHAR(ITC),ITC=1,NTC)
          IF(HSIGN .EQ. 'L_FLUX' .AND. IKFLUX .EQ. 0) THEN
            IPFLUX=KENTRY(IEN)
            IKFLUX=IEN
          ELSE IF(HSIGN .EQ. 'L_TRACK' .AND. IKTRK .EQ. 0) THEN
            IPTRK=KENTRY(IEN)
            IKTRK=IEN
          ELSE IF(HSIGN .EQ. 'L_LIBRARY' .AND. IKLIB .EQ. 0) THEN
            IPLIB=KENTRY(IEN)
            IKLIB=IEN
          ENDIF
        ELSE
          CALL XABORT('EVO: FIRST FOUR DATA STRUCTURE MUST BE '//
     >                'LINKED_LIST OR XSM_FILE.')
        ENDIF
 100  CONTINUE
      IF(IKTRK .EQ. 0) CALL XABORT(NAMSBR//
     >': NO TRACKING DATA STRUCTURE FOUND')
      IF(IKLIB .EQ. 0) CALL XABORT(NAMSBR//
     >': NO LIBRARY DATA STRUCTURE FOUND')
      IF(IKFLUX .NE. 0 ) THEN
        IF(JENTRY(IKFLUX) .NE. 2) CALL XABORT(NAMSBR//
     >  ': INVALID MODE FOR FLUX DATA STRUCTURE ')
      ENDIF
C----
C  FOR A NEW LIB FIND OLD LIB IN READ ONLY MODE
C----
      IF(JENTRY(IKLIB) .EQ. 0) THEN
        IPLIBO=0
        IKLIBO=0
        DO 110 IEN=IKLIB+1,5
          IF(IENTRY(IEN) .EQ. 1 .OR. IENTRY(IEN) .EQ. 2) THEN
            CALL LCMGET(KENTRY(IEN),'SIGNATURE',KCHAR)
            WRITE(HSIGN,'(3A4)') (KCHAR(ITC),ITC=1,NTC)
            IF(HSIGN .EQ. 'L_LIBRARY' .AND. IKLIBO .EQ. 0) THEN
              IPLIBO=KENTRY(IEN)
              IKLIBO=IEN
              GO TO 115
            ENDIF
          ENDIF
 110    CONTINUE
        CALL XABORT(NAMSBR//
     >  ': NO LIBRARY STRUCTURE FOUND.')
 115    CONTINUE
        IF(JENTRY(IKLIBO) .NE. 2) CALL XABORT(NAMSBR//
     <  ': LIBRARY DATA STRUCTURE IN READ-ONLY MODE EXPECTED')
        CALL LCMEQU(KENTRY(IKLIBO),KENTRY(IKLIB))
      ELSE IF(JENTRY(IKLIB) .EQ. 2) THEN
        CALL XABORT(NAMSBR//
     >  ': LIBRARY DATA STRUCTURE NOT IN CREATE OR UPDATE MODE')
      ENDIF
      IF(JENTRY(IKTRK) .NE. 2) CALL XABORT(NAMSBR//
     >': TRACKING DATA STRUCTURE NOT IN READ-ONLY MODE')
C----
C  RECOVER TRACKING INFORMATION
C----
      CALL XDISET(ISTATE,NSTATE,0)
      CALL LCMGET(IPTRK,'STATE-VECTOR',ISTATE)
      NREG=ISTATE(1)
      NUN=ISTATE(2)
      NGMIX=ISTATE(4)
      CALL SETARA(IBASE,NREG,IMAT)
      CALL SETARA(RBASE,NREG,IVOL)
      CALL SETARA(IBASE,NREG,IIDL)
      CALL LCMGET(IPTRK,'MATCOD',IBASE(IMAT))
      CALL LCMGET(IPTRK,'VOLUME',RBASE(IVOL))
      CALL LCMGET(IPTRK,'KEYFLX',IBASE(IIDL))
C----
C  RECOVER GENERAL INTERNAL LIBRARY INFORMATION.
C----
      CALL XDISET(ISTATE,NSTATE,0)
      CALL LCMGET(IPLIB,'STATE-VECTOR',ISTATE)
      MAXMIX=ISTATE(1)
      NBISO=ISTATE(2)
      NGROUP=ISTATE(3)
      NL=ISTATE(4)
      NDEPL=ISTATE(11)
      NCOMB=ISTATE(12)
      NPROC=NDPROC+NL
      NPRODE=ISTATE(18)
      IF(NGMIX .GT. MAXMIX) THEN
         WRITE(HSMG,9000) NAMSBR,NGMIX,MAXMIX
         CALL XABORT(HSMG)
      ENDIF
      CALL SETARA(IBASE,NBISO,JJMIX)
      CALL SETARA(RBASE,NBISO,JJDEN)
      CALL SETARA(IBASE,3*NBISO,ISONAM)
      CALL SETARA(IBASE,3*NBISO,ISONRF)
      CALL SETARA(RBASE,NBISO,ISOAWR)
      CALL LCMGET(IPLIB,'ISOTOPESMIX ',IBASE(JJMIX))
      CALL LCMGET(IPLIB,'ISOTOPESDENS',RBASE(JJDEN))
      CALL LCMGET(IPLIB,'ISOTOPESUSED',IBASE(ISONAM))
      CALL LCMGET(IPLIB,'ISOTOPERNAME',IBASE(ISONRF))
      CALL LCMGET(IPLIB,'ISOTOPESAWR ',RBASE(ISOAWR))
C----
C  RECOVER AVERAGE NEUTRON FLUXES INFORMATION.
C----
      CALL SETARA(RBASE,NREG*NGROUP,JFLUXE)
      IF(IKFLUX .NE. 0) THEN
        CALL SETARA(RBASE,NUN,IFUNKN)
        CALL LCMSIX(IPFLUX,'FLUXDIRECT  ',ILCMUP)
        DO 120 IGR=1,NGROUP
          WRITE(TEXT12,'(4HFLUX,I3)') IGR
          CALL LCMGET(IPFLUX,TEXT12,RBASE(IFUNKN))
          DO 121 IREG=1,NREG
            KEYFLX=IBASE(IIDL+IREG-1)
            RBASE(JFLUXE+(IGR-1)*NREG+IREG-1)=RBASE(IFUNKN+KEYFLX-1)
 121      CONTINUE
 120    CONTINUE
        CALL LCMSIX(IPFLUX,'FLUXDIRECT  ',ILCMDN)
        CALL RLSARA(RBASE(IFUNKN))
      ELSE
        CALL XDRSET(RBASE(JFLUXE),NREG*NGROUP,0.0)
      ENDIF
C----
C  READ THE BURNUP PARAMETERS.
C----
      IPRINT=1
      ISAVE=0
      ISET=0
      ITIXS=0
      INR=-1
      XT(1)=0.0
      XT(2)=0.0
      XT(3)=0.0
      XT(4)=0.0
      XT(5)=0.0
      IF(JENTRY(1) .EQ. 0) THEN
        INDREC=1
        RPAR(1)=1.0E-5
        RPAR(2)=1.0E-4
        RPAR(3)=80.0
        RPAR(4)=1.0E-4
        RPAR(5)=0.0
        ITYPE=2
        NTIME=0
      ELSE IF(JENTRY(1) .EQ. 1) THEN
        INDREC=2
        CALL XDISET(ISTATE,NSTATE,0)
        CALL LCMGET(IPDEPL,'STATE-VECTOR',ISTATE)
        IF(ISTATE(1) .LT. 0) THEN
          ITIXS=1
        ENDIF
        NTIME=ISTATE(3)
        ITYPE=ABS(ISTATE(1))
        IF(ISTATE(4) .NE. NBISO) CALL XABORT(NAMSBR//
     >  ': INVALID NUMBER OF ISOTOPES.')
        IF(ISTATE(5) .NE. NGROUP) CALL XABORT(NAMSBR//
     >  ': INVALID NUMBER OF ENERGY GROUPS.')
        IF(ISTATE(6) .NE. NREG) CALL XABORT(NAMSBR//
     >  ': INVALID NUMBER OF REGIONS.')
        CALL LCMGET(IPDEPL,'EVOLUTION-R ',RPAR)
      ENDIF
      CALL SETARA(RBASE,NTIME+2,ITIMES)
      CALL XDRSET(RBASE(ITIMES),NTIME+2,0.0)
      IF(NTIME .GT. 0) THEN
        CALL LCMGET(IPDEPL,'DEPL-TIMES',RBASE(ITIMES))
        XT(4)=RBASE(ITIMES+NTIME-1)
      ENDIF
C----
C  READ THE BURNUP INPUT DATA.
C----
      CALL EVOGET(IPRINT,ITYPE ,ITIXS ,IEXTR ,IGLOB ,INR   ,
     >            RPAR  ,XT    )
C----
C  INPUT DATA READ COMPLETE
C----
      IF(INR .GT. 0) THEN
        IF(IKFLUX .EQ. 0) CALL XABORT(NAMSBR//
     >  ': FOR POWER/FLUX BURNUP A FLUXUNK DATA STRUCTURE REQUIRED.')
      ENDIF
      IF(MAXMIX .EQ. 0) CALL XABORT(NAMSBR//
     >'EVO: MAXMIX NOT YET DEFINED.')
      IF(NREG .EQ. 0) CALL XABORT(NAMSBR//
     >': NREG = 0')
      IF(NDEPL .EQ. 0) CALL XABORT(NAMSBR//
     >': NO DEPLETING ISOTOPES.')
      IF(NCOMB .EQ. 0) CALL XABORT(NAMSBR//
     >': NO DEPLETING MIXTURES.')
      IF(INR .EQ. -1) CALL XABORT(NAMSBR//
     >': TYPE OF DEPLETION NOT DEFINED.')
      NREGCE=NPRODE+NCOMB
      CALL SETARA(IBASE,2*NDEPL,ISONDP)
      CALL SETARA(IBASE,MAXMIX,LMASKM)
      CALL SETARA(IBASE,NGROUP,LMASKG)
      CALL SETARA(IBASE,NDEPL*NREGCE,IJM)
      CALL SETARA(IBASE,NREGCE,IMILVO)
      CALL SETARA(RBASE,(NDEPL+1)*NREGCE*2,IYDPL)
      CALL SETARA(RBASE,NGROUP*NREGCE*2,IFLX)
      CALL SETARA(RBASE,NREGCE,IVX)
      CALL SETARA(IBASE,NDEPL*NREGCE,ISOCMB)
      CALL EVODRV(IPDEPL,IPLIB ,IPRINT,INDREC,NBISO ,NGROUP,
     >            MAXMIX,NREG  ,NTIME ,NPROC ,NDEPL ,NCOMB ,
     >            NREGCE,ITYPE ,INR   ,ISAVE ,ITIXS ,RPAR  ,
     >            XT    ,
     >            IBASE(ISONAM),IBASE(ISONRF),RBASE(ISOAWR),
     >            IBASE(ISONDP),RBASE(ITIMES),IBASE(JJMIX) ,
     >            RBASE(JJDEN) ,IBASE(IMAT)  ,RBASE(IVOL)  ,
     >            RBASE(JFLUXE),IBASE(LMASKM),IBASE(LMASKG),
     >            IBASE(IJM)   ,IBASE(IMILVO),RBASE(IYDPL) ,
     >            RBASE(IFLX)  ,RBASE(IVX)   ,IBASE(ISOCMB))
      CALL RLSARA(IBASE(ISOCMB))
      CALL RLSARA(RBASE(IVX))
      CALL RLSARA(RBASE(IFLX))
      CALL RLSARA(RBASE(IYDPL))
      CALL RLSARA(IBASE(IMILVO))
      CALL RLSARA(IBASE(IJM))
      CALL RLSARA(IBASE(LMASKG))
      CALL RLSARA(IBASE(LMASKM))
      CALL RLSARA(IBASE(ISONDP))
      CALL LCMPUT(IPLIB,'ISOTOPESDENS',NBISO,2,RBASE(JJDEN))
C----
C  RELEASE FINAL MEMORY
C----
      CALL RLSARA(RBASE(ITIMES))
      CALL RLSARA(RBASE(JFLUXE))
      CALL RLSARA(RBASE(ISOAWR))
      CALL RLSARA(IBASE(ISONRF))
      CALL RLSARA(IBASE(ISONAM))
      CALL RLSARA(RBASE(JJDEN))
      CALL RLSARA(IBASE(JJMIX))
      CALL RLSARA(IBASE(IIDL))
      CALL RLSARA(RBASE(IVOL))
      CALL RLSARA(IBASE(IMAT))
      RETURN
C----
C  ERROR FORMAT
C----
 9000 FORMAT(A6,': NUMBER OF MIXTURES IN TRACKING = ',I5,
     >          ' GREATER THAN NUMBER OF MIXTURES IN',
     >          ' INTERNAL LIBRARY =',I5,'.')
      END
