*DECK EVODPL
      SUBROUTINE EVODPL(IMPX,YDPL,NVAR,XT,EPS1,EXPMAX,H1,ITYPE,MU1,
     1 IMA,MAXA,NSUPF,NFISS,KFISS,YSF,ADPL,BDPL,KSAT,IPERM,YST1,YSAT,
     2 MU12,IMA2,ADPL2,BDPL2,YSF2,KFIS2)
C
C-----------------------------------------------------------------------
C
C  MULTI-PURPOSE DRIVER FOR SOLVING THE ISOTOPIC DEPLETION EQUATIONS,
C  TAKING INTO ACCOUNT THE SATURATION PHENOMENA.
C
C  INPUT AND OUTPUT PARAMETERS:
C   IMPX    : PRINT FLAG. EQUAL TO ZERO FOR NO PRINT.
C   YDPL    : INITIAL/FINAL NUMBER DENSITIES. DIMENSION YDPL(NVAR)
C   NVAR    : NUMBER OF NUCLIDES IN THE COMPLETE DEPLETION CHAIN.
C   XT(1),XT(2): INITIAL AND FINAL VALUE OF THE INDEPENDENT VARIABLE.
C   EPS1    : REQUIRED ACCURACY FOR ODE SOLVER.
C   EXPMAX  : A NUCLIDE IS SATURATING IF -ADPL(MU1(I))C (XT(2)-XT(1))
C             .GT.EXPMAX. SUGGESTED VALUE: EXPMAX=80.0
C             EXPMAX=0.0 MEANS THAT THE SATURATION
C             MODEL IS NOT USED 
C   H1      : GUESSED FIRST STEPSIZE.
C   ITYPE   : =1 FIFTH-ORDER RUNGE-KUTTA METHOD.
C             =2 FOURTH-ORDER KAPS-RENTROP METHOD.
C
C  INPUT DATA RELATED TO THE ISOTOPIC DEPLETION CALCULATION:
C   MU1     : POSITION OF EACH DIAGONAL ELEMENT IN VECTOR ADPL.
C   IMA     : POSITION OF THE FIRST NON-ZERO COLUMN ELEMENT IN VECTOR
C             ADPL. DIMENSION MU1(NVAR),IMA(NVAR)
C   MAXA    : FIRST DIMENSION OF VECTORS ADPL AND ADPL2.
C   NSUPF   : NUMBER OF DEPLETING FISSION PRODUCTS.
C   NFISS   : NUMBER OF FISSILE ISOTOPES PRODUCING FISSION PRODUCTS.
C   KFISS   : POSITION IN CHAIN OF THE FISSILE ISOTOPES.
C             DIMENSION KFISS(NFISS)
C   YSF     : INITIAL/FINAL PRODUCT OF THE FISSION YIELDS AND FISSION
C             RATES. DIMENSION YSF(NFISS,NSUPF,2)
C   ADPL    : INITIAL/FINAL DEPLETION MATRIX. DIMENSION ADPL(MAXA,2)
C   BDPL    : INITIAL/FINAL DEPLETION SOURCE. DIMENSION BDPL(NVAR,2)
C
C  SCRATCH STORAGE:
C   KSAT    : DIMENSION KSAT(NVAR)
C   IPERM   : DIMENSION IPERM(NVAR)
C   YST1    : DIMENSION YST1(NVAR)
C   YSAT    : DIMENSION YSAT(NVAR)
C   MU12    : DIMENSION MU12(NVAR)
C   IMA2    : DIMENSION IMA2(NVAR)
C   ADPL2   : DIMENSION ADPL2(MAXA,2)
C   BDPL2   : DIMENSION BDPL2(NVAR,2)
C   YSF2    : DIMENSION YSF2(NFISS,NSUPF,2)
C   KFIS2   : DIMENSION KFIS2(NFISS)
C
C--------------------------------------- AUTHOR: A. HEBERT 26/06/90 ----
C
      IMPLICIT NONE
C----
C PARAMETERS
C----
      INTEGER      IOUT
      PARAMETER   (IOUT=6)
C----
C MEMORY ALLOCATION
C----
      INTEGER      IBASE(1)
      REAL         RBASE(1),BASE(1)
      COMMON       RBASE
      EQUIVALENCE (RBASE(1),IBASE(1),BASE(1))
      INTEGER      IKEV,IYSTG,IA22,IA21,IA12,IMGAR,IIGAR,IAGAR,IBGAR,
     +             IYSFG,IGAR,IBDPL,IYSF,IYSCAL,IY
C----
C INTERFACE VARIABLES
C----
      INTEGER   NFISS,NSUPF,MAXA,NVAR,NOK,NBAD
      INTEGER   MU1(*),IMA(*),KFISS(*),KSAT(*),IPERM(*),KFIS2(*),
     &          MU12(*),IMA2(*)
      REAL      YDPL(*),XT(*),YSF(NFISS,NSUPF,*),ADPL(MAXA,2),
     &          BDPL(NVAR,2),YST1(*),YSAT(*),ADPL2(MAXA,*),
     &          BDPL2(NVAR,*),YSF2(NFISS,NSUPF,*)
C----
C LOCAL VARIABLES
C----
      INTEGER     I,NTER,INDSAT,II,JMN,J,IMN,N,IMPX,ITYPE,NVARM,
     +            IMAM1,JJ,KK,L,NSUPF2,ITER,I0,NSAT,K,NVAR2
      REAL        EXPMAX,EPS1,H1,FACT
      CHARACTER*2 SHOW(65,65)
C
C COMPUTE THE LUMPING INDEX VECTOR IPERM.
C
      DO 10 I=1,NVAR
   10 IPERM(I)=I
      NTER=0
   20 NTER=NTER+1
      INDSAT=0
      DO 30 I=1,NVAR
        IF(IPERM(I).GE.0) THEN
          IF(EXPMAX .GT. 0.0) THEN
            IF(ABS(ADPL(MU1(I),1)*(XT(2)-XT(1))).GT.EXPMAX.AND.
     >         ABS(ADPL(MU1(I),2)*(XT(2)-XT(1))).GT.EXPMAX) THEN
C----
C  IF SATURATING ISOTOPE IS A FISSILE ISOTOPE
C  DO NOT USE SATURATION MODEL
C----
              DO 31 II=1,NFISS
                IF(I.EQ.KFISS(II)) GO TO 35
   31         CONTINUE
              IPERM(I)=0
              IF(INDSAT.EQ.0) THEN
                IPERM(I)=-NTER
                INDSAT=I
              ENDIF
            ENDIF
          ENDIF
        ENDIF
   35   CONTINUE
   30 CONTINUE
      IF(INDSAT.EQ.0) GO TO 60
      DO 50 I=INDSAT+1,NVAR
      JMN=I-MU1(I)+IMA(I-1)+1
      IMN=I-IMA(I)+MU1(I)
      DO 40 J=MIN(JMN,IMN),I-1
      IF((IPERM(I).EQ.0).AND.(IPERM(J).EQ.-NTER)) THEN
         IPERM(I)=-NTER
         GO TO 50
      ENDIF
   40 CONTINUE
   50 CONTINUE
      GO TO 20
   60 NTER=NTER-1
      N=0
      DO 70 I=1,NVAR
      IF(IPERM(I).GT.0) THEN
         N=N+1
         IPERM(I)=N
      ENDIF
   70 CONTINUE
      IF(IMPX.GE.5) THEN
         WRITE(IOUT,400) NVAR,XT(1),XT(2),EPS1,H1,
     >     ITYPE,NTER,NVAR-N,NFISS,NSUPF,(IPERM(I),I=1,NVAR)
         WRITE(IOUT,410) (YDPL(I),I=1,NVAR)
      ENDIF
      IF(IMPX.GE.10) THEN
         NVARM=MIN(NVAR,65)
         WRITE(IOUT,411)
         DO 80 I=1,NVARM
         DO 80 J=1,NVARM
   80    SHOW(I,J)=' '
         IMAM1=0
         DO 120 I=1,NVARM
         DO 90 J=I-MU1(I)+IMAM1+1,I
   90    SHOW(I,J)='*'
         DO 100 J=I-IMA(I)+MU1(I),I
  100    SHOW(J,I)='*'
         IF(I.GT.NVAR-NSUPF) THEN
            DO 110 K=1,NFISS
  110       SHOW(I,KFISS(K))='*'
         ENDIF
  120    IMAM1=IMA(I)
         DO 130 I=1,NVARM
         WRITE(IOUT,'(1X,65A2)') (SHOW(I,J),J=1,NVARM)
  130    CONTINUE
         IF(NVAR.GT.65) WRITE(IOUT,'(18H MATRIX TRUNCATED.)')
      ENDIF
      IF(IMPX.GE.100) THEN
        WRITE(IOUT,500) NVAR
        WRITE(IOUT,'(3I10)') (II,MU1(II),IMA(II),II=1,NVAR)
        WRITE(IOUT,501) NFISS
        WRITE(IOUT,'(10I10)') (KFISS(II),II=1,NFISS)
        WRITE(IOUT,502) NFISS,NSUPF,2
        WRITE(IOUT,'(1P,8E15.7)')
     >    (((YSF(II,JJ,KK),II=1,NFISS),JJ=1,NSUPF),KK=1,2)
        WRITE(IOUT,503) MAXA,2
        WRITE(IOUT,'(1P,8E15.7)')
     >    ((ADPL(II,JJ),II=1,MAXA),JJ=1,2)
        WRITE(IOUT,504) NVAR,2
        WRITE(IOUT,'(1P,8E15.7)')
     >    ((BDPL(II,JJ),II=1,NVAR),JJ=1,2)
      ENDIF
*
* LUMPING OF THE DEPLETION MATRICES.
      DO 180 L=1,2
      NVAR2=NVAR
      DO 140 I=1,NVAR
      IF(L.EQ.1) YST1(I)=YDPL(I)
      BDPL2(I,L)=BDPL(I,L)
      MU12(I)=MU1(I)
  140 IMA2(I)=IMA(I)
      DO 150 I=1,IMA(NVAR)
  150 ADPL2(I,L)=ADPL(I,L)
      DO 160 I=1,NFISS
      KFIS2(I)=KFISS(I)
      DO 160 J=1,NSUPF
  160 YSF2(I,J,L)=YSF(I,J,L)
      NVAR2=NVAR
      NSUPF2=NSUPF
      DO 180 ITER=1,NTER
      I0=0
      NSAT=0
      DO 170 I=1,NVAR
      IF((IPERM(I).GT.0).OR.(IPERM(I).LT.-ITER)) THEN
        I0=I0+1
      ELSE IF(IPERM(I).EQ.-ITER) THEN
        I0=I0+1
        NSAT=NSAT+1
        KSAT(NSAT)=I0
      ENDIF
  170 CONTINUE
      IF(I0.NE.NVAR2) CALL XABORT('EVODPL: ALGORITHM FAILURE 1.')
      CALL SETARA(BASE,NVAR2,IKEV)
      CALL SETARA(BASE,NVAR2-NSAT,IYSTG)
      CALL SETARA(BASE,NSAT**2,IA22)
      CALL SETARA(BASE,(NVAR2-NSAT)*NSAT,IA21)
      CALL SETARA(BASE,(NVAR2-NSAT)*NSAT,IA12)
      CALL SETARA(BASE,NVAR2-NSAT,IMGAR)
      CALL SETARA(BASE,NVAR2-NSAT,IIGAR)
      CALL SETARA(BASE,MAXA,IAGAR)
      CALL SETARA(BASE,NVAR2-NSAT,IBGAR)
      CALL SETARA(BASE,NFISS*NSUPF2,IYSFG)
      CALL SETARA(BASE,NSAT,IGAR)
      CALL EVOSAT(IMPX,L,NSAT,NVAR2,KSAT,YST1,YSAT,MU12,IMA2,MAXA,
     1 NSUPF2,NFISS,KFIS2,YSF2(1,1,L),ADPL2(1,L),BDPL2(1,L),BASE(IKEV),
     2 BASE(IYSTG),BASE(IA22),BASE(IA21),BASE(IA12),BASE(IMGAR),
     3 BASE(IIGAR),BASE(IAGAR),BASE(IBGAR),BASE(IYSFG),BASE(IGAR))
      CALL RLSARA(BASE(IGAR))
      CALL RLSARA(BASE(IYSFG))
      CALL RLSARA(BASE(IBGAR))
      CALL RLSARA(BASE(IAGAR))
      CALL RLSARA(BASE(IIGAR))
      CALL RLSARA(BASE(IMGAR))
      CALL RLSARA(BASE(IA12))
      CALL RLSARA(BASE(IA21))
      CALL RLSARA(BASE(IA22))
      CALL RLSARA(BASE(IYSTG))
      CALL RLSARA(BASE(IKEV))
      NVAR2=NVAR2-NSAT
      IF(L.EQ.1) THEN
         NSAT=0
         I0=0
         DO 175 I=1,NVAR
         IF((IPERM(I).GT.0).OR.(IPERM(I).LT.-ITER)) THEN
            I0=I0+1
            YDPL(I)=YST1(I0)
         ELSE IF(IPERM(I).EQ.-ITER) THEN
            NSAT=NSAT+1
            YDPL(I)=YSAT(NSAT)
         ENDIF
  175    CONTINUE
      ENDIF
  180 CONTINUE
      IF(IMPX.GT.4) THEN
         WRITE(IOUT,420) (YDPL(I),I=1,NVAR)
      ENDIF
*
* SOLUTION OF THE LUMPED DEPLETION SYSTEM.
      IF(NVAR2.EQ.0) GO TO 315
      DO 190 I=1,NVAR2
      FACT=(BDPL2(I,2)-BDPL2(I,1))/(XT(2)-XT(1))
      BDPL2(I,1)=BDPL2(I,1)-FACT*XT(1)
  190 BDPL2(I,2)=FACT
      DO 200 I=1,IMA2(NVAR2)
      FACT=(ADPL2(I,2)-ADPL2(I,1))/(XT(2)-XT(1))
      ADPL2(I,1)=ADPL2(I,1)-FACT*XT(1)
  200 ADPL2(I,2)=FACT
      DO 210 I=1,NFISS
      DO 210 J=1,NSUPF2
      FACT=(YSF2(I,J,2)-YSF2(I,J,1))/(XT(2)-XT(1))
      YSF2(I,J,1)=YSF2(I,J,1)-FACT*XT(1)
  210 YSF2(I,J,2)=FACT
      IF(IMPX.GT.4) THEN
         WRITE(IOUT,430) NSUPF2
         WRITE(IOUT,440) (YST1(I),I=1,NVAR2)
      ENDIF
      IF(IMPX.GT.5) THEN
         NVARM=MIN(NVAR2,65)
         WRITE(IOUT,412)
         DO 220 I=1,NVARM
         DO 220 J=1,NVARM
  220    SHOW(I,J)=' '
         IMAM1=0
         DO 260 I=1,NVARM
         DO 230 J=I-MU12(I)+IMAM1+1,I
  230    SHOW(I,J)='*'
         DO 240 J=I-IMA2(I)+MU12(I),I
  240    SHOW(J,I)='*'
         IF(I.GT.NVAR2-NSUPF2) THEN
            DO 250 K=1,NFISS
  250       SHOW(I,KFIS2(K))='*'
         ENDIF
  260    IMAM1=IMA2(I)
         DO 270 I=1,NVARM
         WRITE(IOUT,'(1X,65A2)') (SHOW(I,J),J=1,NVARM)
  270    CONTINUE
         IF(NVAR2.GT.65) WRITE(IOUT,'(18H MATRIX TRUNCATED.)')
      ENDIF
      CALL SETARA(BASE,2*NVAR2,IBDPL)
      CALL SETARA(BASE,2*NFISS*NSUPF2,IYSF)
      DO 280 I=1,NVAR2
      BASE(IBDPL+I-1)=BDPL2(I,1)
  280 BASE(IBDPL+NVAR2+I-1)=BDPL2(I,2)
      DO 290 I=1,NFISS
      DO 290 J=1,NSUPF2
      BASE(IYSF+(J-1)*NFISS+I-1)=YSF2(I,J,1)
  290 BASE(IYSF+NFISS*NSUPF2+(J-1)*NFISS+I-1)=YSF2(I,J,2)
      CALL SETARA(BASE,NVAR2,IYSCAL)
      CALL SETARA(BASE,NVAR2,IY)
      CALL EVOODE(YST1,NVAR2,XT(1),XT(2),EPS1,H1,NOK,NBAD,ITYPE,MU12,
     1 IMA2,MAXA,NSUPF2,NFISS,KFIS2,BASE(IYSF),ADPL2,BASE(IBDPL),
     2 BASE(IYSCAL),BASE(IY))
      CALL RLSARA(BASE(IY))
      CALL RLSARA(BASE(IYSCAL))
      CALL RLSARA(BASE(IYSF))
      CALL RLSARA(BASE(IBDPL))
      IF(IMPX.GT.4) THEN
         WRITE(IOUT,450) (YST1(I),I=1,NVAR2)
         IF(ITYPE.LE.2)
     >     WRITE(IOUT,'(13H EVODPL: NOK=,I5,6H NBAD=,I5)')
     >       NOK,NBAD
      ENDIF
      DO 310 I=1,NVAR
      IF(IPERM(I).GT.0) YDPL(I)=YST1(IPERM(I))
  310 CONTINUE
*
* COMPUTE NUMBER DENSITIES OF THE SATURATED ISOTOPES.
  315 IF(NTER.EQ.0) GO TO 370
      DO 320 I=1,NVAR
      YST1(I)=YDPL(I)
      BDPL2(I,2)=BDPL(I,2)
      MU12(I)=MU1(I)
  320 IMA2(I)=IMA(I)
      DO 330 I=1,IMA(NVAR)
  330 ADPL2(I,2)=ADPL(I,2)
      DO 340 I=1,NFISS
      KFIS2(I)=KFISS(I)
      DO 340 J=1,NSUPF
  340 YSF2(I,J,2)=YSF(I,J,2)
      NVAR2=NVAR
      NSUPF2=NSUPF
      DO 360 ITER=1,NTER
      I0=0
      NSAT=0
      DO 350 I=1,NVAR
      IF((IPERM(I).GT.0).OR.(IPERM(I).LT.-ITER)) THEN
        I0=I0+1
      ELSE IF(IPERM(I).EQ.-ITER) THEN
        I0=I0+1
        NSAT=NSAT+1
        KSAT(NSAT)=I0
      ENDIF
  350 CONTINUE
      IF(I0.NE.NVAR2) CALL XABORT('EVODPL: ALGORITHM FAILURE 2.')
      CALL SETARA(BASE,NVAR2,IKEV)
      CALL SETARA(BASE,NVAR2-NSAT,IYSTG)
      CALL SETARA(BASE,NSAT**2,IA22)
      CALL SETARA(BASE,(NVAR2-NSAT)*NSAT,IA21)
      CALL SETARA(BASE,(NVAR2-NSAT)*NSAT,IA12)
      CALL SETARA(BASE,NVAR2-NSAT,IMGAR)
      CALL SETARA(BASE,NVAR2-NSAT,IIGAR)
      CALL SETARA(BASE,MAXA,IAGAR)
      CALL SETARA(BASE,NVAR2-NSAT,IBGAR)
      CALL SETARA(BASE,NFISS*NSUPF2,IYSFG)
      CALL SETARA(BASE,NSAT,IGAR)
      CALL EVOSAT(IMPX,1,NSAT,NVAR2,KSAT,YST1,YSAT,MU12,IMA2,MAXA,
     1 NSUPF2,NFISS,KFIS2,YSF2(1,1,2),ADPL2(1,2),BDPL2(1,2),BASE(IKEV),
     2 BASE(IYSTG),BASE(IA22),BASE(IA21),BASE(IA12),BASE(IMGAR),
     3 BASE(IIGAR),BASE(IAGAR),BASE(IBGAR),BASE(IYSFG),BASE(IGAR))
      CALL RLSARA(BASE(IGAR))
      CALL RLSARA(BASE(IYSFG))
      CALL RLSARA(BASE(IBGAR))
      CALL RLSARA(BASE(IAGAR))
      CALL RLSARA(BASE(IIGAR))
      CALL RLSARA(BASE(IMGAR))
      CALL RLSARA(BASE(IA12))
      CALL RLSARA(BASE(IA21))
      CALL RLSARA(BASE(IA22))
      CALL RLSARA(BASE(IYSTG))
      CALL RLSARA(BASE(IKEV))
      IF(IMPX.GT.4) THEN
         WRITE(IOUT,425) ITER,(YSAT(I),I=1,NSAT)
      ENDIF
      NVAR2=NVAR2-NSAT
      NSAT=0
      I0=0
      DO 360 I=1,NVAR
      IF((IPERM(I).GT.0).OR.(IPERM(I).LT.-ITER)) THEN
        I0=I0+1
        YDPL(I)=YST1(I0)
      ELSE IF(IPERM(I).EQ.-ITER) THEN
        NSAT=NSAT+1
        YDPL(I)=YSAT(NSAT)
      ENDIF
  360 CONTINUE
  370 IF(IMPX.GT.3) THEN
         WRITE(IOUT,460) (YDPL(I),I=1,NVAR)
      ENDIF
      RETURN
*
  400 FORMAT(//45H EVODPL: SOLUTION OF THE DEPLETION EQUATIONS.//14X,
     1 25HTOTAL NUMBER OF NUCLIDES=,I5/26X,13HINITIAL TIME=,1P,E12.4/
     2 28X,11HFINAL TIME=,E12.4/15X,24HACCURACY FOR ODE SOLVER=,E12.4/
     3 16X,23HGUESSED FIRST STEPSIZE=,E12.4,0P/22X,17HTYPE OF SOLUTION=,
     4 I3/39H NUMBER OF GROUP OF SATURATED NUCLIDES=,I5/10X,
     5 29HNUMBER OF SATURATED NUCLIDES=,I5/12X,19HNUMBER OF FISSILE N,
     6 8HUCLIDES=,I5/12X,27HNUMBER OF FISSION PRODUCTS=,I5//
     7 22H LUMPING INDEX VECTOR:/(1X,20I5))
  410 FORMAT(/48H EVODPL: INITIAL VALUES OF THE DEPLETION SYSTEM:/
     1 (1X,1P,10E12.4))
  411 FORMAT(/' EVODPL: DEPLETION MATRIX PROFILE:')
  412 FORMAT(/' EVODPL: LUMPED DEPLETION MATRIX PROFILE')
  420 FORMAT(/53H EVODPL: SATURATED INITIAL CONDITIONS OF THE DEPLETIO,
     1 9HN SYSTEM:/(1X,1P,10E12.4))
  425 FORMAT(/51H EVODPL: FINAL VALUES OF THE SATURATED NUCLIDES IN ,
     1 9HGROUP NO.,I5//(1X,1P,10E12.4))
  430 FORMAT(/42H NUMBER OF NON-SATURATED FISSION PRODUCTS=,I5)
  440 FORMAT(/55H EVODPL: INITIAL VALUES OF THE LUMPED DEPLETION SYSTEM:
     1 /(1X,1P,10E12.4))
  450 FORMAT(/53H EVODPL: ODE SOLUTION OF THE LUMPED DEPLETION SYSTEM:/
     1 (1X,1P,10E12.4))
  460 FORMAT(/42H EVODPL: SOLUTION OF THE DEPLETION SYSTEM:/
     1 (1X,1P,10E12.4))
 500  FORMAT(' VECTOR MU1 AND IMA OF DIMENSION  = ',I10/
     >       '   ISOTOPE       MU1       IMA')
 501  FORMAT(' VECTOR KFISS DIMENSION  = ',I10)
 502  FORMAT(' MATRIX YSF DIMENSION    = ',I10,I10,I10)
 503  FORMAT(' MATRIX ADPL DIMENSION   = ',I10,I10)
 504  FORMAT(' MATRIX BDPL DIMENSION   = ',I10,I10)
      END
