*DECK FLUASR
      SUBROUTINE FLUASR(IPSYS ,IPMACR,IPRINT,KDIR  ,NGROUP,NBMIX ,
     >                  NREGIO,NUNKNO,ILEAK ,MATCOD,KEYFLX,
     >                  FUNKNO,SUNKNO,
     >                  DB2   ,SIGS0 ,XSCAT ,IJJ   ,NJJ   ,IPOS)
C
C---------------------------  FLUASR  ---------------------------------
C
C   1- PROGRAMME STATISTICS:
C       NAME     : FLUASR
C       USE      : COMPLETE PSEUDO-ADJOINT SOURCE
C                  INCLUDING SCATTERING TERMS
C       MODIFIED : 2000/02/08 (G. MARLEAU)
C
C   2- ROUTINE PARAMETERS:
C    INPUT:
C      IPSYS   : POINTER TO THE PIJ                   I
C      IPMACR  : POINTER TO THE MACROLIB              I
C      IPRINT  : PRINT SELECTION FOR FLUX MODULES:    I
C               = 0/1/2/3 NO PRINT/SHORT PRINT/LONG
C      KDIR    : DIRECTION FOR PC                     I
C                = 0 FOR PIJ OR WIJ
C                = K FOR PIJK OR WIJK K=1,2,3 B1
C                = K+3 FOR PIJK OR WIJK K=4,5,6 B0
C      NGROUP  : NUMBER OF GROUPS.                    I
C      NBMIX   : NUMBER OF MIXTURES.                  I
C      NREGIO  : NUMBER OF REGIONS.                   I
C      NUNKNO  : NUMBER OF UNKNOWN IN THE SYSTEM.     I
C      ILEAK   : METHOD USED TO INCLUDE DB2 EFFECT    I
C                = 0 NO DB2 EFFECT
C                = 1 THE SCATTERING MODIFIED CP MATRIX
C                    IS MULTIPLIED BY A PNL FACTOR
C                = 2 THE REDUCED CP MATRIX IS MULTIPLIED BY
C                    PNL FACTOR
C                = 3 SIGS0-DB2 APPROXIMATION
C                = 4 ALBEDO APPROXIMATION
C                = 5,6,7,8,9,10 HETEROGENEOUS WITH PIJK
C                  AND FIXED B->5 OR SEARCH
C                  X->6 Y->7 Z->8C R->9 ALL->10
C      MATCOD  : MATERIAL CODE IN REGIONS.            I(NREGIO)
C      KEYFLX  : FLUX ELEMENTS IN UNKNOWN SYSTEM.     I(NREGIO)
C      FUNKNO  : UNKNOWN VECTOR SOLVED FOR.           R(NUNKNO,NGROUP)
C      SUNKNO  : SOURCE FOR SYSTEM OF UNKNOWN.        R(NUNKNO,NGROUP)
C      DB2     : LEAKAGE PARAMETERS.                  R(NGROUP)
C    OUTPUT:
C      FUNKNO  : UNKNOWN VECTOR SOLVED FOR.           R(NUNKNO,NGROUP)
C    WORK STORAGE:
C      SIGS0   : WITHIN-GROUP SCATTERING              R(0:NBMIX)
C                SIGS0(0)=0.0 IS FOR VOID
C      XSCAT   : SCATTERING VECTOR                    R(0:NBMIX*NGROUP)
C                XSCAT(0) = 0.0
C      IJJ     : LAST SCATTERING GROUP                I(0:NBMIX)
C                IJJ(0) = 0
C      NJJ     : NUMBER OF SCATTERING GROUP           I(0:NBMIX)
C                NJJ(0) = -NGROUP
C      IPOS    : POSITION SELF SCATTERING IN XSCAT    I(0:NBMIX)
C                IPOS(0)= NGROUP+1
C
C---------------------------   FLUASR  --------------------------------
C
      IMPLICIT    NONE
      INTEGER     IOUT,ILCMUP,ILCMDN
      CHARACTER   NAMSBR*6
      PARAMETER  (IOUT=6,ILCMUP=1,ILCMDN=2,NAMSBR='FLUASR')
C----
C  ROUTINE PARAMETERS
C----
      INTEGER     IPSYS,IPMACR,IPRINT,KDIR,NGROUP,
     >            NBMIX,NREGIO,NUNKNO,ILEAK
      INTEGER     MATCOD(NREGIO),KEYFLX(NREGIO),
     >            IJJ(0:NBMIX),NJJ(0:NBMIX),IPOS(0:NBMIX)
      REAL        FUNKNO(NUNKNO,NGROUP),
     >            SUNKNO(NUNKNO,NGROUP),DB2(NGROUP),
     >            SIGS0(0:NBMIX),XSCAT(0:NBMIX*NGROUP)
C----
C  LOCAL PARAMETERS
C----
      CHARACTER   CGRPNM*12,CAN*3,CNSIGW*12
      INTEGER     IDIR,KRSK,IGR,JGR,IREG,IBM,IGSCAT,IRLOC
C----
C  FOR DIRECTIONS KDIR=0 USE ISOTROPIC SCATTERING
C  FOR DIRECTIONS KDIR=1,2,3 USE ANISOTROPIC SCATTERING B1
C  FOR DIRECTIONS KDIR=4,5,6 USE ANISOTROPIC SCATTERING B0
C  KRSK = LOCATION OF FIRST ELEMENT FOR VARIOUS DIRECTION
C----
      IF(KDIR .EQ. 0) THEN
        CAN='A 0'
        CNSIGW='DRAGON-S0XSC'
        IDIR=0
      ELSE IF(KDIR .GT. 3) THEN
        CAN='A 1'
        CNSIGW='SIGW 1'
        IDIR=KDIR-3
      ELSE
        CAN='A 1'
        CNSIGW='SIGW 1'
        IDIR=KDIR
      ENDIF
      KRSK=IDIR*NREGIO
      DO 110 IGR=1,NGROUP
        WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') IGR,NGROUP
        CALL LCMSIX(IPSYS,CGRPNM,ILCMUP)
        CALL LCMSIX(IPMACR,CGRPNM,ILCMUP)
        IF(CNSIGW .EQ. 'DRAGON-S0XSC') THEN
          CALL LCMGET(IPSYS,CNSIGW,SIGS0(0))
        ELSE
          CALL LCMGET(IPMACR,CNSIGW,SIGS0(1))
          SIGS0(0)=0.0
        ENDIF
        CALL LCMGET(IPMACR,'NJJ'//CAN,NJJ(1))
        CALL LCMGET(IPMACR,'IJJ'//CAN,IJJ(1))
        CALL LCMGET(IPMACR,'IPO'//CAN,IPOS(1))
        CALL LCMGET(IPMACR,'SCA'//CAN,XSCAT(1))
C----
C  ADD TO REGION SOURCE SCATTERING SOURCES FROM ALL GROUPS
C----
        DO 120 IREG=1,NREGIO
          IBM=MATCOD(IREG)
          IGSCAT=IJJ(IBM)
          IRLOC=KRSK+KEYFLX(IREG)
          DO 121 JGR=1,NJJ(IBM)
            IF(IGSCAT .EQ. IGR) THEN
              SUNKNO(IRLOC,IGR)=SUNKNO(IRLOC,IGR)
     >         +SIGS0(IBM)*FUNKNO(IRLOC,IGR)
            ELSE
              SUNKNO(IRLOC,IGR)=SUNKNO(IRLOC,IGR)
     >         +XSCAT(IPOS(IBM)+JGR-1)*FUNKNO(IRLOC,IGSCAT)
            ENDIF
            IGSCAT=IGSCAT-1
 121      CONTINUE
 120    CONTINUE
        IF(ILEAK.EQ.3) THEN
            DO 150 IREG=1,NREGIO
              IRLOC=KRSK+KEYFLX(IREG)
              SUNKNO(IRLOC,IGR)=SUNKNO(IRLOC,IGR)
     >                         -DB2(IGR)*FUNKNO(IRLOC,IGR)
 150        CONTINUE
        ENDIF
        CALL LCMSIX(IPMACR,CGRPNM,ILCMDN)
        CALL LCMSIX(IPSYS,CGRPNM,ILCMDN)
 110  CONTINUE
      RETURN
      END
