*DECK FLUGET
      SUBROUTINE FLUGET(IPFLUX,LEAKSW,NGROUP,NREGIO,NUNKNO,ITPIJ ,
     >                  ITYPEC,ISADJ ,ICOPAR,EPSPAR,COPTIO,ILEAK ,
     >                  IPRINT,INITFL,B2    ,REFKEF,
     >                  KEYFLX,FUNKNO)
C
C---------------------------  FLUGET  ---------------------------------
C
C   1- PROGRAMME STATISTICS:
C       NAME     : FLUGET
C       USE      : READ AND ANALYZE DATA FOR FLUX SOLUTION MODULE
C       MODIFIED : EXTRACTED FROM OLD FLUGPI TO MAKE FLUGPI 
C                  COMPATIBLE WITH MOC AND SAD 
C                  2000/06/28  (G. MARLEAU)
C
C   2- ROUTINE PARAMETERS:
C    INPUT
C      IPFLUX  : POINTER TO THE SOLUTION                  I
C      LEAKSW  : LEAKAGE SWITCH                           L
C      NGROUP  : NUMBER OF GROUPS.                        I
C      NREGIO  : NUMBER OF REGIONS.                       I
C      NUNKNO  : NUMBER OF UNKNOWN IN THE SYSTEM.         I
C      ITPIJ   : TYPE OF CP AVAILABLE                     I
C                = 1 SCATT MOD PIJ (WIJ)
C                = 2 STAND. PIJ
C                = 3 SCATT MOD PIJ+PIJK (WIJ,WIJK)
C                = 4 STAND. PIJ+PIJK
C    INPUT/OUTPUT
C      ITYPEC  : TYPE OF FLUX EVALUATION                  I
C                = 0  SKIP THE FLUX CALCULATION
C                = 1  DIRECT PROBLEM FISSION SOURCES/
C                     K EFFECTIVE CONVERGENCE
C                = 2  DIRECT PROBLEM FISSION SOURCES/
C                     K EFFECTIVE CONVERGENCE/
C                     FIXED DB2 BUCKLING
C                = 3  DIRECT PROBLEM FISSION SOURCES/
C                     K EFFECTIVE CONVERGENCE/
C                     FIXED B2 BUCKLING
C                     D EVALUATION
C                = 4  DIRECT PROBLEM FISSION SOURCES/
C                     DB2 BUCKLING CONVERGENCE
C                = 5  DIRECT PROBLEM DB2 SOURCES
C                = 6  DIRECT PROBLEM FIXED SOURCES
C      ISADJ   : ADJOINT CALCULATION AND SAVE OPTION      I
C                = 0 ONLY DIRECT FLUX
C                = 1 DIRECT AND PSEUDO ADJOINT FLUX
C                = 2 DIRECT, PSEUDO-ADJOINT AND ADJOINT
C                = 3 DIRECT, PSEUDO ADJOINT AND
C                    GENERALIZED PSEUDO-ADJOINT
C                = 4 DIRECT, PSEUDO ADJOINT, ADJOINT,
C                    GENERALIZED PSEUDO-ADJOINT AND
C                    GENERALIZED ADJOINT
C      ICOPAR  : ITERATION PARAMETERS                     I(6)
C                IFRITR=ICOPAR(1)
C                IACITR=ICOPAR(2)
C                IREBAL=ICOPAR(3)
C                MAXINR=ICOPAR(4)
C                MAXOUT=ICOPAR(5)
C                ICONTM=ICOPAR(6)
C      EPSPAR  : ITERATION PRECISION                      R(NBEPS)
C                EPSINR=EPSPAR(1)
C                EPSOUT=EPSPAR(2)
C                EPSUNK=EPSPAR(3)
C                EPSGPA=EPSPAR(4)
C                CONGPA=EPSPAR(5)
C      COPTIO  : LEAKAGE TYPE                             C*4
C                'LKRD' (IMPOSED LEAKAGE ), 'B0' (B-0),
C                'P0' (P-0), 'B1' (B-1), 'P1' (P-1)
C                'B0TR' (B-0 WITH TRANSPORT CORRECTION).
C                COPTIO='B0' IS DEFAULT FOR TYPE B AND L
C      ILEAK   : METHOD USED TO INCLUDE LEAKAGE EFFECT    I
C                = 0 NO DB2 EFFECT (DEFAULT FOR TYPE S AND K)
C                = 1 HOMOGENEOUS METHOD WITH SCATTERING MODIFIED
C                    CP MATRIX MULTIPLIED BY A PNL FACTOR
C                = 2 HOMOGENEOUS METHOD REDUCED CP MATRIX
C                    MULTIPLIED BY PNL FACTOR
C                    (DEFAULT FOR TYPE B AND L)
C                = 3 HOMOGENEOUS METHOD SIGS0-DB2 APPROXIMATION
C                = 4 HOMOGENEOUS METHOD ALBEDO APPROXIMATION
C                = 5,6,7,8,9,10 HETEROGENEOUS METHOD WITH PIJK
C                  AND FIXED B->5 OR SEARCH X->6 Y->7 Z->8
C                  R->9 ALL->10
C      IPRINT  : PRINT SELECTION FOR FLU MODULE:          I
C               = 0/1/2/3 NO PRINT/SHORT PRINT/LONG
C      INITFL  : FLUX INITIALIZATION OPTION               I
C                = 0 DEFAULT INITIALIZATION
C                = 1 READ FROM INPUT AND STROE IN FLXUNK
C                = 2 TAKEN FROM DATA STRUCTURE
C      B2      : INITIAL ESTIMATE OF THE BUCKLING FOR     R(4)
C                THE B2 SEARCH OR IMPOSED BUCKLING.
C      REFKEF  : TARGET K-EFFECTIVE FOR TYPE B OR L       D
C      KEYFLX  : FLUX ELEMENTS IN UNKNOWN SYSTEM.         R(NREGIO)
C      FUNKNO  : UNKNOWN VECTOR SOLVED FOR.               R(NUNKNO,
C                                                           NGROUP)
C---------------------------   FLUGET  --------------------------------
C
      IMPLICIT    NONE
      INTEGER     NBUCKN,NLEAK,NSDIR,NBEPS,ILCMUP,ILCMDN 
      CHARACTER   NAMSBR*6
      REAL        EPSCUT
      PARAMETER  (NBUCKN=5,NLEAK=5,NSDIR=6,NBEPS=5,
     >            ILCMUP=1,ILCMDN=2,EPSCUT=1.0E-10,
     >            NAMSBR='FLUGET')
C----
C  ROUTINE PARAMETERS
C----
      INTEGER     IPFLUX,NGROUP,NREGIO,NUNKNO,ITPIJ ,
     >            ITYPEC,ISADJ,ILEAK,IPRINT,INITFL
      CHARACTER   COPTIO*4
      INTEGER     ICOPAR(6),KEYFLX(NREGIO)
      LOGICAL     LEAKSW
      REAL        EPSPAR(NBEPS),B2(4),FUNKNO(NUNKNO,NGROUP)
      DOUBLE PRECISION REFKEF
C----
C  LOCAL PARAMETERS
C----
      CHARACTER   CARLIR*4,CDIRFL*12
      INTEGER     INITLK,IRSDIR(NSDIR),ISDIR,JDIR,ILCMLN,ILCMTY
      DOUBLE PRECISION BRSDIR(NSDIR)
C----
C  INITIALIZE LOCAL OPTIONS
C----
      INITLK=0
      IF(MOD(ITPIJ,2) .EQ. 0) THEN
        ICOPAR(4)=MAX(4*NGROUP,ICOPAR(4))
      ELSE
        ICOPAR(4)=MAX(2*NGROUP,ICOPAR(4))
      ENDIF
      COPTIO='B0'
      DO 100 JDIR=1,NSDIR-1
        BRSDIR(JDIR)=0.0D0
        IRSDIR(JDIR)=0
 100  CONTINUE
      BRSDIR(NSDIR)=1.0D0
      IRSDIR(NSDIR)=0
      IPRINT=1
      CARLIR='    '
C----
C  READ FLU OPTIONS
C----
 1000 CONTINUE
      CALL FLUGPI(IPRINT,ITYPEC,ISADJ ,ICOPAR,EPSPAR,COPTIO,
     >            ILEAK ,INITFL,INITLK,IRSDIR,BRSDIR,CARLIR)
C----
C  VALIDATE OPTIONS
C----
      IF(CARLIR .EQ. ';') THEN
        GO TO 2000
      ELSE IF(CARLIR .EQ. 'INIT') THEN
        IF(INITFL .EQ. 1) THEN
          CALL FLURFL(NGROUP,NREGIO,NUNKNO,KEYFLX,FUNKNO)
        ENDIF
        CARLIR='    '
      ELSE
        CALL XABORT(NAMSBR//
     >  ': READ ERROR - ILLEGAL KEYWORD '//CARLIR)
      ENDIF
      GO TO 1000
 2000 CONTINUE
C----
C  REFERENCE KEFF PROVIDED
C----
      REFKEF=BRSDIR(NSDIR)
      B2(1)=0.0
      B2(2)=0.0
      B2(3)=0.0
      B2(4)=0.0
C----
C  REFERENCE LEAKAGE PROVIDED
C---- 
      IF(INITLK .NE. 0) THEN
        CDIRFL='FLUXDIRECT  '
        CALL LCMSIX(IPFLUX,CDIRFL,ILCMUP)
        CALL LCMLEN(IPFLUX,'B2  HETE',ILCMLN,ILCMTY)
        IF(ILCMLN .EQ. 3) THEN
          CALL LCMGET(IPFLUX,'B2  HETE',B2)
        ENDIF
        CALL LCMLEN(IPFLUX,'B2  B1HOM',ILCMLN,ILCMTY)
        IF(ILCMLN .EQ. 1) THEN
          CALL LCMGET(IPFLUX,'B2  B1HOM',B2(4))
        ENDIF
        CALL LCMSIX(IPFLUX,CDIRFL,ILCMDN)
      ENDIF
      IF(ILEAK .EQ. 1 .AND. MOD(ITPIJ,2) .EQ. 1) THEN
C----
C  PNL USED WHEN SKIP IS OFF
C----
        ILEAK=2
      ELSE IF(ILEAK .EQ. 2 .AND. MOD(ITPIJ,2) .EQ. 0) THEN
C----
C  PNLR USED WHEN SKIP IS ON
C----
        ILEAK=1
      ELSE IF(ILEAK .EQ. 5 .AND. ITPIJ .LT. 3) THEN
        CALL XABORT(NAMSBR//
     >  ': HETEROGENEOUS BUCKLING CALCULATIONS REQUIRE PIJK MATRICES')
      ENDIF
      ISDIR=0
      IF(ITYPEC .EQ. 2 ) THEN
        IF(ILEAK .EQ. 5) THEN
          DO 110 JDIR=1,NSDIR-1
            IF(IRSDIR(JDIR) .NE. 1) GO TO 115
 110      CONTINUE
        ELSE
          IF(IRSDIR(5) .EQ. 1) GO TO 115
        ENDIF
        CALL XABORT(NAMSBR//
     >  ': BUCKLING REQUIRED FOR TYPE K WITH IMPOSED BUCKLING')
 115    CONTINUE
        IF(INITLK .NE. 2) THEN
          ITYPEC=3
        ENDIF
      ELSE
        DO 120 JDIR=1,NSDIR-1
          IF(IRSDIR(JDIR) .EQ. -1) THEN
            ISDIR=JDIR
            GO TO 125
          ENDIF
 120    CONTINUE
 125    CONTINUE
      ENDIF
      IF(ILEAK .EQ. 5) THEN
        IF(IRSDIR(5) .NE. 0) THEN
          DO 130 JDIR=1,NSDIR-2
            IF(IRSDIR(JDIR) .NE. 0) THEN
              CALL XABORT(NAMSBR//
     >        ': GLOBAL BUCKLING INCONSISTENT WITH DIRECTION BUCKLING')
            ENDIF
 130      CONTINUE
          B2(1)=BRSDIR(5)/3.0
          B2(2)=B2(1)
          B2(3)=B2(1)
          B2(4)=BRSDIR(5)
        ELSE IF(IRSDIR(4) .NE. 0) THEN
          DO 140 JDIR=1,2
            IF(IRSDIR(JDIR) .NE. 0) THEN
              CALL XABORT(NAMSBR//
     >        ': RADIAL BUCKLING INCONSISTENT WITH X, Y BUCKLING')
            ENDIF
 140      CONTINUE
          B2(1)=BRSDIR(4)/2.0
          B2(2)=B2(1)
          B2(3)=BRSDIR(3)
          B2(4)=BRSDIR(3)+BRSDIR(4)
        ELSE
          B2(1)=BRSDIR(1)
          B2(2)=BRSDIR(2)
          B2(3)=BRSDIR(3)
          B2(4)=BRSDIR(1)+BRSDIR(2)+BRSDIR(3)
        ENDIF
        ILEAK=ILEAK+ISDIR
      ELSE
        DO 150 JDIR=1,NSDIR-2
          IF(IRSDIR(JDIR) .NE. 0) THEN
            CALL XABORT(NAMSBR//
     >      ': ILLEGAL DIRECTION BUCKLING FOR HOMOGENEOUS LEAKAGE')
          ENDIF
 150    CONTINUE
        B2(4)=BRSDIR(5)
      ENDIF
      IF(LEAKSW) THEN
        IF( (ITYPEC .EQ. 4 .AND. ILEAK .NE. 4) .OR. 
     >      (ITYPEC .EQ. 5 .AND. ILEAK .NE. 4) ) THEN
          CALL XABORT(NAMSBR//': B2 CONVERGENCE NOT ALLOWED HERE')
        ENDIF
      ENDIF
      EPSPAR(1)=MAX(EPSPAR(1),EPSCUT)
      EPSPAR(2)=MAX(EPSPAR(2),EPSCUT)
      EPSPAR(3)=MAX(EPSPAR(1),EPSPAR(3),EPSCUT)
      IF(ICOPAR(5) .EQ. 0) THEN
        IF(ITYPEC .EQ. 3 .OR.
     >     ITYPEC .EQ. 4 .OR.
     >     ITYPEC .EQ. 5      ) THEN
          ICOPAR(5)=10*NREGIO+1
        ELSE
          ICOPAR(5)=2*NREGIO-1
        ENDIF
      ENDIF
C----
C  FOR ADJOINT COMPUTATION
C  ONLY TYPE K OR TYPE S AVAILABLE
C  TO BE COMMENTED TO TREAT
C  BUCKLING ADJOINT CASES
C----
      IF (ISADJ.GT.0 .AND. ILEAK.GT.0) CALL XABORT(NAMSBR//
     >    ': ILLEGAL TYPE FOR GENERALIZED ADJOINT CALCULATION')
C----
C  FOR ADJOINT AND GENERALIZED ADJOINT PROBLEM
C  TEST IF TYPE AND LEAK OPTION ADEQUATE
C  IF NOT SET DEFAULT
C  ILEAK=3 AND ITYPEC =2
C----
      IF(ISADJ .GE. 3) THEN
        IF(ITYPEC .LE. 0 .OR. ITYPEC .GE. 5) THEN
          CALL XABORT(NAMSBR//
     >    ': ILLEGAL TYPE FOR GENERALIZED ADJOINT CALCULATION')
        ELSE IF(ITYPEC .GE. 2) THEN
          IF(ILEAK .GE. 4) THEN
            ILEAK=3
          ENDIF
        ENDIF
      ELSE IF(ISADJ .GE. 1) THEN
        IF(ITYPEC .GE. 2 .AND. ITYPEC .LE. 5) THEN
          IF(ILEAK .GE. 4) THEN
            ILEAK=3
          ENDIF
        ENDIF
      ENDIF
      IF(ITYPEC .EQ. 1) THEN
        ILEAK=0
      ENDIF
      RETURN
      END
