*DECK LIB
      SUBROUTINE LIB(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
C
C-----------------------------------------------------------------------
C
C INTERPOLATION OF NUCLEAR PROPERTIES IN AN INTERNAL LIBRARY.
C
C INPUT/OUTPUT PARAMETERS:
C  NENTRY : NUMBER OF LINKED LISTS OR FILES USED BY THE MODULE.
C  HENTRY : CHARACTER*12 NAME OF EACH LINKED LIST OR FILE.
C  IENTRY : =1 LINKED LIST; =2 XSM FILE; =3 SEQUENTIAL BINARY FILE;
C           =4 SEQUENTIAL ASCII FILE.
C  JENTRY : =0 THE LINKED LIST OR FILE IS CREATED.
C           =1 THE LINKED LIST OR FILE IS OPEN FOR MODIFICATIONS;
C           =2 THE LINKED LIST OR FILE IS OPEN IN READ-ONLY MODE.
C  KENTRY : =FILE UNIT NUMBER; =LINKED LIST ADDRESS OTHERWISE.
C           DIMENSION HENTRY(NENTRY),IENTRY(NENTRY),JENTRY(NENTRY),
C           KENTRY(NENTRY)
C
C LINKED LIST / XSM FILE:
C  HENTRY(1) : CREATE OR MODIFICATION TYPE(L_LIBRARY).
C  HENTRY(2) : OPTIONAL READ-ONLY TYPE(L_MACROLIB) USED TO INITIALIZE A
C              NEW LATTICE CODE LIBRARY.
C
C-------------------------------------- AUTHOR: A. HEBERT ; 01/11/94 ---
C-------------------------------------- MODIF : G.M.      ; 97-02-12 ---
C
      IMPLICIT     NONE
      INTEGER      IOUT,NSTATE,NTC,ILCMUP,ILCMDN,MAXED,MAXISD
      CHARACTER    NAMSBR*6
      PARAMETER   (IOUT=6,NSTATE=20,NTC=3,ILCMUP=1,ILCMDN=2,MAXED=50,
     >             MAXISD=200,NAMSBR='LIB   ')
C----
C  ROUTINE PARAMETERS
C----
      INTEGER      NENTRY                  	
      CHARACTER    HENTRY(NENTRY)*12
      INTEGER      IENTRY(NENTRY),JENTRY(NENTRY),KENTRY(NENTRY)
C----
C  MEMORY ALLOCATION
C----
      INTEGER      IBASE(1)
      REAL         RBASE
      COMMON       RBASE(1)
      EQUIVALENCE (RBASE(1),IBASE(1))
      INTEGER      IENER,IJCEDM,IBSTD,JHLIB,JTMPMX,JMASKI,ILSHI,
     >             JJNTFG,ISNGAR,JJAWR,JJTN,LMASKG,JDENMD,LISM,
     >             LOCUPD,JDENIX,JIMIX,LMASKM,JDENMX,JJDEN,JJMIX,
     >             JJNRF,JJNAM,IADNAM
C----
C  INPUT
C----
      INTEGER      ITYPLU,INTLIR
      CHARACTER    CARLIR*12
      REAL         REALIR
      DOUBLE PRECISION DBLLIR
C----
C  LOCAL PARAPETERS
C----
      CHARACTER    TEXT12*12,HSIGN*12,TEXT4*4,HVECT(MAXED)*6,HADD*6
      INTEGER      ISTATE(NSTATE),KCHAR(NTC),IPRINT,NBISOX,NBMIXX,
     >             IPLIBX,MAXMIX,INDREC,NBISO,NGRO,NGT,NGF,NGFR,
     >             NL,ITRANC,ITIME,NLIB,NIDEPL,NCOMB,
     >             NEDMAC,NBMIX,NRES,MAXISM,IEN,ILCMLN,ILCMTY,
     >             ITC,IPMACR,IED,JED,KED,IDP,ITEXT4,IBSTEP,MAXISO,
     >             IPLIB,NDEPL,NEDMA0,ITPROC,ISOADD,NADDXS
      REAL         TMPDAY(3),DELT,TIMBRN 
      INTEGER      IKSTEP
C----
C  PARAMETER VALIDATION.
C----
      IF(NENTRY .EQ. 0) CALL XABORT(NAMSBR//
     >': PARAMETER EXPECTED.')
      IF(IENTRY(1) .NE. 1 .AND.
     >   IENTRY(1) .NE. 2) CALL XABORT(NAMSBR//
     >': LINKED LIST OR XSM FILE EXPECTED AT LHS.')
      IF(JENTRY(1) .NE. 0 .AND.
     >   JENTRY(1) .NE. 1) CALL XABORT(NAMSBR//': ENTRY'
     1 //' IN CREATE OR MODIFICATION MODE EXPECTED.')
      IPLIB=KENTRY(1)
C----
C  READ THE INPUT DATA.
C  DEFAULT OPTIONS:
C----
      ISOADD=0
      IPRINT=1
      NBISOX=0
      NBMIXX=0
      IPLIBX=0
      IF(JENTRY(1) .EQ. 0) THEN
        MAXMIX=0
        INDREC=1
        NBISO=0
        NGRO=0
        NGT=0
        NGF=0
        NGFR=0
        NL=2
        ITRANC=0
        ITIME=1
        NLIB=0
        NIDEPL=0
        NCOMB=0
        NEDMAC=0
        NBMIX=0
        NRES=0
        MAXISM=MAXISD
C----
C TRY TO FIND A READ-ONLY MACROLIB TO INCLUDE IN THE LIBRARY.
C----
        DO 100 IEN=2,NENTRY
          IF(IENTRY(IEN) .LE. 2 .AND. JENTRY(IEN) .EQ. 2) THEN
            CALL LCMLEN(KENTRY(IEN),'MACROLIB',ILCMLN,ILCMTY)
            IPMACR=0
            IF(ILCMLN .EQ. -1) THEN
              IPMACR=1
              CALL LCMSIX(KENTRY(IEN),'MACROLIB',ILCMUP)
            ENDIF
            CALL LCMLEN(KENTRY(IEN),'SIGNATURE',ILCMLN,ILCMTY)
            IF(ILCMLN .LE. 0) CALL XABORT(NAMSBR//
     >      ': NO SIGNATURE FOUND FOR MACROLIB DATA STRUCTURE')
            CALL LCMGET(KENTRY(IEN),'SIGNATURE',KCHAR)
            WRITE(HSIGN,'(3A4)') (KCHAR(ITC),ITC=1,NTC)
            CALL XDISET(ISTATE,NSTATE,0)
            IF(HSIGN.EQ.'L_MACROLIB') THEN
              INDREC=3
              CALL LCMGET(KENTRY(IEN),'STATE-VECTOR',ISTATE)
              NGRO=ISTATE(1)
              NGT=NGRO
              MAXMIX=ISTATE(2)
              NL=ISTATE(3)
              NADDXS=ISTATE(5)
              ITRANC=ISTATE(6)
              IF(NGT .GT. 0) THEN
                CALL SETARA(RBASE,2*NGT+1,IENER)
                CALL LCMGET(KENTRY(IEN),'ENERGY',RBASE(IENER))
                CALL LCMGET(KENTRY(IEN),'DELTAU',RBASE(IENER+NGT+1))
              ENDIF
              CALL LCMSIX(IPLIB,'MACROLIB',ILCMUP)
              CALL LCMEQU(KENTRY(IEN),IPLIB)
              IF(NADDXS .NE. 0) THEN
                IF(NADDXS .GT. MAXED-NEDMAC) CALL XABORT(NAMSBR//
     >          ': TOO MANY EXTRA EDITS REQUESTED')
                CALL SETARA(IBASE,2*NADDXS,IADNAM)
                CALL LCMGET(IPLIB,'ADDXSNAME',IBASE(IADNAM))
                JED=IADNAM
                DO 110 IED=1,NADDXS
                  WRITE(HADD,'(A4,A2)') IBASE(JED),IBASE(JED+1)
                  DO 111 KED=1,NEDMAC
                    IF(HADD .EQ. HVECT(KED)) GO TO 115
 111              CONTINUE
                  NEDMAC=NEDMAC+1
                  HVECT(NEDMAC)=HADD
 115              CONTINUE
                  JED=JED+2
 110            CONTINUE
                CALL RLSARA(IBASE(IADNAM))
              ENDIF
C----
C  Update 01/01/12
C  Write energy and deltau on macrolib also
C----
              IF(NGT .GT. 0) THEN
                CALL LCMPUT(IPLIB,'ENERGY',NGT+1,2,RBASE(IENER))
                CALL LCMPUT(IPLIB,'DELTAU',NGT,2,RBASE(IENER+NGT+1))
              ENDIF
              CALL LCMSIX(IPLIB,'MACROLIB',ILCMDN)
              IF(NGT .GT. 0) THEN
                CALL LCMPUT(IPLIB,'ENERGY',NGT+1,2,RBASE(IENER))
                CALL LCMPUT(IPLIB,'DELTAU',NGT,2,RBASE(IENER+NGT+1))
                CALL RLSARA(RBASE(IENER))
              ENDIF
              GO TO 150
            ENDIF
            IF(IPMACR .EQ. 1) THEN
              CALL LCMSIX(KENTRY(IEN),'MACROLIB',ILCMDN)
            ENDIF
          ENDIF
 100    CONTINUE
      ELSE IF(JENTRY(1) .EQ. 1) THEN
        CALL LCMGET(IPLIB,'SIGNATURE',KCHAR)
        WRITE(HSIGN,'(3A4)') (KCHAR(ITC),ITC=1,NTC)
        IF(HSIGN.NE.'L_LIBRARY') THEN
          TEXT12=HENTRY(1)
          CALL XABORT(NAMSBR//
     >    ': SIGNATURE OF '//TEXT12//' IS '//HSIGN//
     >    '. L_LIBRARY EXPECTED.')
        ENDIF
        INDREC=2
        CALL XDISET(ISTATE,NSTATE,0)
        CALL LCMGET(IPLIB,'STATE-VECTOR',ISTATE)
        MAXMIX=ISTATE(1)
        NBISO=ISTATE(2)
        NGRO=ISTATE(3)
        NGT=NGRO
        NL=ISTATE(4)
        ITRANC=ISTATE(5)
        ISOADD=ISTATE(6)
        ITIME=ISTATE(7)
        NLIB=ISTATE(8)
        NGF=ISTATE(9)
        NGFR=ISTATE(10)
        NIDEPL=ISTATE(11)
        NCOMB=ISTATE(12)
        NEDMAC=ISTATE(13)
        NBMIX=ISTATE(14)
        NRES=ISTATE(15)
        MAXISM=ISTATE(16)
        CALL SETARA(RBASE,2*NEDMAC,IJCEDM)
        CALL LCMGET(IPLIB,'HVECT',RBASE(IJCEDM))
        IED=IJCEDM
        DO 120 JED=1,NEDMAC
          WRITE(HVECT(JED),'(A4,A2)') RBASE(IED),RBASE(IED+1)
          IED=IED+2
 120    CONTINUE
        CALL RLSARA(RBASE(IJCEDM))
      ENDIF
C----
C  READ LIBRARY DATA
C----
 150  CONTINUE
      CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
      IF(ITYPLU .NE. 3) CALL XABORT(NAMSBR//
     >': KEYWORD EXPECTED')
      IF(CARLIR(1:4) .EQ. 'EDIT') THEN
C---
C  READ THE PRINT INDEX
C----
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU.NE.1) CALL XABORT(NAMSBR//
     >  ': VALUE FOR IPRINT EXPECTED')
        IPRINT=INTLIR
      ELSE IF(CARLIR(1:4) .EQ. 'NGRO') THEN
C----
C  READ THE NUMBER OF ENERGY GROUPS.
C----
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 1) CALL XABORT(NAMSBR//
     >  ': VALUE FOR NGRO EXPECTED')
        IF(INDREC .EQ. 2) THEN
          IF(NGRO .NE. INTLIR) CALL XABORT(NAMSBR//
     >    ': INCOMPATIBLE VALUE OF NGRO')
        ELSE
          NGRO=INTLIR
        ENDIF
      ELSE IF(CARLIR(1:4) .EQ. 'MXIS') THEN
C----
C  CHANGE MAXIMUM NUMBER OF ISOTOPES
C----
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 1) CALL XABORT(NAMSBR//
     >  ': VALUE FOR MXIS EXPECTED')
        MAXISM=MAX(MAXISM,INTLIR)
      ELSE IF(CARLIR(1:4) .EQ. 'NMIX') THEN
C----
C  READ THE MAXIMUM NUMBER OF MATERIAL MIXTURES
C----
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 1) CALL XABORT(NAMSBR//
     >  ': VALUE FOR NMIX EXPECTED')
        MAXMIX=MAX(MAXMIX,INTLIR)
      ELSE IF(CARLIR(1:4) .EQ. 'CTRA') THEN
C----
C  READ TRANSPORT CORRECTION TYPE
C----
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 3) CALL XABORT(NAMSBR//
     >  ': CTRA TYPE EXPECTED')
        IF(CARLIR(1:4) .EQ. 'NONE') THEN
           ITRANC=0
         ELSE IF(CARLIR(1:4) .EQ. 'APOL') THEN
           ITRANC=1
         ELSE IF(CARLIR(1:4) .EQ. 'WIMS') THEN
           ITRANC=2
         ELSE IF(CARLIR(1:4) .EQ. 'OLDW') THEN
           ITRANC=3
         ELSE
           CALL XABORT(NAMSBR//
     >     ': CTRA TYPE APOL OR WIMS EXPECTED.')
         ENDIF
      ELSE IF(CARLIR(1:4) .EQ. 'ANIS') THEN
C----
C  READ THE SCATTERING ANISOTROPY FOR TRANSPORT THEORY CASES
C----
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 1) CALL XABORT(NAMSBR//
     >  ': VALUE FOR ANIS EXPECTED')
        NL=INTLIR
      ELSE IF(CARLIR(1:4) .EQ. 'PROM') THEN
        ITIME=2
      ELSE IF(CARLIR(1:7) .EQ. 'RDEPCHN') THEN
        ISOADD=1
      ELSE IF(CARLIR(1:7) .EQ. 'CDEPCHN') THEN
        ISOADD=0
      ELSE IF(CARLIR(1:4) .EQ. 'DEPL') THEN
        CALL LIBDEP(IPLIB,IPRINT,NIDEPL)
      ELSE IF(CARLIR.EQ.'ADED') THEN
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 1) CALL XABORT(NAMSBR//
     >  ': VALUE FOR ADED EXPECTED')
        NEDMA0=INTLIR
        DO 160 IED=1,NEDMA0
          CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
          IF(ITYPLU .NE. 3) CALL XABORT(NAMSBR//
     >    ': TYPE FOR ADED EXPECTED')
          DO 161 JED=1,NEDMAC
            IF(CARLIR(1:6) .EQ. HVECT(JED)) GO TO 160
 161      CONTINUE
          NEDMAC=NEDMAC+1
          IF(NEDMAC .GT. MAXED) CALL XABORT(NAMSBR//
     >    ': TOO MANY EXTRA EDITS REQUESTED')
          HVECT(NEDMAC)=CARLIR(1:6)
 160    CONTINUE
      ELSE IF(CARLIR(1:4) .EQ. 'MIXS') THEN
         ITPROC=1
         GO TO 200
      ELSE IF(CARLIR(1:4) .EQ. 'MAXS') THEN
         ITPROC=2
         IF(INDREC .NE. 2)  CALL XABORT(NAMSBR//
     >   ': MAXS CAN ONLY BE USE TO UPDATE '//
     >   'A LIBRARY - IT CANNOT CREATE A NEW LIBRARY')
C----
C  TRY TO FIND A SECOND READ-ONLY LIBRARY TO MODIFY
C  ORIGINAL LIBRARY
C----
         DO 170 IEN=2,NENTRY
           IF(IENTRY(IEN) .LE. 2 .AND.
     >        JENTRY(IEN) .EQ.2 ) THEN
             CALL LCMGET(KENTRY(IEN),'SIGNATURE',KCHAR)
             WRITE(HSIGN,'(3A4)') (KCHAR(ITC),ITC=1,NTC)
             IF(HSIGN.EQ.'L_LIBRARY') THEN
               IPLIBX=KENTRY(IEN)
               CALL XDISET(ISTATE,NSTATE,0)
               CALL LCMGET(IPLIBX,'STATE-VECTOR',ISTATE)
               NBMIXX=ISTATE(1)
               NBISOX=ISTATE(2)
             ENDIF
           ENDIF
 170    CONTINUE
        TMPDAY(1)=0.0
        TMPDAY(2)=0.0
        TMPDAY(3)=0.0
        CALL LCMLEN(IPLIB,'MACROLIB',ILCMLN,ILCMTY)
        IF(ILCMLN .EQ. -1) THEN
          CALL LCMSIX(IPLIB,'MACROLIB',ILCMUP)
          CALL LCMLEN(IPLIB,'TIMESTAMP',ILCMLN,ILCMTY)
          IF(ILCMLN .GT. 0 .AND. ILCMLN .LE. 3) THEN
            CALL LCMGET(IPLIB,'TIMESTAMP',TMPDAY)
          ENDIF
          CALL LCMSIX(IPLIB,'MACROLIB',ILCMDN)
        ENDIF
        GO TO 200
      ELSE IF(CARLIR(1:4) .EQ. 'BURN') THEN
        ITPROC=2
        IF(INDREC .NE. 2) CALL XABORT(NAMSBR//
     >  ': MAXS CAN ONLY BE USE TO UPDATE '//
     >  'A LIBRARY - IT CANNOT CREATE A NEW LIBRARY')
        DO 180 IEN=2,NENTRY
          IF(IENTRY(IEN) .LE. 2 .AND.
     >       JENTRY(IEN) .EQ. 2) THEN
            CALL LCMGET(KENTRY(IEN),'SIGNATURE',KCHAR)
            WRITE(HSIGN,'(3A4)') (KCHAR(ITC),ITC=1,NTC)
            IF(HSIGN.EQ.'L_BURNUP') THEN
              IPLIBX=KENTRY(IEN)
              CALL LCMLEN(IPLIBX,'DEPL-TIMES  ',NDEPL,ILCMTY)
              GO TO 185
            ENDIF
          ENDIF
 180    CONTINUE
        CALL XABORT(NAMSBR//': BURNUP FILE MISSING')
 185    CONTINUE
        CALL SETARA(RBASE,NDEPL,IBSTD)
        CALL LCMGET(IPLIBX,'DEPL-TIMES  ',RBASE(IBSTD))
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .EQ. 3) CALL XABORT(NAMSBR//
     >  ': INVALID BURNUP STEP')
        IF(ITYPLU .EQ. 2) THEN
          TMPDAY(1)=REALIR
          TIMBRN=0.000864*TMPDAY(1) 
          IF(TIMBRN .LE. 0.0) THEN  
            IBSTEP=1
            TMPDAY(1)=0.0
          ELSE
            IBSTEP=1
            IKSTEP=0
            DO 190 IDP=1,NDEPL
              DELT=ABS(TIMBRN-RBASE(IBSTD+IDP-1))
              IF(DELT .LT. 1.0E-8) THEN
                IBSTEP=IDP
                GO TO 195
              ELSE IF(TIMBRN .GT. RBASE(IBSTD+IDP-1)) THEN
                IKSTEP=IDP  
              ENDIF
 190        CONTINUE
            WRITE(IOUT,9000) TMPDAY
            WRITE(IOUT,9001) (RBASE(IBSTD+IDP-1)/0.000864,IDP=1,NDEPL)
            IBSTEP=MIN(IKSTEP+1,NDEPL)
            WRITE(IOUT,9002) RBASE(IBSTD+IBSTEP-1)/0.000864
 195        CONTINUE
          ENDIF
        ELSE
          IBSTEP=INTLIR
          IF(IBSTEP .LE. 0 ) THEN 
            WRITE(IOUT,9010) 
            IBSTEP=1
            WRITE(IOUT,9010) RBASE(IBSTD+IBSTEP-1)/0.000864
          ELSE IF(IBSTEP .GT. NDEPL) THEN 
            IBSTEP=NDEPL 
            WRITE(IOUT,9011) RBASE(IBSTD+IBSTEP-1)/0.000864
          ENDIF
          TMPDAY(1)=RBASE(IBSTD+IBSTEP-1)/0.000864
        ENDIF
        CALL RLSARA(RBASE(IBSTD)) 
        TMPDAY(2)=0.0
        TMPDAY(3)=0.0
        NBISOX=-IBSTEP
        IF(IPRINT .GE. 1) THEN
          WRITE(IOUT,6000) IBSTEP,TMPDAY(1)
        ENDIF
        GO TO 200
      ELSE
        CALL XABORT(NAMSBR//': '//CARLIR//' IS AN INVALID KEY-WORD.')
      ENDIF
      GO TO 150
*
* PROCESS THE LIB: MODULE INPUT DATA.
 200  CONTINUE
      IF(MAXMIX .EQ. 0) CALL XABORT(NAMSBR//
     >': MAXMIX NOT YET DEFINED.')
      MAXISO=MAXISM*MAXMIX
C----
C  ALLOCATE
C----
      CALL SETARA(IBASE,NTC*MAXISO,JJNAM)
      CALL SETARA(IBASE,NTC*MAXISO,JJNRF)
      CALL SETARA(IBASE,MAXISO,JJMIX)
      CALL SETARA(RBASE,MAXISO,JJDEN)
      CALL SETARA(RBASE,MAXMIX,JDENMX)
      CALL SETARA(IBASE,MAXMIX,LMASKM)
C----
C  INITIALIZE
C----
      TEXT4='    '
      READ(TEXT4,'(A4)') ITEXT4
      CALL XDISET(IBASE(JJNAM),NTC*MAXISO,ITEXT4)
      CALL XDISET(IBASE(JJNRF),NTC*MAXISO,ITEXT4)
      CALL XDISET(IBASE(JJMIX),MAXISO,0)
      CALL XDRSET(RBASE(JJDEN),MAXISO,0.0)
      CALL XDRSET(RBASE(JDENMX),MAXMIX,-1.0)
      IF(ITPROC .EQ. 1) THEN
C----
C  ALLOCATE
C----
        CALL SETARA(RBASE,MAXISO,JJTN)
        CALL SETARA(RBASE,MAXISO,JJAWR)
        CALL SETARA(RBASE,MAXISO,ISNGAR)
        CALL SETARA(IBASE,MAXISO,JJNTFG)
        CALL SETARA(IBASE,MAXISO,ILSHI)
        CALL SETARA(IBASE,MAXISO,JMASKI)
        CALL SETARA(RBASE,MAXMIX,JTMPMX)
        CALL SETARA(IBASE,(NTC-1)*MAXISO*5,JHLIB)
C----
C  INITIALIZE
C----
        CALL XDRSET(RBASE(JJTN),MAXISO,0.0)
        CALL XDRSET(RBASE(JJAWR),MAXISO,0.0)
        CALL XDRSET(RBASE(ISNGAR),MAXISO,-1.0E10)
        CALL XDISET(IBASE(JJNTFG),MAXISO,0)
        CALL XDISET(IBASE(ILSHI),MAXISO,0)
        CALL XDRSET(RBASE(JTMPMX),MAXMIX,-1.0)
        CALL XDISET(IBASE(JHLIB),2*MAXISO*5,ITEXT4)
        CALL LIBINP(IPLIB ,INDREC,IPRINT,MAXMIX,MAXED ,MAXISO,
     >              NBISO ,NGRO  ,NGT   ,NL    ,ITRANC,ITIME ,
     >              NLIB  ,NIDEPL,NCOMB ,NEDMAC,NBMIX ,NRES  ,
     >              NGF   ,NGFR  ,ISOADD,HVECT ,
     >              IBASE(JJNAM) ,IBASE(JJNRF) ,IBASE(JJMIX) ,
     >              RBASE(JJDEN) ,RBASE(JJTN)  ,RBASE(JJAWR) ,
     >              IBASE(JJNTFG),IBASE(ILSHI) ,RBASE(ISNGAR),
     >              IBASE(JMASKI),IBASE(LMASKM),RBASE(JDENMX),
     >              RBASE(JTMPMX),IBASE(JHLIB) )
C----
C  RELEASE
C----
        CALL RLSARA(IBASE(JHLIB))
        CALL RLSARA(RBASE(JTMPMX))
        CALL RLSARA(IBASE(JMASKI))
        CALL RLSARA(IBASE(ILSHI))
        CALL RLSARA(IBASE(JJNTFG))
        CALL RLSARA(RBASE(ISNGAR))
        CALL RLSARA(RBASE(JJAWR))
        CALL RLSARA(RBASE(JJTN))
      ELSE IF(ITPROC .GE. 2) THEN
C----
C  ALLOCATE
C----
        IF(NGRO .EQ. 0) CALL XABORT(NAMSBR//
     >  ': NUMBER OF GROUP REQUIRED FOR MAXS OF BURN')
        CALL SETARA(IBASE,MAXISO,JIMIX)
        CALL SETARA(RBASE,MAXISO,JDENIX)
        CALL SETARA(IBASE,MAXMIX,LOCUPD)
        CALL SETARA(IBASE,MAXISO,LISM)
        CALL SETARA(RBASE,MAXISO,JDENMD)
        CALL SETARA(IBASE,NGRO,LMASKG)
        CALL LIBMAC(IPLIB ,IPLIBX,IPRINT,MAXISO,NBISO ,NBISOX,
     >              NBMIX ,NBMIXX,NGRO  ,TMPDAY,
     >              IBASE(JJNAM) ,IBASE(JJNRF) ,IBASE(JJMIX) ,
     >              RBASE(JJDEN) ,RBASE(JDENMX),IBASE(JIMIX) ,
     >              RBASE(JDENIX),IBASE(LOCUPD),IBASE(LISM)  ,
     >              RBASE(JDENMD),IBASE(LMASKM),IBASE(LMASKG))
C----
C RELEASE
C----
        CALL RLSARA(IBASE(LMASKG))
        CALL RLSARA(RBASE(JDENMD))
        CALL RLSARA(IBASE(LISM))
        CALL RLSARA(IBASE(LOCUPD))
        CALL RLSARA(RBASE(JDENIX))
        CALL RLSARA(IBASE(JIMIX))
      ENDIF
C----
C RELEASE
C----
      CALL RLSARA(IBASE(LMASKM))
      CALL RLSARA(RBASE(JDENMX))
      CALL RLSARA(RBASE(JJDEN))
      CALL RLSARA(IBASE(JJMIX))
      CALL RLSARA(IBASE(JJNRF))
      CALL RLSARA(IBASE(JJNAM))
      IF(IPRINT .GE. 5) CALL LCMLIB(IPLIB)
      RETURN
C----
C  FORMATS
C----
 6000 FORMAT(' LIBRARY UPDATE AT BURNUP STEP : ',I5,
     >       ' BURNUP TIME = ',F20.7,' DAYS') 
 9000 FORMAT(' **** WARNING  *****'/
     >       ' INVALID BURNUP TIME =',F20.7,' DAYS'/
     >       ' BURNUP TABULATION (DAYS) ')
 9001 FORMAT(6F20.7)
 9002 FORMAT(' BURNUP STEP SELECTED =',F20.7,' DAYS')
 9010 FORMAT(' **** WARNING  *****'/
     >       ' BURNUP STEP NEGATIVE '/
     >       ' USE FIRST BURNUP STEP AT ',F20.7,' DAYS')
 9011 FORMAT(' **** WARNING  *****'/
     >       ' BURNUP STEP TOO LARGE '/
     >       ' USE LAST BURNUP STEP AT ',F20.7,' DAYS')
      END
