*DECK LIBWD4
      SUBROUTINE LIBWD4(IPLIB,IPRINT,CFILNA,NGROUP,NBISO,NL,ISONAM,
     >                  ISONRF,ISHINA,TN,AW,SN,SB,ILIB,ITISOM,DELTA,
     >                  ITRANC,NPROC,INDPRO,ITYPRO,XSREC,SCAT,XSSCMP,
     >                  NJJ,IJJ,XSOUT,ISORD)
C
C------------------------------  LIBWD4  ------------------------------
C
C  PROGRAMME STATISTICS:
C     NAME     : LIBWD4
C     ENTRY    : LIBWD4
C     USE      : TRANSCRIPTION OF THE INTERPOLATED MICROSCOPIC XS READ
C                FROM A MICROSCOPIC XS LIBRARY IN WIMS-D4 FORMAT
C                TO LCM DRAGON FORMAT
C     MODIFIED : 97-01-30
C                98-05-05 (G.M.) SEE BELOW
C     AUTHOR   : G. MARLEAU
C
C  ROUTINE PARAMETERS:
C   INPUT
C     IPLIB  : POINTER TO THE INTERNAL LIBRARY      I
C     IPRINT : PRINT FLAG                           I
C     CFILNA : WIMS FILE NAME                       C*8
C     NGROUP : NUMBER OF GROUPS                     I
C     NBISO  : NUMBER OF ISOTOPES                   I
C     NL     : NUMBER OF LEGENDRE SCATTERING ORDER  I
C              =1 -> ISOTROPIC
C              =2 -> LINEARLY ANISOTROPIC
C              ETC.
C     ISONAM : LOCAL ISOTOPE NAMES                  I(3,NBISO)
C     ISONRF : REFERENCE ISOTOPE NAMES              I(3,NBISO)
C     ISHINA : SHIELDING ISOTOPE NAMES              I(2,NBISO)
C     TN     : ISOTOPE TEMPTERATURE                 R(NBISO)
C     AW     : ISOTOPE ATOMIC WEIGHT                R(NBISO)
C     SN     : DILUTION XS                          R(NGROUP,NBISO)
C     SB     : L-J DILUTION XS                      R(NGROUP,NBISO)
C     ILIB   : LIBRARY NUMBER TO PROCESS            I
C     ITISOM : FLAG ISOTOPE PROCESSING              I(NBISO)
C     DELTA  : LETHARGY                             R(NGROUP)
C     ITRANC : TRANSPORT CORRECTION                 I
C              0 -> NONE ; 1 -> APOL; 2 -> WIMS
C              3 -> OLDWIMS
C     NPROC  : NUMBER OF POSSIBLE XS TO PROCESS     I
C     INDPRO : CROSS SECTION TO PROCESS             I(NPROC)
C     ITYPRO : CROSS SECTION PROCESSED              I(NPROC)
C   WORK
C     XSREC  : GENERAL XS VECTOR                    R(NGROUP,NPROC)
C     SCAT   : COMPLETE SCATTERING MATRIX           R(NGROUP,
C              SCAT(JG,IG) IS FROM IG TO JG         NGROUP,NL)
C     XSSCMP : COMPRESS SCATTERING FOR TRANSFER     R(NGROUP*
C                                                    (NGROUP+2))
C     NJJ    : NB SCATTERING GROUP                  I(NGROUP)
C     IJJ    : MAXIMUM SCATTERING GROUP             I(NGROUP)
C     XSOUT  : SELF SHIELDING PARAMETER             R(NGROUP,7)
C     ISORD  : LOCAL ISOTOPE FLAG                   I(NBISO)
C   INTERNAL PARAMETERS
C     IOUT   : OUTPUT UNIT = 6                      I
C     MAXTEM : MAX. NB. OF TMPT = 20                I
C     MAXDIL : MAX. NB. OF DIL = 50                 I
C     NOTX   : INTERPOLATION METHOD = -1            I
C     CONVM  : CONVERSION FACTOR FOR ATOMIC MASS
C              FROM AMU TO NEUTRON MASS = 0.9908751 R
C   WIMS-D4 LIBRARY PARAMETERS
C     IUTYPE : TYPE OF FILE = 2 (BINARY)            I
C     LRIND  : LENGHT RECORD ON DA FILE = 0         I
C     IACTO  : OPEN ACTION = 2 (READ ONLY)          I
C     IACTC  : CLOSE ACTION = 2 (KEEP)              I
C     MAXISO : MAX. NB. ISOTOPE = 246               I
C     LPZ    : LENGTH OF WIMS PARAMETER ARRAY = 8   I
C     NSETP1 : NUMBER OF P1 SCATTERING SETS = 4     I
C     NPZ    : LIST OF MAIN PARAMETERS              I(LPZ)
C     IWISO  : ID OF ISOTOPE                        I(MAXISO)
C     ID     : ISOTOPIC ID                          I
C     IZ     : ISOTOPIC CHARGE                      I(MAXISO)
C     NF     : NUMBER FISSION                       I(MAXISO)
C     NT     : NUMBER TEMPERATURE                   I(MAXISO)
C     NR     : NUMBER RESONANCE                     I(MAXISO)
C     AWR    : ATOMIC MASS (AMU)                    R(MAXISO)
C
C------------------------------  LIBWD4  ------------------------------
C
      IMPLICIT    NONE
      INTEGER     IOUT,NDPROC,ITLIB,MAXTEM,MAXDIL,NOTX
      REAL        CONVM
      PARAMETER  (IOUT=6,NDPROC=20,ITLIB=2,MAXTEM=20,MAXDIL=20,
     >            NOTX=-1,CONVM=0.9908751)
      INTEGER     IPLIB,IPRINT,NGROUP,NBISO,NL,ILIB,ITRANC,NPROC
      CHARACTER   CFILNA*8
      INTEGER     ISONAM(3,NBISO),ISONRF(3,NBISO),
     >            ISHINA(2,NBISO),ITISOM(NBISO),
     >            INDPRO(NPROC),ITYPRO(NPROC),
     >            NJJ(NGROUP),IJJ(NGROUP),ISORD(NBISO)
      REAL        TN(NBISO),AW(NBISO),SN(NGROUP,NBISO),
     >            SB(NGROUP,NBISO),DELTA(NGROUP),XSREC(NGROUP,NPROC),
     >            SCAT(NGROUP,NGROUP,NL),XSSCMP(NGROUP*(NGROUP+2)),
     >            XSOUT(NGROUP,7)
C----
C  INTERNAL PARAMETERS
C----
      CHARACTER   HNAMIS*12,HSHIR*8
      REAL        TMPT(MAXTEM),DILT(MAXDIL),REST(MAXDIL*MAXTEM),
     >            XSCOR(4)
      DOUBLE PRECISION TERP(MAXTEM)
      INTEGER     IGRF,IGF,ITYP,IGR,NTYP,IXRES,IRES,NTMPR,
     >            NBAC,JG,KDRCLS,IERR,ISETP1,ITT,NDILR,IG1,IGRL,ITC,
     >            JSO,IENDF,J,IEL,NRTOT,IELRT,JC,
     >            MXSCT,IUNIT,KDROPN,IHANDL,IP0,IP1,
     >            NGR,NGTHER,NGFR,II,NEL,NGF,JEL,IDRES,ILCR,NSRES,
     >            IACT,NSCT,IT,IG,NFIS,NISOR,NRDT,JJJ,ITXS,
     >            ILOCX,ILOCY,ILOCS
      REAL        XIND,XRS1,A,ENDR,RIND,XX
C----
C  MEMORY ALLOCATION PARAMETERS
C----
      INTEGER     IBASE(1)
      REAL        RBASE
      COMMON      RBASE(1)
      EQUIVALENCE(RBASE(1),IBASE(1))
      INTEGER     ITMPXS,ITMPSC,IRID,INTM,INDI,IRTMP,IRDIL,IRESI,
     >            JRRI,JRIT
C----
C  WIMS-D4 LIBRARY PARAMETERS
C----
      INTEGER      IUTYPE,LRIND,IACTO,IACTC,MAXISO,LPZ,NSETP1
      PARAMETER   (IUTYPE=2,LRIND=0,IACTO=2,IACTC=1,MAXISO=246,
     >             LPZ=8,NSETP1=4)
      CHARACTER    CWISO(MAXISO)*8,FMT*6
      INTEGER      NPZ(LPZ),IWISO(MAXISO),IDIEL,IZ,NFIEL,
     >             NF(MAXISO),NTMP,NRIEL,NR(MAXISO),IDTEMP(2),
     >             NBATOM(NSETP1)
      REAL         AWR,DSIGPL(MAXISO)
      SAVE         NBATOM
      DATA         NBATOM
     >            /1,2,12,16/
C----
C  OPEN WIMS-D4 LIBRARY
C  READ GENERAL DIMENSIONING
C----
      IP0=NDPROC+1
      IP1=NDPROC+2
      IUNIT=KDROPN(CFILNA,IACTO,IUTYPE,LRIND,IHANDL)
      IF(IUNIT.LE.0) CALL XABORT('LIBWD4: WIMS-D4 LIBRARY '//
     >    CFILNA//' CANNOT BE OPENED FOR MIXS')
      IF(IPRINT.GE.5) THEN
        WRITE(IOUT,6000) CFILNA
      ENDIF
      READ(IUNIT) (NPZ(II),II=1,LPZ)
      IF(NPZ(2).NE.NGROUP) THEN
        WRITE(IOUT,9001) NGROUP,NPZ(2)
        CALL XABORT('LIBWD4: INVALID NUMBER OF GROUPS')
      ENDIF
      NEL=NPZ(1)
      NGF=NPZ(4)
      NGR=NPZ(5)
      NGTHER=NPZ(6)
      NGFR=NGF+NGR
      MXSCT=NGROUP*(NGROUP+2)
      IF(NGFR+NGTHER.NE.NGROUP) THEN
        WRITE(IOUT,9001) NGROUP,NGFR+NGTHER
        CALL XABORT('LIBWD4: INVALID NUMBER OF GROUPS')
      ENDIF
      IF(NEL.GT.MAXISO) THEN
        WRITE(IOUT,9002) MAXISO,NEL
        CALL XABORT('LIBWD4: INVALID NUMBER OF ISOTOPES')
      ENDIF
      IENDF=0
C----
C  READ ISOTOPE ID NUMBER AND CREATE EQUIVALENT ISOTOPE NAME
C  SCAN TO ASSOCIATE WIMS ISOTOPE NUMBER WITH DRAGON ISOTOPE NUMBER
C  VERIFY IF ALL ISOTOPES REQUIRED ARE PRESENT
C----
      READ(IUNIT) (IWISO(J),J=1,NEL)
      CALL XDISET(ISORD,NBISO,0)
      DO 100 IEL=1,NEL
        CWISO(IEL)='        '
        IF     (IWISO(IEL).LT.10) THEN
          WRITE(CWISO(IEL),'(I1)') IWISO(IEL)
        ELSE IF(IWISO(IEL).LT.100) THEN
          WRITE(CWISO(IEL),'(I2)') IWISO(IEL)
        ELSE IF(IWISO(IEL).LT.1000) THEN
          WRITE(CWISO(IEL),'(I3)') IWISO(IEL)
        ELSE IF(IWISO(IEL).LT.10000) THEN
          WRITE(CWISO(IEL),'(I4)') IWISO(IEL)
        ELSE IF(IWISO(IEL).LT.100000) THEN
          WRITE(CWISO(IEL),'(I5)') IWISO(IEL)
        ELSE IF(IWISO(IEL).LT.1000000) THEN
          WRITE(CWISO(IEL),'(I6)') IWISO(IEL)
        ELSE IF(IWISO(IEL).LT.10000000) THEN
          WRITE(CWISO(IEL),'(I7)') IWISO(IEL)
        ELSE IF(IWISO(IEL).LT.100000000) THEN
          WRITE(CWISO(IEL),'(I8)') IWISO(IEL)
        ENDIF
        READ(CWISO(IEL),'(2A4)') (IDTEMP(ITC),ITC=1,2)
        DO 101 JSO=1,NBISO
          IF(ITISOM(JSO).EQ.ILIB) THEN
            IF(ISONRF(1,JSO).EQ.IDTEMP(1).AND.
     >         ISONRF(2,JSO).EQ.IDTEMP(2)) ISORD(JSO)=IEL
          ENDIF
 101    CONTINUE
 100  CONTINUE
      DO 102 JSO=1,NBISO
        IF(ITISOM(JSO).EQ.ILIB.AND.
     >     ISORD(JSO).EQ.0) THEN
          WRITE(IOUT,9003) (ISONRF(ITC,JSO),ITC=1,3),CFILNA
          CALL XABORT('LIBWD4: MISSING ISOTOPE')
        ENDIF
 102  CONTINUE
C----
C  SKIP GROUP STRUCTURE
C----
      READ(IUNIT) (A,J=1,NGROUP+1)
C----
C  RECOVER FISSION SPECTRUM
C  INITIALIZE FAST AND THERMAL GROUPS XPO AND GOLD
C  ONLY RESONANCE GROUPS FOR THESE CROSS SECTIONS
C  INITIALIZE XN2N
C  NO SUCH REACTION FOR THIS LIBRARY
C----
      CALL XDRSET(XSREC(1,5),NGROUP,0.0)
      READ(IUNIT) (XSREC(J,5),J=1,NPZ(3))
C----
C  READ DEPLETION CHAIN
C----
      DO 120 IEL=1,NEL
        READ(IUNIT) JC
 120  CONTINUE
      READ(IUNIT) ENDR
C----
C  ALLOCATE MEMORY FOR TEMPERATURE DEPENDENT XS
C  AND FOR RESONANCE CALCULATION
C----
      CALL SETARA(RBASE,NGROUP*5*MAXTEM,ITMPXS)
      CALL SETARA(RBASE,NGROUP*NGROUP*MAXTEM,ITMPSC)
C----
C READ FILE
C CROSS SECTION ARE SAVED ONLY IF ISOTOPE IS USED
C----
      NRTOT=0
      DO 130 IELRT=1,NEL
        READ(IUNIT) IDIEL,AWR,IZ,NFIEL,NTMP,NRIEL
        IF(NRIEL.GT.0) THEN
          NRTOT=NRTOT+NRIEL
        ENDIF
        IF(NTMP.GT.MAXTEM) THEN
          CALL XABORT('LIBWD4: INVALID MAXTEM FOR P0.')
        ENDIF
C----
C  LOCATE ISOTOPE IN LIST OF LIBRARY ISOTOPES IN THE CASE
C  WHERE LIBRARY IS NOT COMPLETE OR THE ORDER OF ISOTOPE
C  STORED IS DIFFERENT FROM THAT OF THE ISOTOPE NAMES
C----
        IEL=0
        DO 140 JEL=1,NEL
          IF(IDIEL.EQ.IWISO(JEL)) THEN
            IEL=JEL
            NF(IEL)=NFIEL
            NFIS=0
            IF(NF(IEL).GT.1) NFIS=1
            NR(IEL)=NRIEL
            GO TO 145
          ENDIF
 140    CONTINUE
        CALL XABORT('LIBWD4: WIMSD4 LIBRARY INCOMPLETE')
 145    CONTINUE
        NISOR=0
C----
C  SCAN TO SEE IF ISOTOPE IS REQUIRED
C----
        DO 150 JSO=1,NBISO
          IF(ITISOM(JSO).EQ.ILIB.AND.
     >       ISORD(JSO).EQ.IEL) THEN
             NISOR=1
             GO TO 155
          ENDIF
 150    CONTINUE
 155    CONTINUE
        IF(NISOR.EQ.0) THEN
C----
C  ISOTOPE NOT REQUIRED/SKIP RECORDS
C----
          READ(IUNIT) XX
          IF(NF(IEL).GT.1) THEN
            READ(IUNIT) XX
          ENDIF
          READ(IUNIT) NSCT
          IF(NTMP.GT.0) THEN
            READ(IUNIT) XX
            DO 160 IT=1,NTMP
              READ(IUNIT) XX
              IF(NF(IEL).GT.1) THEN
                READ(IUNIT) XX
              ENDIF
              READ(IUNIT) NSCT
 160        CONTINUE
          ENDIF
        ELSE
C----
C  ISOTOPE REQUIRED READ FAST AND/OR RESONANCE XS
C----
          CALL XDRSET(XSREC(1,1),NGROUP*4,0.0)
          CALL XDRSET(XSREC(1,6),NGROUP*(NPROC-5),0.0)
          READ(IUNIT) (XSREC(NGF+II,15),II=1,NGR),
     >                (XX,II=1,NGR),
     >                (XSREC(II,19),II=1,NGFR),
     >                (XSREC(II,20),II=1,NGFR),
     >                (XX,II=1,NGR),
     >                (XSREC(NGF+II,14),II=1,NGR)
          DSIGPL(IEL)=0.0
          DO 180 IG=NGF+1,NGFR
            DSIGPL(IEL)=DSIGPL(IEL)+XSREC(IG,15)*XSREC(IG,14)
 180      CONTINUE
          DSIGPL(IEL)=DSIGPL(IEL)/FLOAT(NGR)
          IF(NF(IEL).GT.1) THEN
            READ(IUNIT) (XSREC(II,3),II=1,NGFR),
     >                  (XSREC(II,4),II=1,NGFR)
          ENDIF
C----
C  READ AND DECOMPRESS P0 SCATTERING CROSS SECTIONS
C  COMPUTE P0 SCATTERING OUT OF GROUP
C----
          READ(IUNIT) NSCT,(XSSCMP(II),II=1,NSCT)
          CALL LIBWSC(NGROUP,1,NGFR,NSCT,XSSCMP,SCAT,XSREC(1,IP0))
C----
C  THERMAL XS
C----
          IF(NTMP.EQ.1) THEN
            READ(IUNIT) XX
            READ(IUNIT) (XSREC(NGFR+II,19),II=1,NGTHER),
     >                  (XSREC(NGFR+II,20),II=1,NGTHER)
            IF(NF(IEL).GT.1) THEN
              READ(IUNIT) (XSREC(NGFR+II,3),II=1,NGTHER),
     >                    (XSREC(NGFR+II,4),II=1,NGTHER)
            ENDIF
            READ(IUNIT) NSCT,(XSSCMP(II),II=1,NSCT)
C----
C  READ AND DECOMPRESS P0 SCATTERING CROSS SECTIONS
C  COMPUTE P0 SCATTERING OUT OF GROUP
C----
            CALL LIBWSC(NGROUP,NGFR+1,NGROUP,NSCT,XSSCMP,
     >                  SCAT,XSREC(1,IP0))
C----
C  SAVE INFORMATION FOR ISOTOPES
C  NO SELF SHIELDING
C----
            DO 200 JSO=1,NBISO
              IF(ITISOM(JSO).EQ.ILIB.AND.
     >          ISORD(JSO).EQ.IEL) THEN
                WRITE(HNAMIS,'(3A4)') (ISONAM(ITC,JSO),ITC=1,3)
                IF(IPRINT.GE.5) THEN
                  WRITE(IOUT,6001) HNAMIS
                  IF(IPRINT.GE.100) THEN
                    WRITE(IOUT,6200) TN(JSO)
                  ENDIF
                ENDIF
                AW(JSO)=CONVM*AWR
C----
C  BUILT TOTAL CROSS SECTION FROM INFORMATION IN XSNG WHICH IS
C  CURRENTLY ABSORPTION AND SIGS WHICH IS TOTAL SCATTERING
C  OUT OF GROUP
C  COMPUTE REAL NG CROSS SECTION WHICH IS
C  CURRENT NG (ABSORPTION)-FISSION-N2N
C----
                DO 201 IG=1,NGROUP
                  XSREC(IG,1)=XSREC(IG,20)+XSREC(IG,IP0)
                  IF(NF(IEL).GT.1) THEN
                    XSREC(IG,7)=XSREC(IG,20)-XSREC(IG,9)-XSREC(IG,4)
                  ELSE
                    XSREC(IG,7)=XSREC(IG,20)-XSREC(IG,9)
                  ENDIF
                  IF(XSREC(IG,4).NE.0) THEN
                    XSREC(IG,6)=XSREC(IG,3)/XSREC(IG,4)
                  ELSE
                    XSREC(IG,6)=0.0
                  ENDIF
 201            CONTINUE
                IF(IENDF.LT.2) THEN
C----
C  COMPUTE TRANSPORT CORRECTION AND STORE IN TRAN
C----
                  DO 202 IG=1,NGROUP
                    XSREC(IG,2)=XSREC(IG,1)-XSREC(IG,19)
 202              CONTINUE
                ELSE
                  DO 203 IG=1,NGROUP
                    XSREC(IG,2)=XSREC(IG,19)
 203              CONTINUE
                ENDIF
C----
C  SAVE ISOTOPE INFORMATION
C----
                CALL LCMSIX(IPLIB,HNAMIS,1)
                IF(NF(IEL).LE.1) THEN
                  INDPRO(5)=0
                  CALL XDRLGS(IPLIB,1,NPROC,0,0,1,NGROUP,INDPRO,
     >                        ITYPRO,XSREC,SCAT,XSSCMP,NJJ,IJJ)
                  INDPRO(5)=1
                ELSE
                  CALL XDRLGS(IPLIB,1,NPROC,0,0,1,NGROUP,INDPRO,
     >                        ITYPRO,XSREC,SCAT,XSSCMP,NJJ,IJJ)
                ENDIF
                IF(IPRINT.GE.100) THEN
                  CALL XDRLPR(NPROC,0,0,1,NGROUP,INDPRO,ITYPRO,
     >                        XSREC,SCAT)
                ENDIF
                CALL LCMSIX(IPLIB,HNAMIS,2)
              ENDIF
 200        CONTINUE
          ELSE IF(NTMP.GT.1) THEN
C----
C  READ TEMPERATURE DEPENDENT XS
C----
            READ(IUNIT) (TMPT(II),II=1,NTMP)
            ILOCX=ITMPXS
            ILOCY=ITMPXS+NGFR
            ILOCS=ITMPSC
            NRDT=NGTHER-1
            DO 210 IT=1,NTMP
              READ(IUNIT) (RBASE(ILOCY+II),II=0,NRDT),
     >                    (RBASE(ILOCY+II+NGROUP),II=0,NRDT)
              IF(NF(IEL).GT.1) THEN
                READ(IUNIT) (RBASE(ILOCY+II+2*NGROUP),II=0,NRDT),
     >                      (RBASE(ILOCY+II+3*NGROUP),II=0,NRDT)
              ENDIF
              READ(IUNIT) NSCT,(XSSCMP(II),II=1,NSCT)
C----
C  READ AND DECOMPRESS P0 SCATTERING CROSS SECTIONS
C  COMPUTE P0 SCATTERING OUT OF GROUP
C  COMPUTE TOTAL XS
C----
              CALL LIBWSC(NGROUP,NGFR+1,NGROUP,NSCT,XSSCMP,
     >                    RBASE(ILOCS),RBASE(ILOCX+4*NGROUP))
              ILOCX=ILOCX+5*NGROUP
              ILOCY=ILOCY+5*NGROUP
              ILOCS=ILOCS+NGROUP*NGROUP
 210        CONTINUE
C----
C  SAVE INFORMATION FOR ISOTOPES
C  NO SELF SHIELDING
C----
            DO 220 JSO=1,NBISO
              IF(ITISOM(JSO).EQ.ILIB.AND.
     >          ISORD(JSO).EQ.IEL) THEN
                WRITE(HNAMIS,'(3A4)') (ISONAM(ITC,JSO),ITC=1,3)
                IF(IPRINT.GE.5) THEN
                  WRITE(IOUT,6001) HNAMIS
                ENDIF
                AW(JSO)=CONVM*AWR
C----
C  FIND TEMPERATURE INTERPOLATION COEFFICIENTS
C  INTERPOLATE IN TEMPERATURE
C----
                CALL LIBLEX(NTMP,TN(JSO),TMPT,NOTX,TERP)
                IF(IPRINT.GE.100) THEN
                  WRITE(IOUT,6201) TN(JSO)
                  WRITE(IOUT,6202) (TMPT(JJJ),JJJ=1,NTMP)
                  WRITE(IOUT,6203) (TERP(JJJ),JJJ=1,NTMP)
                ENDIF
                ITXS=1
                IACT=1
                CALL LIBWTE(IACT,ITXS,NGROUP,NGTHER,NTMP,NF(IEL),
     >                      TERP,SCAT,XSREC(1,IP0),XSREC(1,20),
     >                      XSREC(1,3),XSREC(1,4),XSREC(1,19),
     >                      RBASE(ITMPXS),RBASE(ITMPSC))
C----
C  BUILT TOTAL CROSS SECTION FROM INFORMATION IN XSNG WHICH IS
C  CURRENTLY ABSORPTION AND SIGS WHICH IS TOTAL SCATTERING
C  OUT OF GROUP
C  COMPUTE REAL NG CROSS SECTION WHICH IS
C  CURRENT NG (ABSORPTION)-FISSION-N2N
C----
                DO 221 IG=1,NGROUP
                  XSREC(IG,1)=XSREC(IG,20)+XSREC(IG,IP0)
                  IF(NF(IEL).GT.1) THEN
                    XSREC(IG,7)=XSREC(IG,20)-XSREC(IG,9)-XSREC(IG,4)
                  ELSE
                    XSREC(IG,7)=XSREC(IG,20)-XSREC(IG,9)
                  ENDIF
                  IF(XSREC(IG,4).NE.0) THEN
                    XSREC(IG,6)=XSREC(IG,3)/XSREC(IG,4)
                  ELSE
                    XSREC(IG,6)=0.0
                  ENDIF
 221            CONTINUE
                IF(IENDF.LT.2) THEN
C----
C  COMPUTE TRANSPORT CORRECTION AND STORE IN TRAN
C----
                  DO 222 IG=1,NGROUP
                    XSREC(IG,2)=XSREC(IG,1)-XSREC(IG,19)
 222              CONTINUE
                ELSE
                  DO 223 IG=1,NGROUP
                    XSREC(IG,2)=XSREC(IG,19)
 223              CONTINUE
                ENDIF
C----
C  SAVE ISOTOPE INFORMATION
C----
                CALL LCMSIX(IPLIB,HNAMIS,1)
                IF(NF(IEL).LE.1) THEN
                  INDPRO(5)=0
                  CALL XDRLGS(IPLIB,1,NPROC,0,0,1,NGROUP,INDPRO,
     >                        ITYPRO,XSREC,SCAT,XSSCMP,NJJ,IJJ)
                  INDPRO(5)=1
                ELSE
                  CALL XDRLGS(IPLIB,1,NPROC,0,0,1,NGROUP,INDPRO,
     >                        ITYPRO,XSREC,SCAT,XSSCMP,NJJ,IJJ)
                ENDIF
                IF(IPRINT.GE.100) THEN
                  CALL XDRLPR(NPROC,0,0,1,NGROUP,INDPRO,ITYPRO,
     >                        XSREC,SCAT)
                ENDIF
                CALL LCMSIX(IPLIB,HNAMIS,2)
              ENDIF
 220        CONTINUE
          ENDIF
        ENDIF
        READ(IUNIT) ENDR
 130  CONTINUE
C----
C  RELEASE MEMORY FOR TEMPERATURE DEPENDENT XS
C----
      CALL RLSARA(RBASE(ITMPSC))
      CALL RLSARA(RBASE(ITMPXS))
C----
C  ALLOCATE MEMORY FOR RESONANCE READ
C  READ ALL GROUP AND ALL RESONANCES
C----
      CALL SETARA(RBASE,NRTOT,IRID)
      CALL SETARA(IBASE,2*NRTOT*NGR,INTM)
      CALL SETARA(IBASE,2*NRTOT*NGR,INDI)
      CALL SETARA(RBASE,MAXTEM*2*NRTOT*NGR,IRTMP)
      CALL SETARA(RBASE,MAXDIL*2*NRTOT*NGR,IRDIL)
      CALL SETARA(RBASE,MAXDIL*MAXTEM*2*NRTOT*NGR,IRESI)
      CALL XDRSET(RBASE(IRID),NRTOT,0.0)
      CALL XDISET(IBASE(INTM),2*NRTOT*NGR,0)
      CALL XDISET(IBASE(INDI),2*NRTOT*NGR,0)
      CALL XDRSET(RBASE(IRTMP),MAXTEM*2*NRTOT*NGR,0.0)
      CALL XDRSET(RBASE(IRDIL),MAXDIL*2*NRTOT*NGR,0.0)
      CALL XDRSET(RBASE(IRESI),MAXDIL*MAXTEM*2*NRTOT*NGR,0.0)
      CALL LIBWRG(IUNIT,NEL,NGR,NRTOT,MAXTEM,MAXDIL,NF,NSRES,
     >            RBASE(IRID),IBASE(INTM),IBASE(INDI),
     >            RBASE(IRTMP),RBASE(IRDIL),RBASE(IRESI),
     >            TMPT,DILT,REST)
C----
C  ALLOCATE MEMORY FOR RESONANCE PROCESSING
C----
      CALL SETARA(RBASE,MAXDIL*MAXTEM*2,JRRI)
      CALL SETARA(RBASE,MAXDIL,JRIT)
C----
C   PROCESS RESONANCES
C----
      IF(IPRINT.GE.5) THEN
        WRITE(IOUT,6010)
      ENDIF
      DO 230 JSO=1,NBISO
        IF(ITISOM(JSO).NE.ILIB) GO TO 235
        IEL=ISORD(JSO)
        IF(NR(IEL).EQ.0) GO TO 235
        NFIS=0
        IF(NF(IEL).GT.1) NFIS=1
        WRITE(HNAMIS,'(3A4)') (ISONAM(ITC,JSO),ITC=1,3)
        CALL LCMSIX(IPLIB,HNAMIS,1)
        WRITE(HSHIR,'(2A4)') (ISHINA(ITC,JSO),ITC=1,2)
        IDRES=INDEX(HSHIR,'.')
        IF(IDRES.GT.0) THEN
          WRITE(FMT,'(2H(F,I1,3H.1))') IDRES+1
          READ(HSHIR,FMT) RIND
        ELSE
          RIND=FLOAT(IWISO(IEL))
        ENDIF
C----
C  IDENTIFY RESONANCE SET
C  DEFAULT IS RESONNANCE ID SPECIFIED OR FIRST SET ENCOUNTERED
C----
        ILCR=IRID
        DO 231 IXRES=1,NSRES
          XIND=RBASE(ILCR)
          IF(IDRES.EQ.0) THEN
            XRS1=FLOAT(INT((XIND+0.01)*10.)
     >          -INT(XIND+0.01)*10)/10.
            XRS1=ABS(XIND-XRS1-RIND)
          ELSE
            XRS1=ABS(XIND-RIND)
          ENDIF
          IF(XRS1.LE.0.01) THEN
            IRES=IXRES
            GO TO 236
          ENDIF
          ILCR=ILCR+1
 231    CONTINUE
C----
C  START MODIFICATION: 98/05/05 (G.M.)
C  1) NO SPECIFIC RESONNANCE ID SPECIFIED AND
C     NO RESONNANCE ID FOUND WITH ADEQUATE IWISO
C     EVEN IF NR(IEL) > 0
C     ASSUME NO RESONANCE PRESENT
C     REQUIRED FOR ANL 69 GROUPS WIMSD4 LIBRARY
C     ISOTOPE '10' HAS NRES=1 BUT ID DIFFERENT
C     FROM 10.X WHERE x IS AN INTEGER
C  2) CORRECT ERROR IN LOOP INDEX FOR
C     WRITE(IOUT,9004)
C----
        IF(IDRES.EQ.0) THEN
          CALL LCMSIX(IPLIB,HNAMIS,2)
          GO TO 235
        ENDIF
        WRITE(IOUT,9004) (ISONAM(ITC,JSO),ITC=1,3),RIND
C----
C  END MODIFICATION: G.M. (98/05/05)
C----
        CALL XABORT('LIBWD4: UNABLE TO IDENTIFY RESONANCE SET '//
     >              'FOR THIS ISOTOPE')
 236    CONTINUE
C----
C  THIS ISOTOPE NEEDS TO BE CORRECTED FOR SELF SHIELDING
C  FIRST READ UNCORRECTED CROSS SECTIONS
C----
        NTYP=1
        XSCOR(1)=0.0
        IF(NF(IEL).EQ.3) THEN
          NTYP=2
          XSCOR(2)=0.0
        ENDIF
        XSCOR(3)=0.0
        XSCOR(4)=0.0
        IF(IPRINT.GE.5) THEN
          WRITE(IOUT,6011) HNAMIS,RIND
        ENDIF
        CALL XDRLGS(IPLIB,-1,NPROC,0,0,1,NGROUP,INDPRO,
     >              ITYPRO,XSREC,SCAT,XSSCMP,NJJ,IJJ)
C----
C  SCAN RESONAMCE GROUPS AND CORRECT CROSS SECTIONS
C----
        DO 232 IGF=1,NGFR
          XSOUT(IGF,1)=XSREC(IGF,1)-XSREC(IGF,IP0)
          XSOUT(IGF,2)=XSREC(IGF,3)
          XSOUT(IGF,3)=XSREC(IGF,IP0)
          XSOUT(IGF,4)=1.0
          XSOUT(IGF,5)=1.0
 232    CONTINUE
        IGRF=NGF
        DO 240 IGR=1,NGR
          IGRF=IGRF+1
C----
C  PREPARE VECTORS FOR SELF SHIELDING
C----
          IF(IPRINT.GE.100) THEN
            WRITE(IOUT,6003) TN(JSO)
            WRITE(IOUT,6004) SN(IGRF,JSO),DSIGPL(IEL)
          ENDIF
          DO 250 ITYP=1,NTYP
            CALL LIBWRP(IPRINT,NGR,NRTOT,MAXTEM,MAXDIL,IGR,IRES,ITYP,
     >                  DSIGPL(IEL),IBASE(INTM),IBASE(INDI),
     >                  RBASE(IRTMP),RBASE(IRDIL),RBASE(IRESI),
     >                  NTMPR,NDILR,TMPT,DILT,REST)
            IF(NDILR.GT.0.AND.NTMPR.GT.0) THEN
              CALL LIBWRI(NGR,NTMPR,NDILR,IGRF,TN(JSO),SN(IGRF,JSO),
     >                    TMPT,DILT,REST,RBASE(JRIT),
     >                    XSOUT(IGRF,ITYP),XSCOR(ITYP))
              IF(IPRINT.GE.100) THEN
                IF(ITYP.EQ.1) THEN
                  WRITE(IOUT,6005) XSOUT(IGRF,ITYP)
                ELSE IF(ITYP.EQ.2) THEN
                  WRITE(IOUT,6006) XSOUT(IGRF,ITYP)
                ELSE IF(ITYP.EQ.3) THEN
                  WRITE(IOUT,6007) XSOUT(IGRF,ITYP)
                ENDIF
                WRITE(IOUT,6008) (REST(ITT),ITT=1,NTMPR*NDILR)
              ENDIF
            ENDIF
 250      CONTINUE
 240    CONTINUE
C----
C  CORRECT CROSS SECTIONS FOR ALL RESONANCE GROUPS
C----
        IGRF=NGF+1
        IGRL=NGF+NGR
        CALL LIBWRE(NTYP,ITLIB,NGROUP,IGRF,IGRL,SCAT,XSREC(1,IP0),
     >              XSREC(1,1),XSREC(1,7),XSREC(1,3),XSREC(1,4),
     >              XSREC(1,6),DELTA,SN(1,JSO),SB(1,JSO),
     >              XSOUT,XSCOR)
C----
C  PRINT CROSS SECTIONS IF REQUIRED
C----
        IF(IPRINT.GE.5) THEN
          WRITE(IOUT,6100) HNAMIS
          DO 233 IG1=NGF+1,NGFR
            WRITE(IOUT,6101) IG1,SN(IG1,JSO),SB(IG1,JSO),
     >                       XSOUT(IG1,4),XSREC(IG1,1),
     >                       XSREC(IG1,IP0),XSREC(IG1,3),XSREC(IG1,14)
 233      CONTINUE
        ENDIF
C----
C  SET NWT0 THE RESONANCE FLUX WEIGHTING
C----
        CALL XDRSET(XSREC(1,16),NGROUP,1.0)
        DO 234 IG1=NGF+1,NGFR
          XSREC(IG1,16)=XSOUT(IG1,4)
 234    CONTINUE
C----
C  SAVE SELF SHIELDING CORRECTED XS
C----
        CALL XDRLGS(IPLIB,1,NPROC,0,0,1,NGROUP,INDPRO,
     >              ITYPRO,XSREC,SCAT,XSSCMP,NJJ,IJJ)
        CALL LCMSIX(IPLIB,HNAMIS,2)
 235    CONTINUE
 230  CONTINUE
C----
C  RELEASE MEMORY FOR RESONANCE PROCESSING
C----
      CALL RLSARA(RBASE(JRIT))
      CALL RLSARA(RBASE(JRRI))
      CALL RLSARA(RBASE(IRID))
C----
C  RELEASE MEMORY FOR RESONANCE READ
C----
      CALL RLSARA(RBASE(IRESI))
      CALL RLSARA(RBASE(IRDIL))
      CALL RLSARA(RBASE(IRTMP))
      CALL RLSARA(IBASE(INDI))
      CALL RLSARA(IBASE(INTM))
C----
C  P1 SCATTERING
C----
      IF(NL.EQ.2) THEN
        DO 260 ISETP1=1,NSETP1
          DO 270 IG=1,NGROUP
            XSREC(IG,IP1)=0.0
            READ(IUNIT) (SCAT(JG,IG,2),JG=1,NGROUP)
            DO 280 JG=1,NGROUP
              XSREC(IG,IP1)=XSREC(IG,IP1)+SCAT(JG,IG,2)
 280        CONTINUE
 270      CONTINUE
          DO 290 JSO=1,NBISO
            NBAC=INT(AW(JSO)/CONVM+0.01)
            IF(NBAC.EQ.NBATOM(ISETP1)) THEN
C----
C  CLASSIFY BY ATOMIC NUMBER
C----
              WRITE(HNAMIS,'(3A4)') (ISONAM(ITC,JSO),ITC=1,3)
              IF(IPRINT.GT.5) THEN
                WRITE(IOUT,6300) HNAMIS
              ENDIF
              CALL LCMSIX(IPLIB,HNAMIS,1)
              CALL XDRLGS(IPLIB,-1,NPROC,0,0,1,NGROUP,INDPRO,
     >                    ITYPRO,XSREC,SCAT,XSSCMP,NJJ,IJJ)
              CALL XDRLGS(IPLIB,1,NPROC,0,NL-1,1,NGROUP,INDPRO,
     >                    ITYPRO,XSREC,SCAT,XSSCMP,NJJ,IJJ)
              CALL LCMSIX(IPLIB,HNAMIS,2)
            ENDIF
 290      CONTINUE
 260    CONTINUE
      ENDIF
      IERR=KDRCLS(IUNIT,IACTC)
      IF(IERR.LT.0)
     >  CALL XABORT('LIBWD4: WIMS-D4 LIBRARY '//
     >    CFILNA//' CANNOT BE CLOSED')
C----
C  RETURN
C----
      RETURN
C----
C  FORMAT
C----
 9001 FORMAT(/'  NUMBER OF GROUPS SPECIFIED :',I10/
     >        ' NUMBER OF GROUPS IN LIBRARY :',I10)
 9002 FORMAT(/' MAXIMUM NUMBER OF ISOTOPE SPECIFIED :',I10/
     >        '        NUMBER OF ISOTOPE IN LIBRARY :',I10)
 9003 FORMAT(/' LIBWD4: MATERIAL/ISOTOPE ',3A4,
     >        ' IS MISSING ON WIMS-D4 FILE ',A8)
 9004 FORMAT(/' LIBWD4: FOR ISOTOPE ',3A4,
     >        ' SELF-SHIELDING ISOTOPE ',F8.1,' NOT AVAILABLE')
 6000 FORMAT(' READING WIMS-D4 LIBRARY NAME ',A8)
 6001 FORMAT('   PROCESSING ISOTOPE/MATERIAL = ',A12)
 6003 FORMAT('   RESONANCE TEMPERATURE  = ',1P,E15.7)
 6004 FORMAT('   RESONANCE DILUTIONS    = ',1P,2E15.7)
 6005 FORMAT('   ABSORPTION RATE        = ',1P,E15.7)
 6006 FORMAT('   FISSION RATE           = ',1P,E15.7)
 6007 FORMAT('   SCATTERING RATE        = ',1P,E15.7)
 6008 FORMAT(1P,5E15.7)
 6010 FORMAT('   RESONANCE IDENTIFICATION')
 6011 FORMAT('   ISOTOPE ID = ',A12,' RESONANCE ID = ',F8.1)
 6100 FORMAT('   SELF SHIELDING PROPERTIES FOR ISOTOPE =',A12/
     > 5X,'GROUP',10X,'DILUT',13X,'SB',11X,'NPHI',10X,'TOTAL',
     > 11X,'SIGS',9X,'NUSIGF',11X,'GOLD')
 6101 FORMAT(5X,I5,1P,8E15.5)
 6200 FORMAT(' TEMPERATURE = ',F10.5,10X,
     >  ' CROSS SECTION TABULATED AT A SINGLE TEMPERATURE')
 6201 FORMAT(' TEMPERATURE = ',F10.5,10X,
     >  ' CROSS SECTION TABULATED AT MULTIPLE TEMPERATURES')
 6202 FORMAT(' TABULATION TEMPERATURES= ',/(5F15.5))
 6203 FORMAT(' INTERPOLATION FACTORS  = ',1P,/(5E15.5))
 6300 FORMAT(' P1 SCATTERING CROSS SECTION FOUND FOR =',A12)
      END
