*DECK MACDRV
      SUBROUTINE MACDRV(IPMACR,INDREC,IPRINT,NBMIX ,NGROUP,NANISO,
     >                  NIFISS,NEDMAC,ITRANC,NALBD )
C
C---------------------------  MACDRV  ---------------------------------
C
C   1- PROGRAMME STATISTICS:
C       NAME     : MACDRV
C       USE      : INPUT MACROSCOPIC X-SECTIONS IN DRAGON
C       MODIFIED : 2000/01/20 (T. COURAU)
C                    INSERT OPTION TO CREATE AN ADJOINT MACROLIB
C                    AND TO DEAL WITH ADJOINT AND GENERALIZED ADJOINT
C                    FIXED SOURCES: 'FIXE' , 'FIXA' , 'FIXG'
C                    PERFORM MACROLIB LINEAR COMBINATIONS (LINC)
C                  1991/01/24 (G. MARLEAU)
C
C   2- ROUTINE PARAMETERS:
C    INPUT
C       IPMACR   : POINTER TO THE MACROLIB                    I
C       INDREC   : =1: THE MACROLIB IS CREATED                I
C                  =2: AN EXISTING MACROLIB IS MODIFIED
C                  =3: AN EXISTING LIBRARY IS MODIFIED
C       IPRINT   : PRINT LEVEL                                I
C    OUTPUT
C       NBMIX    : NUMBER OF MIXTURES                         I
C       NGROUP   : MAXIMUM NUMBER OF GROUPS                   I
C                  DEFAULT = 1
C       NANISO   : MAXIMUN SCATTERING ANISOTROPY              I
C                  DEFAULT = 1 ISOTROPIC COLLISION IN LAB
C       NIFISS   : NUMBER OF FISSILE ISOTOPES PER MIXTURES    I
C                  DEFAULT = 0
C       NEDMAC   : NUMBER OF ADITIONAL EDITION X-S            I
C       ITRANC   : TYPE OF TRANSPORT CORRECTION               I
C       NALBD    : NUMBER OF PHYSICAL ALBEDOS                 I
C
C---------------------------   MACDRV  --------------------------------
C
      IMPLICIT    NONE
      INTEGER     IOUT,NTC,ILCMUP,ILCMDN,ITGOXS,NFILCN,
     >            NBXS
      CHARACTER   NAMSBR*6
      PARAMETER  (IOUT=6,NTC=3,ILCMUP=1,ILCMDN=2,
     >            ITGOXS=2,NFILCN=10,NBXS=10,NAMSBR='MACDRV')
C----
C  LOCAL VARIABLES
C  FOR AVERAGED NEUTRON VELOCITY
C  V=SQRT(2*ENER/M)=SQRT(2/M)*SQRT(ENER)
C  SQFMAS=SQRT(2/M) IN CM/S/SQRT(EV) FOR V IN CM/S AND E IN EV
C        =SQRT(2*1.602189E-19(J/EV)* 1.0E4(CM2/M2) /1.67495E-27 (KG))
C        =1383155.30602 CM/S/SQRT(EV)
C----
      REAL        SQFMAS
      PARAMETER  (SQFMAS=1383155.30602)
C----
C  ROUTINE PARAMETERS
C----
      INTEGER     IPMACR,INDREC,IPRINT,NBMIX,NGROUP,NANISO,
     >            NIFISS,NEDMAC,ITRANC,NALBD
C----
C  FUNCTIONS
C----
      INTEGER     KDROPN,KDRCLS
C----
C  MEMORY ALLOCATION
C----
      INTEGER     IBASE(1)
      REAL        RBASE
      COMMON      RBASE(1)
      EQUIVALENCE(RBASE(1),IBASE(1))
      INTEGER     ISCATA,IFGUPS,IPOSCT,IFGSCT,INGSCT,INBSCT,LINIXS,
     >            IXSS,IINS,IMXS,IEXS,IPERMU,IVEL,IENE,IALB,JTRAN,
     >            JTOTL,JSCAT,JFISS,JFIXE,JWORK,ICHWOR,
     >            ISCWOR,IRSCAT,IRNIJJ,IXSCAT,NIJJ,
     >            IXSGEN,JEFIS,IADNAM
C----
C  INPUT
C----
      CHARACTER   CARLIR*12
      INTEGER     ITYPLU,INTLIR
      REAL        REALIR
      DOUBLE PRECISION DBLLIR
C----
C  LOCAL PARAMETERS
C----
      INTEGER     IFILCN(NFILCN),IMATER,NEXTRE,NEXTMI,MAXFIS,IL,
     >            NBELEM,NUMMAT,JVEL,IAL,IGR,
     >            KEXS,KMXS,KINS,KXSS,IUGOXS,IDGOXS,IWGOXS,
     >            IRETRN,NEXTRI,NPRXS,NUDEPR,NBWRK
      LOGICAL     LIREAD,LGOXS,LOLDXS(NBXS),LNEWXS(NBXS),LNORM,
     >            LGOXSW,LADD,LUPD
      REAL        ENEMAX,VELG1,VELG2
C----
C  DATA
C----
      INTEGER     NUMREC
      CHARACTER   CGOXNM*7,CGOXSW*7
      SAVE        NUMREC,CGOXNM,CGOXSW
      DATA        NUMREC  /0/
      DATA        CGOXNM   ,CGOXSW 
     >           /'       ','goxsout'/
C----
C  INITIALIZE USEFUL PARAMETERS
C----
      LIREAD=.TRUE.
      LGOXS =.FALSE.
      LGOXSW=.FALSE.
      LNORM =.FALSE.
      LADD  =.FALSE.
      LUPD  =(INDREC .GE. 2)
      IDGOXS=1
      NEXTRE=1
      NEXTMI=1
      MAXFIS=MAX(1,NIFISS)
      CALL SETARA(IBASE,NANISO,ISCATA)
      CALL XDISET(IBASE(ISCATA),NANISO,0)
      DO 120 IL=1,NBXS
        LOLDXS(IL)=.FALSE.
        LNEWXS(IL)=.FALSE.
 120  CONTINUE
C----
C  READ A MAIN OPTION
C----
 1000 CONTINUE
      CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
C----
C  CHECK FOR STOP/RETURN
C----
 1010 CONTINUE
      IF(ITYPLU .NE. 3) CALL XABORT(NAMSBR//
     >': READ ERROR - MAC KEYWORD EXPECTED')
      IF(CARLIR .EQ. ';') THEN
        IF(LGOXS) IRETRN=KDRCLS(IUGOXS,IDGOXS)
        GO TO 2000
      ELSE IF(CARLIR .EQ. 'EDIT') THEN
C----
C  READ THE PRINT INDEX
C----
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 1) CALL XABORT(NAMSBR//
     >  ': READ ERROR - PRINT LEVEL EXPECTED')
        IPRINT=INTLIR
      ELSE IF(CARLIR .EQ. 'READ') THEN
        IF(NBMIX .EQ. 0) CALL XABORT(NAMSBR//
     >  ': NBMIX NOT YET DEFINED')
        CALL SETARA(IBASE,NBMIX,IPERMU)
        NBELEM=0
C----
C  IDENTIFY MIXTURES TO READ
C----
        NUMMAT=0
        DO 100 IMATER=1,NBMIX
          CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
          IF(ITYPLU .NE. 1) GO TO 105
          IF(INTLIR .GT. NBMIX) CALL XABORT(NAMSBR//
     >    ': MIXTURE NUMBER IS LARGER THAN NBMIX')
          IBASE(IPERMU+IMATER-1)=INTLIR
          NUMMAT=MAX(NUMMAT,INTLIR)
          NBELEM=NBELEM+1
 100    CONTINUE
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
 105    CONTINUE
        IF(ITYPLU .NE. 3) CALL XABORT(NAMSBR//
     >  ': FILE NAME FOR READ EXPECTED')
        IF(CARLIR .EQ. 'INPUT') THEN
          CGOXNM=CARLIR
        ELSE
          IF(NIFISS .GT. 1) CALL XABORT(NAMSBR//
     >    ': READ ERROR - ONLY NIFISS=0 OR 1 IS VALID')
          IF(LGOXS .AND. (CARLIR .NE. CGOXNM))
     >      IRETRN=KDRCLS(IUGOXS,IDGOXS)
          CGOXNM=CARLIR
          IUGOXS=KDROPN(CGOXNM,2,ITGOXS,0,0)
          LGOXS=.TRUE.
          IDGOXS=1
          CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
          IF(ITYPLU .EQ. 3) THEN
            IF(CARLIR .EQ. 'DELE') THEN
              IDGOXS=2
              CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
            ENDIF
          ENDIF
          IF(ITYPLU .EQ. 1) THEN
            IF(NUMREC .GT. INTLIR) THEN
              NUDEPR=1
              NUMREC=INTLIR
              NEXTRE=1
            ELSE
              NUDEPR=NUMREC+1
              NUMREC=INTLIR
            ENDIF
            CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
          ELSE
            NUMREC=NUMREC+1
            NUDEPR=NUMREC
          ENDIF
          CALL GXSINI(IPMACR,IPRINT,IUGOXS,NBMIX ,NGROUP,NBELEM,
     >                NEXTMI,NUMMAT,NEXTRE,NUDEPR,NUMREC,LIREAD,
     >                NFILCN,IFILCN)
          IF(LIREAD) THEN
            IF(IFILCN(3) .GT. NANISO) THEN
              NANISO=IFILCN(3)
              CALL RLSARA(IBASE(ISCATA))
              CALL SETARA(IBASE,NANISO,ISCATA)
            ENDIF
            CALL XDISET(IBASE(ISCATA),IFILCN(3),2)
          ENDIF
        ENDIF
        IF(LIREAD) THEN
          CALL SETARA(RBASE,NBMIX*(NGROUP+1)   ,JWORK)
          CALL SETARA(RBASE,NBMIX*NGROUP       ,JTOTL)
          CALL SETARA(RBASE,NBMIX*NGROUP       ,JTRAN)
          CALL SETARA(RBASE,NBMIX*NGROUP*MAXFIS*3,JFISS)
          CALL SETARA(RBASE,NBMIX*NGROUP*3       ,JFIXE)
          CALL SETARA(RBASE,NBMIX*MAXFIS       ,JEFIS)
          CALL SETARA(RBASE,NBMIX*NGROUP*NGROUP*NANISO,JSCAT)
          CALL SETARA(IBASE,NBMIX,INBSCT)
          CALL SETARA(IBASE,NBMIX,INGSCT)
          CALL MACIXS(IPMACR,NBMIX ,NGROUP,NANISO,NIFISS,MAXFIS,
     >                LOLDXS,
     >                RBASE(JTOTL) ,RBASE(JTRAN) ,RBASE(JFISS) ,
     >                RBASE(JFIXE) ,RBASE(JSCAT) ,RBASE(JWORK) ,
     >                IBASE(INBSCT),IBASE(INGSCT),IBASE(ISCATA),
     >                RBASE(JEFIS) )
          CALL RLSARA(IBASE(INGSCT))
          CALL RLSARA(IBASE(INBSCT))
          LIREAD=.FALSE.
        ENDIF
        IF(CGOXNM .EQ. 'INPUT') THEN
          CALL SETARA(IBASE,NBXS*NBMIX,LINIXS)
          CALL MACXSR(IPRINT,NBMIX ,NGROUP,NANISO,NIFISS,MAXFIS,
     >                LOLDXS,LNEWXS,CARLIR,LADD  ,LUPD  ,
     >                RBASE(JTOTL) ,RBASE(JTRAN) ,RBASE(JFISS) ,
     >                RBASE(JFIXE) ,RBASE(JSCAT) ,IBASE(ISCATA),
     >                RBASE(JEFIS) ,IBASE(LINIXS))
          CALL RLSARA(IBASE(LINIXS))
        ELSE
          LNEWXS(1)=.TRUE.
          LNEWXS(2)=.TRUE.
          LNEWXS(4)=.TRUE.
          LNEWXS(5)=.TRUE.
          LNEWXS(6)=.TRUE.
          LNEWXS(7)=.TRUE.
          LNEWXS(8)=.TRUE.
          LNEWXS(9)=.TRUE.
          IF(IFILCN(5) .EQ. 0) THEN
            IFILCN(5)=1
          ENDIF
          KEXS=IFILCN(4)+IFILCN(9)
          KMXS=KEXS*IFILCN(2)*IFILCN(5)
          KINS=2*IFILCN(2)*IFILCN(3)
          KXSS=NGROUP*IFILCN(2)*IFILCN(3)*IFILCN(5)
          CALL SETARA(IBASE,KEXS,IEXS)
          CALL SETARA(RBASE,KMXS,IMXS)
          CALL SETARA(IBASE,KINS,IINS)
          CALL SETARA(RBASE,KXSS,IXSS)
          CALL GXSGET(IPMACR,IUGOXS,NBMIX ,NGROUP,NANISO,NBELEM,
     >                NEXTMI,NEXTRE,NFILCN,IFILCN,KEXS  ,KMXS  ,
     >                KINS  ,KXSS  ,
     >                IBASE(IPERMU),RBASE(JTOTL) ,RBASE(JTRAN) ,
     >                RBASE(JFISS) ,RBASE(JSCAT) ,RBASE(JWORK) ,
     >                IBASE(IEXS)  ,RBASE(IMXS)  ,IBASE(IINS)  ,
     >                RBASE(IXSS)  )
          CALL XDISET(IBASE(ISCATA),IFILCN(3),2)
          CALL RLSARA(RBASE(IXSS))
          CALL RLSARA(IBASE(IINS))
          CALL RLSARA(RBASE(IMXS))
          CALL RLSARA(IBASE(IEXS))
        ENDIF
        CALL RLSARA(IBASE(IPERMU))
        GO TO 1010
      ELSE IF(CARLIR .EQ. 'NGRO') THEN
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 1) CALL XABORT(NAMSBR//
     >  ': READ ERROR - NUMBER OF GROUPS EXPECTED')
        NGROUP=INTLIR
      ELSE IF(CARLIR .EQ. 'NMIX') THEN
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 1) CALL XABORT(NAMSBR//
     >  ': READ ERROR - NUMBER OF MIXTURES EXPECTED')
        NBMIX=INTLIR
      ELSE IF(CARLIR .EQ. 'ANIS') THEN
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 1) CALL XABORT(NAMSBR//
     >  ': READ ERROR - ANIS LEVEL EXPECTED')
        IF(INTLIR .GT. NANISO) THEN
          CALL RLSARA(IBASE(ISCATA))
          CALL SETARA(IBASE,INTLIR,ISCATA)
          CALL XDISET(IBASE(ISCATA),NANISO,0)
        ENDIF
        NANISO=INTLIR
      ELSE IF(CARLIR .EQ. 'CTRA') THEN
C----
C  READ TRANSPORT CORRECTION TYPE
C----
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 3) CALL XABORT(NAMSBR//
     >  ': READ ERROR - CTRA TYPE EXPECTED')
        IF(CARLIR .EQ. 'OFF') THEN
          ITRANC=0
        ELSE IF(CARLIR .EQ. 'APOL') THEN
          ITRANC=1
        ELSE IF(CARLIR .EQ. 'WIMS') THEN
          ITRANC=2
        ELSE
          CALL XABORT(NAMSBR//': OFF, APOL OR WIMS EXPECTED')
        ENDIF
      ELSE IF(CARLIR .EQ. 'NIFI') THEN
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU.NE.1) CALL XABORT(NAMSBR//
     >  ': READ ERROR - NUMBER FISSILE ISOTOPES EXPECTED')
        NIFISS=INTLIR
        MAXFIS=MAX(1,NIFISS)
      ELSE IF(CARLIR .EQ. 'NORM') THEN
        LNORM=.TRUE.
      ELSE IF(CARLIR .EQ. 'ADD') THEN
        IF(.NOT. LUPD) CALL XABORT(NAMSBR//
     >  ': CANNOT USE THE ADD OPTION ON A MACROLIB IN CREATION MODE')
        LADD=.TRUE.
      ELSE IF(CARLIR .EQ. 'ENER') THEN
C----
C  READ ENERGY GROUPS
C----
        IF(NGROUP .GT. 0) THEN
          CALL SETARA(RBASE,NGROUP+1,IENE)
          CALL SETARA(RBASE,NGROUP,IVEL)
          CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
          IF(ITYPLU .NE. 2) CALL XABORT(NAMSBR//
     >    ': READ ERROR - MAXIMUM ENERGY LIMIT EXPECTED')
          ENEMAX=REALIR
          DO 179 IGR=IENE,IENE+NGROUP-1
            RBASE(IGR)=ENEMAX
            CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
            IF(ITYPLU .NE. 2) CALL XABORT(NAMSBR//
     >      ': READ ERROR - REAL ENERGY VARIABLE EXPECTED')
            IF(REALIR .GT. ENEMAX) CALL XABORT(NAMSBR//
     >      ': READ ERROR - ENERGY GOES FROM MAX TO MIN')
            ENEMAX=REALIR
 179      CONTINUE
          IF(ENEMAX .LE. 0.0) THEN
            RBASE(IENE+NGROUP)=1.0E-5
          ELSE
            RBASE(IENE+NGROUP)=ENEMAX
          ENDIF
          CALL LCMPUT(IPMACR,'ENERGY',NGROUP+1,2,RBASE(IENE))
          VELG1=SQRT(RBASE(IENE))
          JVEL=IVEL
          DO 178 IGR=IENE,IENE+NGROUP-1
            VELG2=SQRT(RBASE(IGR+1))
            RBASE(JVEL)=SQFMAS*SQRT(VELG1*VELG2)
            VELG1=VELG2
            RBASE(IGR)=LOG(RBASE(IGR)/RBASE(IGR+1))
            JVEL=JVEL+1
 178      CONTINUE
          CALL LCMPUT(IPMACR,'AVGVEL',NGROUP,2,RBASE(IVEL))
          CALL LCMPUT(IPMACR,'DELTAU',NGROUP,2,RBASE(IENE))
          CALL RLSARA(RBASE(IVEL))
          CALL RLSARA(RBASE(IENE))
        ENDIF
      ELSE IF(CARLIR .EQ. 'ALBP') THEN
C----
C  READ GROUP INDEPENDENT PHYSICAL ALBEDOS
C----
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 1) CALL XABORT(NAMSBR//
     >  ': READ ERROR - NUMBER OF PHYSICAL ALBEDO EXPECTED')
        NALBD=INTLIR
        IF(NALBD .GT. 0) THEN
          CALL SETARA(RBASE,NALBD,IALB)
          DO 180 IAL=1,NALBD
            CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
            IF(ITYPLU .NE. 2) CALL XABORT(NAMSBR//
     >      ': REAL PHYSICAL ALBEDO EXPECTED.')
            RBASE(IALB+IAL-1)=REALIR
 180      CONTINUE
          CALL LCMPUT(IPMACR,'ALBEDO',NALBD,2,RBASE(IALB))
          CALL RLSARA(RBASE(IALB))
        ELSE
          NALBD=0
        ENDIF
      ELSE IF(CARLIR .EQ. 'WRIT') THEN
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 3) CALL XABORT(NAMSBR//
     >  ': READ ERROR - GOXS FILE NAME EXPECTED')
        LGOXSW=.TRUE.
        CGOXSW=CARLIR(1:7)
      ELSE
        CALL XABORT(NAMSBR//': INVALID KEY-WORD :'//CARLIR)
      ENDIF
      GO TO 1000
 2000 CONTINUE
C----
C  TRANSFER MODIFIED X-S ON THE MACROLIB
C----
      IF( .NOT. LIREAD) THEN
        CALL SETARA(IBASE,NBMIX,INGSCT)
        CALL SETARA(IBASE,NBMIX,IFGSCT)
        CALL SETARA(IBASE,NBMIX,IPOSCT)
        CALL SETARA(IBASE,NBMIX*NANISO,IFGUPS)
        CALL MACPXS(IPMACR,NBMIX ,NGROUP,NANISO,NIFISS,MAXFIS,
     >              ITRANC,LNEWXS,NEDMAC,
     >              RBASE(JTOTL) ,RBASE(JTRAN) ,RBASE(JFISS) ,
     >              RBASE(JFIXE) ,RBASE(JSCAT) ,RBASE(JWORK) ,
     >              IBASE(INGSCT),IBASE(IFGSCT),IBASE(IPOSCT),
     >              IBASE(ISCATA),RBASE(JEFIS) ,IBASE(IFGUPS))
        CALL RLSARA(IBASE(IFGUPS))
        CALL RLSARA(IBASE(IPOSCT))
        CALL RLSARA(IBASE(IFGSCT))
        CALL RLSARA(IBASE(INGSCT))
        CALL RLSARA(RBASE(JSCAT))
        CALL RLSARA(RBASE(JEFIS))
        CALL RLSARA(RBASE(JFIXE))
        CALL RLSARA(RBASE(JFISS))
        CALL RLSARA(RBASE(JTRAN))
        CALL RLSARA(RBASE(JTOTL))
        CALL RLSARA(RBASE(JWORK))
      ENDIF
C----
C  NORMALIZATION OF X-S INFORMATION.
C----
      IF(LNORM) THEN
        NBWRK=NBMIX*MAX(NGROUP,MAXFIS)
        CALL SETARA(IBASE,NBMIX,INGSCT)
        CALL SETARA(IBASE,NBMIX,IFGSCT)
        CALL SETARA(RBASE,NBWRK,JWORK)
        CALL SETARA(RBASE,2*NBMIX*NANISO*NGROUP,ISCWOR)
        CALL SETARA(RBASE,NBMIX*MAXFIS,ICHWOR)
        CALL MACNXS(IPMACR,NBMIX ,NGROUP,NANISO,NIFISS,MAXFIS,
     >              NBWRK ,
     >              IBASE(INGSCT),IBASE(IFGSCT),RBASE(JWORK) ,
     >              RBASE(ISCWOR),RBASE(ICHWOR))
        CALL RLSARA(RBASE(ICHWOR))
        CALL RLSARA(RBASE(ISCWOR))
        CALL RLSARA(RBASE(JWORK))
        CALL RLSARA(IBASE(IFGSCT))
        CALL RLSARA(IBASE(INGSCT))
      ENDIF
C----
C  PRINT/CHECK X-S INFORMATION.
C----
      IF(IPRINT .LT. 0 .OR. IPRINT .GT. 1) THEN
        NBWRK=NBMIX*MAX(NGROUP+1,3*MAXFIS)
        CALL SETARA(IBASE,NBMIX,INGSCT)
        CALL SETARA(IBASE,NBMIX,IFGSCT)
        CALL SETARA(IBASE,NBMIX,IPOSCT)
        CALL SETARA(RBASE,NBWRK,JWORK)
        CALL SETARA(IBASE,2*(NEDMAC+1),IADNAM)
        CALL MACWXS(IPMACR,IPRINT,NBMIX ,NGROUP,NANISO,NIFISS,
     >              ITRANC,NEDMAC,NBWRK ,
     >              IBASE(INGSCT),IBASE(IFGSCT),IBASE(IPOSCT),
     >              RBASE(JWORK) ,IBASE(IADNAM))
        CALL RLSARA(IBASE(IADNAM))
        CALL RLSARA(RBASE(JWORK))
        CALL RLSARA(IBASE(IPOSCT))
        CALL RLSARA(IBASE(IFGSCT))
        CALL RLSARA(IBASE(INGSCT))
      ENDIF
C----
C  PRINT X-S ON GOXS FILE
C----
      IF(LGOXSW) THEN
        NPRXS=4
        IFILCN( 1)=NGROUP
        IFILCN( 2)=NBMIX
        IFILCN( 3)=NANISO
        IFILCN( 4)=NEDMAC+2
        IFILCN( 5)=1
        IFILCN( 6)=NGROUP
        IFILCN( 7)=NGROUP
        IFILCN( 8)=NGROUP
        IFILCN( 9)=NPRXS
        IFILCN(10)=0
        NEXTRI=1
        IWGOXS=KDROPN(CGOXSW,0,ITGOXS,0,0)
        KEXS=NEDMAC+NPRXS+2
        CALL SETARA(RBASE,KEXS*NBMIX,IXSGEN)
        CALL SETARA(IBASE,2*NANISO*NBMIX,NIJJ)
        CALL SETARA(RBASE,NGROUP*NANISO*NBMIX,IXSCAT)
        CALL SETARA(IBASE,NBMIX,IRNIJJ)
        CALL SETARA(RBASE,NGROUP*NBMIX,IRSCAT)
        CALL GXSPUT(IPMACR,IWGOXS,NBMIX ,NGROUP,NANISO,NEXTRI,
     >              NFILCN,IFILCN,KEXS,
     >              RBASE(IXSGEN),IBASE(NIJJ)  ,IBASE(IRNIJJ),
     >              RBASE(IXSCAT),RBASE(IRSCAT))
        CALL RLSARA(RBASE(IRSCAT))
        CALL RLSARA(IBASE(IRNIJJ))
        CALL RLSARA(RBASE(IXSCAT))
        CALL RLSARA(IBASE(NIJJ))
        CALL RLSARA(RBASE(IXSGEN))
        IRETRN=KDRCLS(IWGOXS,1)
      ENDIF
      IF(.NOT.LOLDXS(2) .AND. .NOT.LNEWXS(2) ) THEN
        NIFISS=0
      ENDIF
      CALL RLSARA(IBASE(ISCATA))
      RETURN
      END
