*DECK PIJI3D
      SUBROUTINE PIJI3D(IFTRAK,NREG,NSOUT,NTRK,MXSEG,NCOR,SWVOID,
     >                  SIGTAL,PROB,SEGLEN,NRSEG,STAYIN,GOSOUT,DPR)
C
C-------------------------    PIJI3D    -------------------------------
C
C 1- SUBROUTINE STATISTICS:
C     NAME     : PIJI3D
C     LEVEL    : 2 (CALLED BY 'EXCELP')
C     USE      : INTEGRATION FOR GENERAL 3D ISOTROPIC B.C. TRACKING
C     MODIFIED : 91/07/12 (R.R.)
C     AUTHOR   : R. ROY
C
C 2- PARAMETERS:
C  INPUT
C     IFTRAK  : UNIT ASSOCIATED WITH TRACKING FILE     I
C     NREG    : TOTAL NUMBER OF REGIONS                I
C     NSOUT   : NUMBER OF OUTER SURFACE                I
C     NTRK    : NUMBER OF TRACKS                       I
C     MXSEG   : MAXIMUM SEGMENT LENGTH                 I
C     NCOR    : MAXIMUM NUMBER OF CORNERS              I
C     SWVOID  : FLAG TO INDICATE IF THERE ARE VOIDS    L
C     SIGTAL  : ALBEDO-CROSS SECTION VECTOR            R(-NSOUT:NREG)
C               IS=-NSOUT,-1; SIGTAL(IS)=ALBEDO(-IS)
C               IV=1,NREG ;   SIGTAL(IV)=SIGT(IV)
C  OUTPUT
C     PROB    : COMPRESS PROBABILITY MATRIX            R(NPLEN)
C               NPLEN=(NREG+NSOUT+2)*(NREG+NSOUT+1)/2
C               IND(I,J)=MAX(I+NSOUT+1,J+NSOUT+1)
C                       *(MAX(I+NSOUT+1,J+NSOUT+1)-1)/2
C                       +MIN(I+NSOUT+1,J+NSOUT+1)
C         IS=-NSOUT,-1; JS=-NSOUT,IS; I=IND(IS,JS)
C           PROB(I)=VOLSUR(IS)*PSS(IS,JS)
C         IV=1,NREG; JS=-NSOUT,-1;    I=IND(IV,JS)
C           SIGT(IV).GT.0.0
C             PROB(I)=SIGT(IV)*VOLSUR(IV)*PVS(IV,JS)
C           SIGT(IV).EQ.0.0
C             PROB(I)=VOLSUR(IV)*PVS(IV,JS)
C         IV=1,NREG; JV=1,IV;       I=IND(IV,JV)
C           SIGT(IV).GT.0.0 AND SIGT(JV).GT.0.0
C             PROB(I)=SIGT(IV)*SIGT(JV)*VOLSUR(IV)*PVV(IV,JV)
C           SIGT(IV).GT.0.0 AND SIGT(JV).EQ.0.0
C             PROB(I)=SIGT(IV)*VOLSUR(IV)*PVV(IV,JV)
C           SIGT(IV).EQ.0.0 AND SIGT(JV).GT.0.0
C             PROB(I)=SIGT(JV)*VOLSUR(IV)*PVV(IV,JV)
C           SIGT(IV).EQ.0.0 AND SIGT(JV).EQ.0.0
C             PROB(I)=VOLSUR(IV)*PVV(IV,JV)
C  WORK
C     SEGLEN  : LENGTH OF TRACK              R(MXSEG)
C     NRSEG   : REGION CROSSED BY TRACK      I(MXSEG)
C     STAYIN  : STAY-IN ZONE PROBABILITY     R(MXSEG)
C     GOSOUT  : GOES-OUT ZONE PROBABILITY    R(MXSEG)
C     DPR     : COLLISION PROBABILITIES      D(-NSOUT:NREG,-NSOUT:NREG)
C
C-------------------------    PIJI3D    -------------------------------
C
      IMPLICIT          NONE
      INTEGER           IFTRAK,NREG,NSOUT,NTRK,MXSEG,NCOR,
     >                  NRSEG(MXSEG), ISD(6), ISF(6),
     >                  IL, JL, IANG, LINE, ISD1, ISF1, ITRAK, NOIL, IPR
      REAL              SIGTAL(-NSOUT:NREG),PROB(*),
     >                  SEGLEN(MXSEG),STAYIN(MXSEG),GOSOUT(MXSEG),
     >                  ZERO, ONE, HALF,
     >                  WEIGHT, SSD1, SSF1, XSIL, PRODUC,
     >                  DSCBEG, DSCEND, ZCOR, ZCOR2
      DOUBLE PRECISION  DPR(-NSOUT:NREG,-NSOUT:NREG)
      LOGICAL           SWVOID
      PARAMETER       ( ZERO=0.0E0, ONE=1.0E0, HALF=0.5E0 )
      REAL              SIXT,XSIL2,CUTEXP
      PARAMETER       ( CUTEXP=0.02)
      SIXT=HALF/3.0
C
C0)   INITIALIZATION OF PROB ------------------------------------------
      DO 20 JL     = -NSOUT, NREG
         DO 20 IL  = -NSOUT, NREG
            DPR(IL,JL)= 0.0D0
   20 CONTINUE
C
      IF( NCOR.EQ.1 )THEN
C
C1)   ONLY ONE EXTERNAL SURFACE AT END --------------------------------
      IF( SWVOID )THEN
C
C1.1) VOIDS ARE POSSIBLE
      DO 30 ITRAK=1, NTRK
         READ(IFTRAK)  IANG,LINE,WEIGHT,
     >     ISD1,( NRSEG(IL),IL= 1,LINE-2),ISF1,
     >     SSD1,(SEGLEN(IL),IL= 1,LINE-2),SSF1
         PRODUC= WEIGHT
C        PII CALCULATION AND ESCAPE
         DO 40 IL = 1,LINE-2
            NOIL  = NRSEG(IL)
            XSIL  = SIGTAL(NOIL)*SEGLEN(IL)
            IF( XSIL.EQ.ZERO )THEN
               GOSOUT(IL)= ONE
               STAYIN(IL)= SEGLEN(IL)
               DPR(NOIL,NOIL)= DPR(NOIL,NOIL)
     >                       + HALF*WEIGHT*SEGLEN(IL)*SEGLEN(IL)
            ELSE IF(XSIL .LT. CUTEXP) THEN
               XSIL2=XSIL*XSIL
               STAYIN(IL)=XSIL-XSIL2*(HALF-SIXT*XSIL)
               GOSOUT(IL)=ONE-STAYIN(IL)
               PRODUC= PRODUC * GOSOUT(IL)
               DPR(NOIL,NOIL)= DPR(NOIL,NOIL)
     >          + WEIGHT*XSIL2*(HALF-SIXT*XSIL)
            ELSE
               GOSOUT(IL)= EXP( - XSIL )
               STAYIN(IL)= (ONE - GOSOUT(IL))
               PRODUC= PRODUC * GOSOUT(IL)
               DPR(NOIL,NOIL)= DPR(NOIL,NOIL)
     >                       + WEIGHT*(XSIL-STAYIN(IL))
            ENDIF
   40    CONTINUE
C        PIJ CALCULATION
         DSCBEG= WEIGHT
         DO 60 IL = 1, LINE-2
            NOIL  = NRSEG(IL)
            DSCEND= WEIGHT*STAYIN(IL)
            DO 50 JL  = IL+1, LINE-2
               DPR(NRSEG(JL),NOIL)= DPR(NRSEG(JL),NOIL)
     >                            + STAYIN(JL)*DSCEND
               DSCEND= DSCEND*GOSOUT(JL)
   50       CONTINUE
C           PIS CALCULATION
            DPR(ISD1,NOIL)= DPR(ISD1,NOIL)+DSCBEG*STAYIN(IL)
            DPR(ISF1,NOIL)= DPR(ISF1,NOIL)+DSCEND
            DSCBEG= DSCBEG * GOSOUT(IL)
   60    CONTINUE
C        PSS CALCULATION
         DPR(ISD1,ISF1)= DPR(ISD1,ISF1) + PRODUC
   30 CONTINUE
      ELSE
C
C1.2) NO VOID REGION
      DO 130 ITRAK=1,NTRK
         READ(IFTRAK)  IANG,LINE,WEIGHT,
     >     ISD1,( NRSEG(IL),IL= 1,LINE-2),ISF1,
     >     SSD1,(SEGLEN(IL),IL= 1,LINE-2),SSF1
         PRODUC= WEIGHT
C        PII CALCULATION AND ESCAPE
         DO 140 IL = 1,LINE-2
            NOIL  = NRSEG(IL)
            XSIL  = SIGTAL(NOIL)*SEGLEN(IL)
            IF(XSIL .LT. CUTEXP) THEN
               XSIL2=XSIL*XSIL
               STAYIN(IL)=XSIL-XSIL2*(HALF-SIXT*XSIL)
               GOSOUT(IL)=ONE-STAYIN(IL)
               PRODUC= PRODUC * GOSOUT(IL)
               DPR(NOIL,NOIL)= DPR(NOIL,NOIL)
     >          + WEIGHT*XSIL2*(HALF-SIXT*XSIL)
            ELSE
               GOSOUT(IL)= EXP( - XSIL )
               STAYIN(IL)= (ONE - GOSOUT(IL))
               PRODUC= PRODUC * GOSOUT(IL)
               DPR(NOIL,NOIL)= DPR(NOIL,NOIL)
     >                       + WEIGHT*(XSIL-STAYIN(IL))
            ENDIF
  140    CONTINUE
C        PIJ CALCULATION
         DSCBEG= WEIGHT
         DO 160 IL = 1, LINE-2
            NOIL  = NRSEG(IL)
            DSCEND= WEIGHT*STAYIN(IL)
            DO 150 JL  = IL+1, LINE-2
               DPR(NRSEG(JL),NOIL)= DPR(NRSEG(JL),NOIL)
     >                            + STAYIN(JL)*DSCEND
               DSCEND= DSCEND*GOSOUT(JL)
  150       CONTINUE
C           PIS CALCULATION
            DPR(ISD1,NOIL)= DPR(ISD1,NOIL)+DSCBEG*STAYIN(IL)
            DPR(ISF1,NOIL)= DPR(ISF1,NOIL)+DSCEND
            DSCBEG= DSCBEG * GOSOUT(IL)
  160    CONTINUE
C        PSS CALCULATION
         DPR(ISD1,ISF1)= DPR(ISD1,ISF1) + PRODUC
  130 CONTINUE
      ENDIF
      ELSE
C
C2)   MORE THAN ONE SURFACE PER LINE ----------------------------------
      ZCOR= 1./FLOAT(NCOR)
      ZCOR2= ZCOR*ZCOR
      IF( SWVOID )THEN
C
C2.1) VOIDS ARE POSSIBLE
      DO 230 ITRAK=1,NTRK
         READ(IFTRAK)  IANG,LINE,WEIGHT,
     >     (ISD(JL),JL=1,NCOR),( NRSEG(IL),IL= 1,LINE-2*NCOR),
     >     (ISF(JL),JL=1,NCOR),
     >     (SSD1   ,JL=1,NCOR),(SEGLEN(IL),IL= 1,LINE-2*NCOR),
     >     (SSF1   ,JL=1,NCOR)
         PRODUC= WEIGHT*ZCOR2
C        PII CALCULATION AND ESCAPE
         DO 240 IL = 1,LINE-2*NCOR
            NOIL  = NRSEG(IL)
            XSIL  = SIGTAL(NOIL)*SEGLEN(IL)
            IF( XSIL.EQ.ZERO )THEN
               GOSOUT(IL)= ONE
               STAYIN(IL)= SEGLEN(IL)
               DPR(NOIL,NOIL)= DPR(NOIL,NOIL)
     >                       + HALF*WEIGHT*SEGLEN(IL)*SEGLEN(IL)
            ELSE IF(XSIL .LT. CUTEXP) THEN
               XSIL2=XSIL*XSIL
               STAYIN(IL)=XSIL-XSIL2*(HALF-SIXT*XSIL)
               GOSOUT(IL)=ONE-STAYIN(IL)
               PRODUC= PRODUC * GOSOUT(IL)
               DPR(NOIL,NOIL)= DPR(NOIL,NOIL)
     >          + WEIGHT*XSIL2*(HALF-SIXT*XSIL)
            ELSE
               GOSOUT(IL)= EXP( - XSIL )
               STAYIN(IL)= (ONE - GOSOUT(IL))
               PRODUC= PRODUC * GOSOUT(IL)
               DPR(NOIL,NOIL)= DPR(NOIL,NOIL)
     >                       + WEIGHT*(XSIL-STAYIN(IL))
            ENDIF
  240    CONTINUE
C        PIJ CALCULATION
         DSCBEG= WEIGHT*ZCOR
         DO 260 IL = 1, LINE-2*NCOR
            NOIL  = NRSEG(IL)
            DSCEND= WEIGHT*STAYIN(IL)
            DO 250 JL  = IL+1, LINE-2*NCOR
               DPR(NRSEG(JL),NOIL)= DPR(NRSEG(JL),NOIL)
     >                            + STAYIN(JL)*DSCEND
               DSCEND= DSCEND*GOSOUT(JL)
  250       CONTINUE
C           PIS CALCULATION
            DO 261 JL = 1, NCOR
               DPR(ISD(JL),NOIL)= DPR(ISD(JL),NOIL)+DSCBEG*STAYIN(IL)
               DPR(ISF(JL),NOIL)= DPR(ISF(JL),NOIL)+DSCEND*ZCOR
  261       CONTINUE
            DSCBEG= DSCBEG*GOSOUT(IL)
  260    CONTINUE
C        PSS CALCULATION
         DO 265 IL = 1, NCOR
         DO 265 JL = 1, NCOR
            DPR(ISD(IL),ISF(JL))= DPR(ISD(IL),ISF(JL)) + PRODUC
  265    CONTINUE
  230 CONTINUE
      ELSE
C
C2.2) NO VOID REGION
      DO 330 ITRAK=1,NTRK
         READ(IFTRAK)  IANG,LINE,WEIGHT,
     >     (ISD(JL),JL=1,NCOR),( NRSEG(IL),IL= 1,LINE-2*NCOR),
     >     (ISF(JL),JL=1,NCOR),
     >     (SSD1   ,JL=1,NCOR),(SEGLEN(IL),IL= 1,LINE-2*NCOR),
     >     (SSF1   ,JL=1,NCOR)
         PRODUC= WEIGHT*ZCOR2
C        PII CALCULATION AND ESCAPE
         DO 340 IL = 1,LINE-2*NCOR
            NOIL  = NRSEG(IL)
            XSIL  = SIGTAL(NOIL)*SEGLEN(IL)
            IF(XSIL .LT. CUTEXP) THEN
               XSIL2=XSIL*XSIL
               STAYIN(IL)=XSIL-XSIL2*(HALF-SIXT*XSIL)
               GOSOUT(IL)=ONE-STAYIN(IL)
               PRODUC= PRODUC * GOSOUT(IL)
               DPR(NOIL,NOIL)= DPR(NOIL,NOIL)
     >          + WEIGHT*XSIL2*(HALF-SIXT*XSIL)
            ELSE
               GOSOUT(IL)= EXP( - XSIL )
               STAYIN(IL)= (ONE - GOSOUT(IL))
               PRODUC= PRODUC * GOSOUT(IL)
               DPR(NOIL,NOIL)= DPR(NOIL,NOIL)
     >                       + WEIGHT*(XSIL-STAYIN(IL))
            ENDIF
  340    CONTINUE
C        PIJ CALCULATION
         DSCBEG= WEIGHT*ZCOR
         DO 360 IL = 1, LINE-2*NCOR
            NOIL  = NRSEG(IL)
            DSCEND= WEIGHT*STAYIN(IL)
            DO 350 JL  = IL+1, LINE-2*NCOR
               DPR(NRSEG(JL),NOIL)=DPR(NRSEG(JL),NOIL)
     >                            + STAYIN(JL)*DSCEND
               DSCEND= DSCEND*GOSOUT(JL)
  350       CONTINUE
C           PIS CALCULATION
            DO 361 JL = 1, NCOR
               DPR(ISD(JL),NOIL)= DPR(ISD(JL),NOIL)+DSCBEG*STAYIN(IL)
               DPR(ISF(JL),NOIL)= DPR(ISF(JL),NOIL)+DSCEND*ZCOR
  361       CONTINUE
            DSCBEG= DSCBEG * GOSOUT(IL)
  360    CONTINUE
C        PSS CALCULATION
         DO 365 IL = 1, NCOR
         DO 365 JL = 1, NCOR
            DPR(ISD(IL),ISF(JL))= DPR(ISD(IL),ISF(JL)) + PRODUC
  365    CONTINUE
  330 CONTINUE
      ENDIF
      ENDIF
C
C3)   RECONSTRUCT SYMMETRIC PROBABILITY MATRIX ------------------------
      IPR= 0
      DO 1101 IL    = -NSOUT, NREG
         DO 1103 JL = -NSOUT, IL
            IPR= IPR+1
            PROB(IPR)= DPR(IL,JL) + DPR(JL,IL)
 1103    CONTINUE
 1101 CONTINUE
C
      RETURN
      END
