*DECK XELDRV
      SUBROUTINE XELDRV(IPTRK ,IPGEOM,IPRT  ,MAXPTS,NANIS ,NORE  ,
     >                  KSPEC ,KTOPT ,TITREC,CUTOFX,CFTRAK,IFTRAK,
     >                  IDISP ,ISYMM ,LCACT ,NMU   ,
     >                  MATCOD,VOLUME,KEYFLX)
************************************************************************
*                                                                      *
*           NAME: XELDRV                                               *
*      COMPONENT: EXCELL                                               *
*          LEVEL: 1 (CALLED BY MULTI-GROUP CODE 'DRAGON')              *
*        VERSION: 1.0                                                  *
*       CREATION: 91/07                                                *
*       MODIFIED: 00/03 (R.R.) DECLARE ALL VARIABLE TYPES              *
*         AUTHOR: ROBERT ROY                                           *
*                                                                      *
*     SUBROUTINE: THIS ROUTINE IS USED TO:                             *
*                 1) GET A DRAGON TRACKING FILE TO COMPUTE "PIJ".      *
*                 2) NORMALIZE A TRACKING FILE TO DRAGON FORMAT.       *
*                 3) PRODUCE A NEW TRACKING FILE IN DRAGON FORMAT.     *
*                                                                      *
*          ENTRY: EXCELR ( TO RELEASE SPACE OF EXCELL MODULES).        *
*                                                                      *
*--------+-------------- V A R I A B L E S -------------+--+-----------*
*  NAME  /                  DESCRIPTION                 /IO/MOD(DIMENS)*
*--------+----------------------------------------------+--+-----------*
* IPTRK  / POINTER TO THE EXCELL TRACKING (L_TRACK)     /IO/INT        *
* IPGEOM / POINTER TO THE GEOMETRY (L_GEOM)             /I./INT        *
* IPRT   / TRACKING PRINT LEVEL                         /I./I          *
* MAXPTS / # OF ZONES ACCORDING TO USER                 /I./INT        *
* NANIS  / ANISOTROPY OF THE SOLUTION                   /I./I          *
* NORE   / TRACK NORMALIZATION (0:YES 1:NO)             /I./I          *
* KSPEC  / KIND OF PIJ INTEGRATION (0:ISOTR.,1:SPEC.)   /I./INT        *
* KTOPT  / TRACKING TYPE OPTION                         /IO/INT        *
* TITREC / TITLE FOR THIS CASE                          /I./C*72       *
* CUTOFX / MFP CUTOFF FOR SPECULAR INTEGRATION          /I./REL        *
* CFTRAK / NAME OF THE SEQUENTIAL BINARY TRACKING FILE  /I./C*12       *
* IFTRAK / UNIT OF THE SEQUENTIAL BINARY TRACKING FILE  /I./INT        *
* IDISP  / MODE OF THE SEQUENTIAL BINARY TRACKING FILE  /I./INT        *
* ISYMM  / SYMMETRY FACTOR                              /I./INT        *
* LCACT   type of polar integration for the method of characteristics. *
* NMU     number of polar angles for the method of characteristics.    *
* MATCOD / MATERIAL #S FOR ZONES IN THE SUPERCELL       /.O/INT(MAXPTS)*
* VOLUME / VOLUMES OF ZONES IN THE SUPERCELL            /.O/REL(MAXPTS)*
* KEYFLX / ZONE KEY FOR THE UNKNOWN VECTORS (FLUXES...) /.O/REL(MAXPTS)*
*--------+---------------- R O U T I N E S -------------+--+-----------*
*  NAME  /                  DESCRIPTION                                *
*--------+-------------------------------------------------------------*
* LELCHK / TO CHECK CONSISTENCY BETWEEN TRACKING FILE AND GEOMETRY.    *
* XELCOP / TO COPY A TRACKING FILE(VBS FORMAT).                        *
* XELCMP / TO REDECLARE VARIABLES FOR MERGED ZONES.                    *
* XCWTRK / TO TREAT ANY CLUSTER CELL            (GEOMETRY/TRACKING).   *
* XELTRK / TO TREAT A CARTESIAN ASSEMBLY OF CELL(GEOMETRY/TRACKING).   *
* XHXTRK / TO TREAT A HEXAGONAL ASSEMBLY OF CELL(GEOMETRY/TRACKING).   *
* XELNTR / TO PRODUCE FINAL TRACKING FILE(RENORMALIZED LENGHTS OR NOT).*
************************************************************************
C
      IMPLICIT        NONE
C
      INTEGER         IPTRK ,IPGEOM,IPRT  ,MAXPTS,NANIS ,NORE  ,
     >                KSPEC ,KTOPT ,IFTRAK,IDISP,ISYMM,
     >                LCACT,NMU,MATCOD(MAXPTS), KEYFLX(MAXPTS)
      REAL            VOLUME(MAXPTS)
C
      INTEGER         NSTATE
      PARAMETER      (NSTATE=20)
      INTEGER         NREG,NUNKNO,IUTYPE,ISTATE(NSTATE),
     >                IFTEMP,IFILE,ITYPE,ITOPT,ITROP,
     >                NCOMNT,NTRK,IREC,IC,IR,JR,IUNK,NSOUT,IDISPO,
     >                NDIM,NV,NS,NALBGO,NCOR,NANGL,MXSEG,
     >                NDIMO,NSO,NVO,NUNOLD,VOLOLD,MATOLD,KDROPN,KDRCLS,
     >                NUNKNX,VOLMRG,MATMRG,SEGLEN,NRSEG,VOLTRK,IOUT,
     >                ITGEO,VOLSUR,MATALB,KEYMRG,ANGLES,DENSTY,IUSED(6),
     >                ICMAX,ICODE(6),ICOLD(6),NANGLO,MXSEGO,I
      LOGICAL         LEAKSW, LELCHK, SWNOGE, SWCONS, LCM
      REAL            CUTOFX,ASCRP,ALBEDO(6),ALBOLD(6),EXTKOP(3),
     >                ZERO,ONE,DENS,PCORN
      CHARACTER       GEONAM*12,TITREC*72,CORIEN(0:3,6)*4,CUSED(0:1)*6,
     >                TEXT12*12,NAMT*12,CFTRAK*12,COMENT*80,CTRK*4
C
      PARAMETER     ( IUTYPE=2, IOUT=6, ZERO=0.0, ONE=1.0 )
C
      SAVE   /CEXGEO/
      COMMON /CEXGEO/ KEYMRG, VOLSUR, MATALB
C
C     CEXGEO: COMMON BLOCK WITH GEOMETRIC POINTERS
C             ALLOCATED IN TRACKING ROUTINES: XCWTRK,XHXTRK,XELTRK.
C
C             KEYMRG: POINTER FOR INTEGER MERGE VECTOR IN ID
C             VOLSUR: POINTER FOR REAL VOLUME-SURFACE VECTOR IN RD
C             MATALB: POINTER FOR INTEGER MATERIAL-ALBEDO VECTOR IN ID
C
      REAL            RD
      COMMON          RD(1)
      INTEGER         ID(1)
      EQUIVALENCE    (RD(1),ID(1))
C
      DATA          (( CORIEN(JR,IR),IR=1,6),JR=0,3)
     >              / ' 1  ',' 2  ',' 3  ',' 4  ',' 5  ',' 6  ',
     >                '****',' R+ ','****','****',' Z- ',' Z+ ',
     >                'HBC ','****','****','****',' Z- ',' Z+ ',
     >                ' X- ',' X+ ',' Y- ',' Y+ ',' Z- ',' Z+ '/
      DATA           ( CUSED(JR),JR=0,1 ) / 'UNUSED','  USED' /
C
      IF( IPRT.GT.0 ) WRITE(IOUT,'(1X,A72//)') TITREC
      SWNOGE= IPGEOM.EQ.0
      IF(SWNOGE) THEN
         GEONAM=' '
      ELSE
         NAMT=' '
         CALL LCMNXT(IPGEOM,NAMT,GEONAM,TEXT12,LCM)
      ENDIF
      NTRK  = 0
C
      IF( IPRT.GT.0 )THEN
         WRITE(IOUT,'(1H )')
         IF( SWNOGE )THEN
            WRITE(IOUT,'(27H ECHO = >>>   NO GEOMETRY     )')
         ELSE
            WRITE(IOUT,'(27H ECHO = >>> GEOMETRY NAME: ,A8)') GEONAM
         ENDIF
         WRITE(IOUT,'(27H ECHO = >>> TRACKING FILE: ,A8)') CFTRAK
         IF(     IDISP.EQ.-1 )THEN
            WRITE(IOUT,'(27H ECHO = >>>          DISP: ,A4)') 'MODT'
         ELSEIF( IDISP.EQ. 0 )THEN
            WRITE(IOUT,'(27H ECHO = >>>          DISP: ,A4)') 'OLDT'
         ELSEIF( IDISP.EQ.+1 )THEN
            WRITE(IOUT,'(27H ECHO = >>>          DISP: ,A4)') 'NEWT'
         ENDIF
         IF(     NORE.EQ.-2 )THEN
            WRITE(IOUT,'(30H ECHO = >>>                   )')
         ELSEIF( NORE.EQ.-1 )THEN
            WRITE(IOUT,'(30H ECHO = >>> NORMALIZED-GLOBAL )')
         ELSEIF( NORE.EQ. 0 )THEN
            WRITE(IOUT,'(30H ECHO = >>> NORMALIZED-LOCAL  )')
         ELSEIF( NORE.EQ.+1 )THEN
            WRITE(IOUT,'(30H ECHO = >>> NOT NORMALIZED    )')
         ENDIF
      ENDIF
C
C.1)  REWIND TRACKING FILE ---------------------------------------------
C
      IF(IFTRAK.EQ.0) CALL XABORT('XELDRV: NO SEQUENTIAL BINARY TRACKIN'
     > //'G FILE YET DEFINED')
      REWIND IFTRAK
C
C.1.2) GET HEADER INFORMATIONS FROM *OLDT*/*MODT* FILES
      IF( IDISP.LE.0 )THEN
         IREC= 1
         READ(IFTRAK,ERR=997) CTRK,NCOMNT,NTRK
         DO 10 IC= 1, NCOMNT
            IREC= IREC+1
            READ (IFTRAK,ERR=997) COMENT
   10    CONTINUE
         IREC= IREC+1
         READ (IFTRAK,ERR=997) NDIMO,ITOPT,NVO,NSO,NALBGO,
     >                         NCOR,NANGL,MXSEG
         IF( NALBGO.LE.0.OR.NALBGO.GT.6 )THEN
            CALL XABORT('XELDRV: NALBG.GT.6.OR.NALBG.LE.0'//
     >                  ' ON TRACKING FILE')
         ENDIF
         NUNOLD= NVO+NSO+1
         CALL SETARA( RD,  NUNOLD, VOLOLD)
         CALL SETARA( ID,  NUNOLD, MATOLD)
         IREC= IREC+1
         READ (IFTRAK,ERR=997) (RD(VOLOLD+IR),IR=0,NUNOLD-1)
         IREC= IREC+1
         READ (IFTRAK,ERR=997) (ID(MATOLD+IR),IR=0,NUNOLD-1)
         IREC= IREC+1
         READ (IFTRAK,ERR=997) ( ICOLD(IR),IR=1,NALBGO)
         IREC= IREC+1
         READ (IFTRAK,ERR=997) (ALBOLD(IR),IR=1,NALBGO)
         IREC= IREC+1
         READ (IFTRAK,ERR=997) (ASCRP,IR=0,NDIMO*NANGL-1)
         IREC= IREC+1
         READ (IFTRAK,ERR=997) (ASCRP,IR=0,NANGL-1)
         REWIND IFTRAK
      ENDIF
C
C.1.3) OPEN TEMP TRACKING FILE FOR *MODT*/*NEWT* FILES
      IF( IDISP.NE.0 )THEN
         IFILE= KDROPN('DUMMYSQ',0,IUTYPE,0,0)
         IF( IFILE.LE.0 ) GO TO 998
         IFTEMP = IFILE
         REWIND IFTEMP
         IF( IDISP.LT.0 )THEN
C
C           FOR *MODT* FILES, MAIN TRACKING IS COPIED ON TEMPORARY
            CALL XELCOP( IFTRAK, IFTEMP )
         ENDIF
      ENDIF
C
C.2)  GET GEOMETRIC INFORMATIONS AND TRACK IF NECESSARY----------------
C
      IF( SWNOGE )THEN
C
C.2.1)  NO GEOMETRY, GET INFORMATIONS FROM *OLDT*/*MODT* FILE
         NDIM=  NDIMO
         NSOUT= NSO
         NREG=  NVO
         NS=    NSO
         NV=    NVO
         NUNKNX= NUNOLD
         CALL SETARA( RD,     NUNKNX, VOLSUR)
         CALL SETARA( ID,     NUNKNX, MATALB)
         CALL SETARA( ID,     NUNKNX, KEYMRG)
         IUNK=  0
         DO 20 IR= -NS, NV
            RD(VOLSUR+IUNK)= RD(VOLOLD+IUNK)
            ID(MATALB+IUNK)= ID(MATOLD+IUNK)
            ID(KEYMRG+IUNK)= IR
            IUNK= IUNK+1
   20    CONTINUE
         VOLMRG= VOLOLD
         MATMRG= MATOLD
         DO 25 IR= 1, NALBGO
             ICODE(IR)=  ICOLD(IR)
   25    CONTINUE
         ITGEO= 0
      ELSE
C
C.2.2)  THERE IS A GEOMETRY, GO TO EXCELL MODULES TO ANALYZE IT
         CALL XDISET(ISTATE,NSTATE,0)
         CALL LCMGET(IPGEOM,'STATE-VECTOR',ISTATE)
         ITYPE= ISTATE(1)
         IF( ITYPE.EQ.3.OR.ITYPE.EQ.6 )THEN
            ITGEO= 1
         ELSEIF( ITYPE.EQ. 8 .OR. ITYPE.EQ. 9 .OR.
     >           ITYPE.EQ.24 .OR. ITYPE.EQ.25 )THEN
            ITGEO= 2
         ELSEIF( ITYPE.EQ. 5 .OR. ITYPE.EQ. 7 .OR. ITYPE.EQ.20 .OR.
     >           ITYPE.EQ.21 .OR. ITYPE.EQ.22 .OR. ITYPE.EQ.23 )THEN
            ITGEO= 3
         ELSE
            ITGEO= 0
         ENDIF
         IDISPO= IDISP
         NANGLO= NANGL
         MXSEGO= MXSEG
         ITROP = 0
         IF( ISTATE(13).GE.1 )THEN
           IF( ITYPE.EQ.3.OR.ITYPE.EQ.20.OR.ITYPE.EQ.24 )THEN
C
C.2.2.1.1)     EXCELL DRIVER FOR CLUSTER SINGLE CELLS
C
             CALL XCWTRK(IPTRK ,IPGEOM,GEONAM,IDISP ,IFTEMP,MAXPTS,
     >                   IPRT  ,NDIM  ,ITOPT ,NV    ,NS    ,NANGL ,
     >                   ISYMM ,DENS  ,PCORN ,MXSEG ,ICODE ,TITREC,
     >                   RD    ,ID    )
             ITROP=3
           ELSE
             CALL XABORT('XELDRV: ONLY ONE-CELL TUBE/CARCEL/HEXCEL'//
     >                   ' CLUSTERS ARE AVAILABLE')
           ENDIF
         ELSEIF( ITGEO.EQ.2 )THEN
C
C.2.2.1.2)  EXCELL DRIVER FOR HEXAGONAL 2D/3D ASSEMBLIES
C
           CALL XHXTRK(IPTRK ,IPGEOM,GEONAM,IDISP,IFTEMP,MAXPTS,
     >                 IPRT  ,NDIM  ,ITOPT ,NV   ,NS    ,NANGL ,
     >                 ISYMM ,DENS  ,PCORN ,MXSEG,ICODE ,TITREC,
     >                 RD    ,ID    )
           ITROP=2
         ELSEIF( ITGEO.EQ.3 )THEN
C
C.2.2.1.3)  EXCELL DRIVER FOR CARTESIAN 2D/3D ASSEMBLIES
            CALL XELTRK(IPTRK ,IPGEOM,GEONAM,IDISP ,IFTEMP,MAXPTS,
     >                  IPRT  ,NDIM  ,ITOPT ,NV    ,NS    ,NANGL ,
     >                  ISYMM ,DENS  ,PCORN ,MXSEG ,ICODE ,TITREC,
     >                  RD    ,ID    )
            ITROP=1
C----
C  For case with intrinsic symmetry
C  tracking performed on unfolded geometry assuming angular and spatial
C  symmetry
C  Normalization must be on global volume since only this option
C  makes sense.
C----  
            IF(ISYMM .GT. 1) THEN
              IF(NORE .EQ. 0) NORE=-1
            ENDIF
            IF(NMU .EQ. 0 .AND. NDIM .EQ. 2 ) THEN
              IF(LCACT .EQ. 3) THEN 
                NMU=2
              ELSE
                NMU= (NANGL+1)/2
                IF(ITOPT .EQ. 0) THEN
                  IF(ISYMM .GT. 1) THEN
                    NMU=NANGL
                  ENDIF
                ENDIF
              ENDIF
            ENDIF
         ELSE
            CALL XABORT('XELDRV: INVALID TYPE OF GEOMETRY')
         ENDIF
         IF( IDISP.NE.IDISPO )THEN
           CALL XABORT('XELDRV: *HALT* OPTION REQUESTED '//
     >                 ' NO FURTHER CALCULATION IS POSSIBLE')
         ELSEIF( IDISP.GT.0 )THEN
           REWIND IFTEMP
         ELSE
           IF( NANGL.NE.NANGLO )THEN
             CALL XABORT('XELDRV: NOT POSSIBLE TO CHANGE '//
     >                   ' *NANGL* PARAMETER OF TRACKING FILE')
           ENDIF
           IF( MXSEG.NE.MXSEGO )THEN
             CALL XABORT('XELDRV: NOT POSSIBLE TO CHANGE '//
     >                   ' *MXSEG* PARAMETER OF TRACKING FILE')
           ENDIF
         ENDIF
C
C.2.2.2)  MERGE SURFACES AND ZONES FOR THIS GEOMETRY
         NUNKNX= NV+NS+1
         CALL SETARA( RD,     NUNKNX, VOLMRG)
         CALL SETARA( ID,     NUNKNX, MATMRG)
         CALL XELCMP( NS,   NV, RD(VOLSUR), ID(MATALB), ID(KEYMRG),
     >               NSOUT,  NREG, RD(VOLMRG), ID(MATMRG), ITGEO, ICODE)
C
C.2.2.3)  CHECK CONSISTENCY BETWEEN  *MODT* FILE
C                                AND GEOMETRY BEFORE MERGE
         SWCONS=.FALSE.
         IF( IDISP.LT.0 )THEN
            SWCONS= LELCHK(NSO  ,NVO ,RD(VOLOLD),ID(MATOLD),ICOLD,
     >                     NS   ,NV  ,RD(VOLSUR),ID(MATALB),ICODE, 0)
            IF( IPRT.GT.0 )THEN
               WRITE(IOUT,'(1H )')
               IF( SWCONS )THEN
                  WRITE(IOUT,'(70H ECHO = >>>   CONSISTENCY BETWEEN '//
     >                     'TRACKING FILE AND UNMERGED GEOMETRY    /)')
               ELSE
                  WRITE(IOUT,'(70H ECHO = >>> INCONSISTENCY BETWEEN '//
     >                     'TRACKING FILE AND UNMERGED GEOMETRY    /)')
               ENDIF
            ENDIF
         ENDIF
C
C.2.2.4)  CHECK CONSISTENCY BETWEEN *OLDT*/*MODT* FILE
C                                AND GEOMETRY AFTER MERGE
         IF( IDISP.EQ.0.OR.(IDISP.LT.0.AND.(.NOT.SWCONS)) )THEN
            IF( NDIMO.NE.NDIM )THEN
              CALL XABORT('XELDRV: DIMENSION (2-D/3-D) INCONSISTENCY')
            ENDIF
            SWCONS= LELCHK(NSO  ,NVO ,RD(VOLOLD),ID(MATOLD),ICOLD,
     >                     NSOUT,NREG,RD(VOLMRG),ID(MATMRG),ICODE,IPRT)
            IF( SWCONS )THEN
               IF( IPRT.GT.0 )THEN
                  WRITE(IOUT,'(1H )')
                  WRITE(IOUT,'(70H ECHO = >>>   CONSISTENCY BETWEEN '//
     >                     'TRACKING FILE AND   MERGED GEOMETRY    /)')
               ENDIF
            ELSE
               WRITE(IOUT,'(70H ECHO = >>> INCONSISTENCY BETWEEN '//
     >                  'TRACKING FILE AND   MERGED GEOMETRY    /)')
               CALL XABORT('XELDRV: INCONSISTENCY OF MERGED GEOMETRY '//
     >                    'WITH OLD TRACKING FILE ' )
            ENDIF
C
C           CONSISTENCY WITH MERGED GEOMETRY
C           COPY MERGED VOLUMES INTO VOLSUR ARRAY SINCE MERGE WAS DONE
            NS= NSOUT
            NV= NREG
            IUNK=  0
            DO 50 IR= -NS, NV
               RD(VOLSUR+IUNK)= RD(VOLOLD+IUNK)
               ID(MATALB+IUNK)= ID(MATOLD+IUNK)
               ID(KEYMRG+IUNK)= IR
               IUNK= IUNK+1
   50       CONTINUE
         ENDIF
      ENDIF
C
C.3)  NORMALIZE TEMPORARY FILE FOR *MODT*/*NEWT* FILES ----------------
C
      IF( IDISP.NE.0 )THEN
C
C.3.1)  WARNING IF THE FILE HAS *NTRK*.NE.0
         IF( NTRK.NE.0 )THEN
            WRITE(IOUT,'(1H )')
            WRITE(IOUT,'(60H ECHO = >>> WARNING: TRACKING FILE'//
     >                 ' MAY ALREADY BE NORMALIZED             /)')
         ENDIF
         CALL SETARA( RD, 3*NANGL, ANGLES)
         CALL SETARA( RD,   NANGL, DENSTY)
         CALL SETARA( RD,   MXSEG, SEGLEN)
         CALL SETARA( ID,   MXSEG,  NRSEG)
         CALL SETARA( RD, 2*(NANGL+1)*NUNKNX, VOLTRK)
         CALL XELNTR( NDIM, IFTEMP, IFTRAK, NORE, IPRT,
     >                NS, NV, RD(VOLSUR), ID(MATALB), ID(KEYMRG),
     >                NSOUT, NREG, RD(VOLMRG), ID(MATMRG), CUTOFX,
     >                ITGEO, ICODE, ALBOLD, NANGL, MXSEG,
     >                RD(VOLTRK), RD(ANGLES), RD(DENSTY),
     >                RD(SEGLEN), ID( NRSEG) )
         CALL RLSARA( RD( VOLTRK ) )
         CALL RLSARA( ID(  NRSEG ) )
         CALL RLSARA( RD( SEGLEN ) )
         CALL RLSARA( RD( DENSTY ) )
         CALL RLSARA( RD( ANGLES ) )
         IFILE= IFTEMP
         IFTEMP= KDRCLS(IFTEMP,2)
         IF( IFTEMP.LT.0 ) GO TO 999
      ENDIF
C
C.5)  CHARGE GEOMETRIC ALBEDOS & GET PHYSICAL ALBEDOS IF NECESSARY
      ICMAX= 0
      DO 60 IR= 1, 6
        ALBEDO(IR)= ONE
        IUSED(IR)= 0
        ICMAX= MAX(ICMAX,ICODE(IR))
        IF( ICODE(IR).LT.0 ) ALBEDO(IR)= ALBOLD(-ICODE(IR))
   60 CONTINUE
      IF( ICMAX.GT.0 )THEN
         CALL XABORT('XELDRV: PHYSICAL ALBEDOS NOT IMPLEMENTED')
      ENDIF
      IF( KTOPT .EQ. -1) THEN
        KTOPT= ITOPT
      ENDIF
      IF(KSPEC .EQ. -1 ) THEN
        KSPEC= KTOPT
      ELSE
        KSPEC=MIN(KSPEC,KTOPT)
      ENDIF
C
C.5)  STOCK INFORMATION (OUTPUT TO DRAGON DRIVER) --------------------
      IF(NANIS.EQ.1) THEN
        NUNKNO= NREG
      ELSE IF(NANIS.EQ.2) THEN
        NUNKNO=4*NREG
      ENDIF
      IF( NREG.GT.MAXPTS ) THEN
        WRITE(IOUT,'(28H ****** XELDRV ERROR  ******,/
     >   28H NUMBER OF REGION COMPUTED =,I10/
     >   28H MAXIMUM NUMBER OF REGION  =,I10)') NREG,MAXPTS
        CALL XABORT('XELDRV: MAXR TOO SMALL')
      ENDIF
      DO 70 IR= 1, NREG
         KEYFLX(IR)= IR
         VOLUME(IR)= RD( VOLMRG+IR+NSOUT )
         MATCOD(IR)= ID( MATMRG+IR+NSOUT )
   70 CONTINUE
C
C     COMPUTE LEAKAGE SWITCH
      LEAKSW=.TRUE.
      DO 80 IR= -NSOUT, -1
         IUSED(-ID(MATMRG+IR+NSOUT))= 1
         LEAKSW= LEAKSW .AND. ALBEDO(-ID(MATMRG+IR+NSOUT)).EQ.ONE
   80 CONTINUE
      LEAKSW=.NOT.LEAKSW
      CALL RLSARA( ID( MATMRG ) )
      CALL RLSARA( RD( VOLMRG ) )
      IF( (IDISP.LE.0).AND.(.NOT.SWNOGE) )THEN
         CALL RLSARA( ID( MATOLD ) )
         CALL RLSARA( RD( VOLOLD ) )
      ENDIF
C
      IF( IPRT.GT.0 )THEN
         IF( IPRT.GT.1 )THEN
            WRITE(IOUT,'(8H  SIDE  ,2X,6(7X,A4))')
     >                                  (CORIEN(ITGEO,IR),IR=1,6)
            WRITE(IOUT,'(8H  GEOM #,2X,6(7X,I4.0))')
     >                                  (MAX(0,-ICODE(IR)),IR=1,6)
            WRITE(IOUT,'(8H  PHYS #,2X,6(7X,I4.0))')
     >                                  (MAX(0,ICODE(IR)),IR=1,6)
            WRITE(IOUT,'(8H  ALBEDO,2X,1P,6E11.4)')
     >                                  (ALBEDO(IR),IR=1,6)
            WRITE(IOUT,'(8H        ,2X,6(5X,A6))')
     >                                  (CUSED(IUSED(IR)),IR=1,6)
            WRITE(IOUT,'(1H )')
         ENDIF
         WRITE(IOUT,'(1H )')
         IF( KSPEC.EQ.0 )THEN
          WRITE(IOUT,'(40H ECHO = >>> ISOTROPIC CP CALCULATION       )')
         ELSEIF( KSPEC.EQ.1 )THEN
          WRITE(IOUT,'(40H ECHO = >>>  SPECULAR CP CALCULATION       )')
          IF( CUTOFX.EQ.ZERO )THEN
             WRITE(IOUT,'(27H ECHO = >>>    NO CUT-OFF     )')
          ELSE
             WRITE(IOUT,'(27H ECHO = >>> MFP.  CUT-OFF: ,1P,E10.4 )')
     >       CUTOFX
          ENDIF
         ENDIF
         WRITE(IOUT,'(28H ECHO = >>> NB. OF REGIONS: ,I5)') NREG
      ENDIF
C
C.5.2) RELEASE SPACE ACCORDING TO INVERSE ORDER OF ALLOCATIONS
      CALL RLSARA( ID(KEYMRG) )
      CALL RLSARA( ID(MATALB) )
      CALL RLSARA( RD(VOLSUR) )
C
C     SAVE GENERAL TRACKING INFORMATION.
      CALL XDISET(ISTATE,NSTATE,0)
      ISTATE(1)=NREG
      ISTATE(2)=NUNKNO
      IF(LEAKSW) THEN
         ISTATE(3)=0
      ELSE
         ISTATE(3)=1
      ENDIF
      IR=0
      DO 100 I=1,NREG
  100 IR=MAX(IR,MATCOD(I))
      ISTATE(4)=IR
      ISTATE(5)=NSOUT
      ISTATE(6)=NANIS
      ISTATE(7)=ITROP
      ISTATE(8)=NORE
      ISTATE(9)=KTOPT
      ISTATE(10)=KSPEC
      ISTATE(11)=NANGL
      ISTATE(12)=ISYMM
      ISTATE(13)=LCACT
      ISTATE(14)=NMU
      CALL LCMPUT(IPTRK,'STATE-VECTOR',NSTATE,1,ISTATE)
C
C     SAVE EXCELL SPECIFIC TRACKING INFORMATION.
      EXTKOP(1)=CUTOFX
      EXTKOP(2)=DENS
      EXTKOP(3)=PCORN
      CALL LCMPUT(IPTRK,'EXCELTRACKOP',3,2,EXTKOP)
      CALL LCMPUT(IPTRK,'MATCOD',NREG,1,MATCOD)
      CALL LCMPUT(IPTRK,'VOLUME',NREG,2,VOLUME)
      CALL LCMPUT(IPTRK,'KEYFLX',NREG,1,KEYFLX)
      CALL LCMPUT(IPTRK,'ALBEDO',6,2,ALBEDO)
      IF(IPRT.GE.1) THEN
        WRITE(IOUT,6000) IPRT,12,(ISTATE(IR),IR=1,14)
        WRITE(IOUT,6001)(EXTKOP(IR),IR=1,3)
      ENDIF
      RETURN
C
 6000 FORMAT(' EXCELT PRINT LEVEL =',I8/
     > ' TRACK STATE-VECTOR  (',I2,') ELEMENTS'/
     > ' -----------------'/
     > ' NREG   =',I8,' (NUMBER OF REGIONS)'/
     > ' NUNKNO =',I8,' (NUMBER OF UNKNOWNS IN SYSTEM)'/
     > ' ILEAK  =',I8,' (LEAKAGE FLAG)'/
     > ' MAXMIX =',I8,' (NUMBER OF MIXTURES)'/
     > ' NSOUT  =',I8,' (NUMBER OF OUTER SURFACES)'/
     > ' NANIS  =',I8,' (FLUX ANISOTROPY ORDER)'/
     > ' ITROP  =',I8,' (GEOMETRY TYPE)'/
     > ' NORE   =',I8,' (TRACK NORMALIZATION OPTION)'/
     > ' KTOPT  =',I8,' (TYPE OF TRACKING)'/
     > ' KSEPC  =',I8,' (TYPE OF BOUNDARY CONDITION)'/
     > ' NANGL  =',I8,' (NUMBER OF TRACKING ANGLES)'/
     > ' ISYMM  =',I8,' (TRACKING SYMMETRY FACTOR)'/
     > ' LCACT  =',I8,' (Polar quadrature option)'/
     > ' NMU    =',I8,' (Number of polar quadrature points)'/
     > ' -----------------'/)
 6001 FORMAT(
     > ' EXCELL TRACK OPTIONS '/
     > ' CUTOFX =',F20.8,' (CUTOFF FOR TRACK LENGTH)'/
     > ' DENS   =',F20.8,' (TRACK DENSITY)'/
     > ' PCORN  =',F20.8,' (CORNER DUPLICATION DISTANCE)'/
     > ' -----------------'/)
C
  997 WRITE(IOUT,'(31H ERROR= RECORD DESTROYED...        )')
      WRITE(IOUT,'(31H ERROR= UNABLE TO READ  RECORD ,I10)') IREC
      WRITE(IOUT,'(31H ERROR=              ON FILE FT,I2.2)') IFILE
      CALL XABORT('XELDRV: READ  TRACKING FILE FAILED' )
  998 WRITE(IOUT,'(31H ECHO = UNABLE TO OPEN  FILE FT,I4)') IFILE
      CALL XABORT('XELDRV: OPEN  FAILED')
  999 WRITE(IOUT,'(31H ECHO = UNABLE TO CLOSE FILE FT,I4)') IFILE
      CALL XABORT('XELDRV: CLOSE FAILED')
C
      END
